#include <iostream>
#include <stdint.h>
#include <picojson_misc.h>
#include "command_aec.h"
#include <dsp_lib/substitute/RaspberryPi_app.h>
#include <dsp_lib/include/dsp/aec.h>
#include <dsp_lib/include/misc/smoothing.h>
#include <spi/spi_api.h>
#include <control/misc.h>

using namespace std;
using namespace dsp_lib;

namespace control {

bool
CommandAec(
	picojson::object				&command_obj_,
	std::string						access_,
	dsp_lib::DspManager::NetList	netlist_
)
{
	//uint32_t config_base	= reinterpret_cast<uint32_t>(netlist_.config);
	uint32_t param_base		= reinterpret_cast<uint32_t>(netlist_.param);
	uint32_t status_base	= reinterpret_cast<uint32_t>(netlist_.status);

	//Aec::Config*	config_offset	= 0x00000000;
	Aec::Param*	param_offset = 0x00000000;
	Aec::Status*	status_offset	= 0x00000000;

	if (access_ == "set") {
		// paramがないとエラー
		picojson::object param_obj;
		if (get_object_member(command_obj_, "param", param_obj) == false) {
			cout << "\"param\" not found!" << endl;
			return false;
		}

		if (param_obj.count("on") == 1) {
			if (param_obj["on"].is<bool>() == false) {
				cout << "param.on must be bool!" << endl;
				return false;
			}
			bool on = param_obj["on"].get<bool>();
			uint32_t on_uint32;
			if (on == true) {
				on_uint32 = 1;
			} else {
				on_uint32 = 0;
			}

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->cear_aec), on_uint32) == false) {
				return false;
			}
		}

		if (param_obj.count("param_get_flag") == 1) {
			if (param_obj["param_get_flag"].is<double>() == false) {
				cout << "param.param_get_flag must be float!" << endl;
				return false;
			}
			float param_get_flag = static_cast<float>(param_obj["param_get_flag"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->param_get_flag), param_get_flag) == false) {
				return false;
			}
		}

		if (param_obj.count("param_set_flag") == 1) {
			if (param_obj["param_set_flag"].is<double>() == false) {
				cout << "param.param_set_flag must be float!" << endl;
				return false;
			}
			float param_set_flag = static_cast<float>(param_obj["param_set_flag"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->param_set_flag), param_set_flag) == false) {
				return false;
			}
		}

		if (param_obj.count("mu_") == 1) {
			int mu_size = 0;
			picojson::array& ary = param_obj["mu_"].get<picojson::array>();
			for (const auto& e : ary) {
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->mu_[mu_size]), static_cast<float>(e.get<double>())) == false) {
					return false;
				}
				mu_size++;
			}
		}

		if (param_obj.count("errmu_limit") == 1) {
			if (param_obj["errmu_limit"].is<double>() == false) {
				cout << "param.errmu_limit must be float!" << endl;
				return false;
			}
			float errmu_limit = static_cast<float>(param_obj["errmu_limit"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->errmu_limit), errmu_limit) == false) {
				return false;
			}
		}

		if (param_obj.count("rcv_dly_") == 1) {
			if (param_obj["rcv_dly_"].is<double>() == false) {
				cout << "param.rcv_dly_ must be float!" << endl;
				return false;
			}
			float rcv_dly_ = static_cast<float>(param_obj["rcv_dly_"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->rcv_dly_), rcv_dly_) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_ratio0") == 1) {
			if (param_obj["nlp_ratio0"].is<double>() == false) {
				cout << "param.nlp_ratio0 must be float!" << endl;
				return false;
			}
			float nlp_ratio0 = static_cast<float>(param_obj["nlp_ratio0"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_ratio0), nlp_ratio0) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_ratio1") == 1) {
			if (param_obj["nlp_ratio1"].is<double>() == false) {
				cout << "param.nlp_ratio1 must be float!" << endl;
				return false;
			}
			float nlp_ratio1 = static_cast<float>(param_obj["nlp_ratio1"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_ratio1), nlp_ratio1) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_enhance0") == 1) {
			if (param_obj["nlp_enhance0"].is<double>() == false) {
				cout << "param.nlp_enhance0 must be float!" << endl;
				return false;
			}
			float nlp_enhance0 = static_cast<float>(param_obj["nlp_enhance0"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_enhance0), nlp_enhance0) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_enhance1") == 1) {
			if (param_obj["nlp_enhance1"].is<double>() == false) {
				cout << "param.nlp_enhance1 must be float!" << endl;
				return false;
			}
			float nlp_enhance1 = static_cast<float>(param_obj["nlp_enhance1"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_enhance1), nlp_enhance1) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_alpha0") == 1) {
			if (param_obj["nlp_alpha0"].is<double>() == false) {
				cout << "param.nlp_alpha0 must be float!" << endl;
				return false;
			}
			float nlp_alpha0 = static_cast<float>(param_obj["nlp_alpha0"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_alpha0), nlp_alpha0) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_alpha1") == 1) {
			if (param_obj["nlp_alpha1"].is<double>() == false) {
				cout << "param.nlp_alpha1 must be float!" << endl;
				return false;
			}
			float nlp_alpha1 = static_cast<float>(param_obj["nlp_alpha1"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_alpha1), nlp_alpha1) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_alpha2") == 1) {
			if (param_obj["nlp_alpha2"].is<double>() == false) {
				cout << "param.nlp_alpha2 must be float!" << endl;
				return false;
			}
			float nlp_alpha2 = static_cast<float>(param_obj["nlp_alpha2"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_alpha2), nlp_alpha2) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_step") == 1) {
			if (param_obj["nlp_step"].is<double>() == false) {
				cout << "param.nlp_step must be float!" << endl;
				return false;
			}
			float nlp_step = static_cast<float>(param_obj["nlp_step"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_step), nlp_step) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_compTh") == 1) {
			if (param_obj["drc_compTh"].is<double>() == false) {
				cout << "param.drc_compTh must be float!" << endl;
				return false;
			}
			float drc_compTh = static_cast<float>(param_obj["drc_compTh"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_compTh), drc_compTh) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_compRatio") == 1) {
			if (param_obj["drc_compRatio"].is<double>() == false) {
				cout << "param.drc_compRatio must be float!" << endl;
				return false;
			}
			float drc_compRatio = static_cast<float>(param_obj["drc_compRatio"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_compRatio), drc_compRatio) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_compGain") == 1) {
			if (param_obj["drc_compGain"].is<double>() == false) {
				cout << "param.drc_compGain must be float!" << endl;
				return false;
			}
			float drc_compGain = static_cast<float>(param_obj["drc_compGain"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_compGain), drc_compGain) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_boostTh") == 1) {
			if (param_obj["drc_boostTh"].is<double>() == false) {
				cout << "param.drc_boostTh must be float!" << endl;
				return false;
			}
			float drc_boostTh = static_cast<float>(param_obj["drc_boostTh"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_boostTh), drc_boostTh) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_boostRatio") == 1) {
			if (param_obj["drc_boostRatio"].is<double>() == false) {
				cout << "param.drc_boostRatio must be float!" << endl;
				return false;
			}
			float drc_boostRatio = static_cast<float>(param_obj["drc_boostRatio"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_boostRatio), drc_boostRatio) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_boostGain") == 1) {
			if (param_obj["drc_boostGain"].is<double>() == false) {
				cout << "param.drc_boostGain must be float!" << endl;
				return false;
			}
			float drc_boostGain = static_cast<float>(param_obj["drc_boostGain"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_boostGain), drc_boostGain) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_AT") == 1) {
			if (param_obj["drc_AT"].is<double>() == false) {
				cout << "param.drc_AT must be float!" << endl;
				return false;
			}
			float drc_AT = static_cast<float>(param_obj["drc_AT"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_AT), drc_AT) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_RT") == 1) {
			if (param_obj["drc_RT"].is<double>() == false) {
				cout << "param.drc_RT must be float!" << endl;
				return false;
			}
			float drc_RT = static_cast<float>(param_obj["drc_RT"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_RT), drc_RT) == false) {
				return false;
			}
		}

		if (param_obj.count("mic_mute") == 1) {
			if (param_obj["mic_mute"].is<bool>() == false) {
				cout << "param.mic_mute must be bool!" << endl;
				return false;
			}
			bool on = param_obj["mic_mute"].get<bool>();
			uint32_t on_uint32;
			if (on == true) {
				on_uint32 = 1;
			}
			else {
				on_uint32 = 0;
			}

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->mic_mute), on_uint32) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_gain") == 1) {
			if (param_obj["drc_gain"].is<double>() == false) {
				cout << "param.drc_gain must be float!" << endl;
				return false;
			}
			float drc_gain = static_cast<float>(param_obj["drc_gain"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_gain), drc_gain) == false) {
				return false;
			}
		}

		if (param_obj.count("cng_noise") == 1) {
			if (param_obj["cng_noise"].is<double>() == false) {
				cout << "param.cng_noise must be float!" << endl;
				return false;
			}
			float cng_noise = static_cast<float>(param_obj["cng_noise"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->cng_noise), cng_noise) == false) {
				return false;
			}
		}

		if (param_obj.count("bmInput_fixedGain") == 1) {
			if (param_obj["bmInput_fixedGain"].is<double>() == false) {
				cout << "param.bmInput_fixedGain must be float!" << endl;
				return false;
			}
			float bmInput_fixedGain = static_cast<float>(param_obj["bmInput_fixedGain"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->bmInput_fixedGain), bmInput_fixedGain) == false) {
				return false;
			}
		}

		if (param_obj.count("nlp_fixedGain") == 1) {
			if (param_obj["nlp_fixedGain"].is<double>() == false) {
				cout << "param.nlp_fixedGain must be float!" << endl;
				return false;
			}
			float nlp_fixedGain = static_cast<float>(param_obj["nlp_fixedGain"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->nlp_fixedGain), nlp_fixedGain) == false) {
				return false;
			}
		}

		if (param_obj.count("drc_fixedGain") == 1) {
			if (param_obj["drc_fixedGain"].is<double>() == false) {
				cout << "param.drc_fixedGain must be float!" << endl;
				return false;
			}
			float drc_fixedGain = static_cast<float>(param_obj["drc_fixedGain"].get<double>());
			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->drc_fixedGain), drc_fixedGain) == false) {
				return false;
			}
		}

	} else if (access_ == "get") {
		// paramがなくてもOK
		if (command_obj_.count("param") == 1) {
			if (command_obj_["param"].is<picojson::object>() == false) {
				cout << "\"param\" must be object!" << endl;
			}
			picojson::object param_obj = command_obj_["param"].get<picojson::object>();

			if (param_obj.count("on") == 1) {
				bool	 on;
				uint32_t on_uint32;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->cear_aec), &on_uint32) == false) {
					return false;
				}
				if (on_uint32 == 0) {
					on = false;
				} else {
					on = true;
				}
				set_object_member(param_obj, "on", on);
			}

			if (param_obj.count("param_get_flag") == 1) {
				float param_get_flag;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->param_get_flag), &param_get_flag) == false) {
					return false;
				}
				set_object_member(param_obj, "param_get_flag", param_get_flag);
			}

			if (param_obj.count("param_set_flag") == 1) {
				float param_set_flag;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->param_set_flag), &param_set_flag) == false) {
					return false;
				}
				set_object_member(param_obj, "param_set_flag", param_set_flag);
			}

			if (param_obj.count("mu_") == 1) {
				uint32_t MAX_mu_SIZE = 8;
				float mu_[MAX_mu_SIZE];

				for (uint32_t i = 0; i < MAX_mu_SIZE; i++) {
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->mu_[i]), &mu_[i]) == false) {
						return false;
					}
				}

				picojson::array mu_array(MAX_mu_SIZE);
				for (uint32_t mu_no = 0; mu_no < MAX_mu_SIZE; mu_no++) {
					mu_array[mu_no] = static_cast<picojson::value>(mu_[mu_no]);
				}

				set_object_member(param_obj, "mu_", mu_array);

			}

			if (param_obj.count("errmu_limit") == 1) {
				float errmu_limit;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->errmu_limit), &errmu_limit) == false) {
					return false;
				}
				set_object_member(param_obj, "errmu_limit", errmu_limit);
			}

			if (param_obj.count("rcv_dly_") == 1) {
				float rcv_dly_;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->rcv_dly_), &rcv_dly_) == false) {
					return false;
				}
				set_object_member(param_obj, "rcv_dly_", rcv_dly_);
			}

			if (param_obj.count("nlp_ratio0") == 1) {
				float nlp_ratio0;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_ratio0), &nlp_ratio0) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_ratio0", nlp_ratio0);
			}

			if (param_obj.count("nlp_ratio1") == 1) {
				float nlp_ratio1;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_ratio1), &nlp_ratio1) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_ratio1", nlp_ratio1);
			}

			if (param_obj.count("nlp_enhance0") == 1) {
				float nlp_enhance0;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_enhance0), &nlp_enhance0) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_enhance0", nlp_enhance0);
			}

			if (param_obj.count("nlp_enhance1") == 1) {
				float nlp_enhance1;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_enhance1), &nlp_enhance1) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_enhance1", nlp_enhance1);
			}

			if (param_obj.count("nlp_alpha0") == 1) {
				float nlp_alpha0;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_alpha0), &nlp_alpha0) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_alpha0", nlp_alpha0);
			}

			if (param_obj.count("nlp_alpha1") == 1) {
				float nlp_alpha1;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_alpha1), &nlp_alpha1) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_alpha1", nlp_alpha1);
			}

			if (param_obj.count("nlp_alpha2") == 1) {
				float nlp_alpha2;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_alpha2), &nlp_alpha2) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_alpha2", nlp_alpha2);
			}

			if (param_obj.count("nlp_step") == 1) {
				float nlp_step;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_step), &nlp_step) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_step", nlp_step);
			}

			if (param_obj.count("drc_compTh") == 1) {
				float drc_compTh;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_compTh), &drc_compTh) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_compTh", drc_compTh);
			}

			if (param_obj.count("drc_compRatio") == 1) {
				float drc_compRatio;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_compRatio), &drc_compRatio) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_compRatio", drc_compRatio);
			}

			if (param_obj.count("drc_compGain") == 1) {
				float drc_compGain;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_compGain), &drc_compGain) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_compGain", drc_compGain);
			}

			if (param_obj.count("drc_boostTh") == 1) {
				float drc_boostTh;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_boostTh), &drc_boostTh) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_boostTh", drc_boostTh);
			}

			if (param_obj.count("drc_boostRatio") == 1) {
				float drc_boostRatio;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_boostRatio), &drc_boostRatio) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_boostRatio", drc_boostRatio);
			}

			if (param_obj.count("drc_boostGain") == 1) {
				float drc_boostGain;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_boostGain), &drc_boostGain) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_boostGain", drc_boostGain);
			}

			if (param_obj.count("drc_AT") == 1) {
				float drc_AT;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_AT), &drc_AT) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_AT", drc_AT);
			}

			if (param_obj.count("drc_RT") == 1) {
				float drc_RT;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_RT), &drc_RT) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_RT", drc_RT);
			}

			if (param_obj.count("microphonearray") == 1) {
				uint32_t microphonearray;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->microphonearray), &microphonearray) == false) {
					return false;
				}
				set_object_member(param_obj, "microphonearray", microphonearray);
			}

			if (param_obj.count("mic_mute") == 1) {
				bool	 mic_mute;
				uint32_t mic_mute_uint32;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->mic_mute), &mic_mute_uint32) == false) {
					return false;
				}
				if (mic_mute_uint32 == 0) {
					mic_mute = false;
				}
				else {
					mic_mute = true;
				}
				set_object_member(param_obj, "mic_mute", mic_mute);
			}

			if (param_obj.count("drc_gain") == 1) {
				cout << "GET param.drc_gain!" << endl;
				float drc_gain;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_gain), &drc_gain) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_gain", drc_gain);
			}

			if (param_obj.count("cng_noise") == 1) {
				float cng_noise;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->cng_noise), &cng_noise) == false) {
					return false;
				}
				set_object_member(param_obj, "cng_noise", cng_noise);
			}

			if (param_obj.count("bmInput_fixedGain") == 1) {
				float bmInput_fixedGain;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->bmInput_fixedGain), &bmInput_fixedGain) == false) {
					return false;
				}
				set_object_member(param_obj, "bmInput_fixedGain", bmInput_fixedGain);
			}

			if (param_obj.count("nlp_fixedGain") == 1) {
				float nlp_fixedGain;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->nlp_fixedGain), &nlp_fixedGain) == false) {
					return false;
				}
				set_object_member(param_obj, "nlp_fixedGain", nlp_fixedGain);
			}

			if (param_obj.count("drc_fixedGain") == 1) {
				float drc_fixedGain;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->drc_fixedGain), &drc_fixedGain) == false) {
					return false;
				}
				set_object_member(param_obj, "drc_fixedGain", drc_fixedGain);
			}

			set_object_member(command_obj_, "param", param_obj);
		}

		// statusがなくてもOK
		if (command_obj_.count("status") == 1) {
			if (command_obj_["status"].is<picojson::object>() == false) {
				cout << "\"status\" must be object!" << endl;
			}
			picojson::object status_obj = command_obj_["status"].get<picojson::object>();

			if (status_obj.count("focus_x") == 1) {
				float focus_x;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->focus_x), &focus_x) == false) {
					return false;
				}
				set_object_member(status_obj, "focus_x", focus_x);
			}

			if (status_obj.count("focus_y") == 1) {
				float focus_y;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->focus_y), &focus_y) == false) {
					return false;
				}
				set_object_member(status_obj, "focus_y", focus_y);
			}

			set_object_member(command_obj_, "status", status_obj);
		}
	}
	else {
		cout << "invalid accessor!\n" << endl;
		return false;
	}
	command_obj_["result"] = static_cast<picojson::value>(true);
	return true;
}

} // namespace command
