#include <iostream>
#include <stdint.h>
#include <picojson_misc.h>
#include "command_comp.h"
#include <dsp_lib/include/dsp/comp.h>
#include <dsp_lib/include/misc/smoothing.h>
#include <spi/spi_api.h>
#include <control/misc.h>

using namespace std;
using namespace dsp_lib;

namespace control {

bool
CommandComp(
	picojson::object				&command_obj_,
	std::string						access_,
	dsp_lib::DspManager::NetList	netlist_
)
{
	//uint32_t config_base	= reinterpret_cast<uint32_t>(netlist_.config);
	uint32_t param_base		= reinterpret_cast<uint32_t>(netlist_.param);
	uint32_t status_base	= reinterpret_cast<uint32_t>(netlist_.status);

	//Comp::Config*	config_offset	= 0x00000000;
	Comp::Param*	param_offset = 0x00000000;
	Comp::Status*	status_offset	= 0x00000000;

	if (access_ == "set") {
		// paramがないとエラー
		picojson::object param_obj;
		if (get_object_member(command_obj_, "param", param_obj) == false) {
			cout << "\"param\" not found!" << endl;
			return false;
		}

		if (param_obj.count("attack_time") == 1) {
			if (param_obj["attack_time"].is<double>() == false) {
				cout << "param.attack_time must be number!" << endl;
				return false;
			}
			float attack_time = static_cast<float>(param_obj["attack_time"].get<double>());
			float attack_ff = TimeToForgetFactor(attack_time, DSP_SAMPLING_FREQ);

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->attack), attack_ff) == false) {
				return false;
			}
		}

		if (param_obj.count("release_time") == 1) {
			if (param_obj["release_time"].is<double>() == false) {
				cout << "param.release_time must be number!" << endl;
				return false;
			}
			float release_time = static_cast<float>(param_obj["release_time"].get<double>());
			float release_ff = TimeToForgetFactor(release_time, DSP_SAMPLING_FREQ);

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->release), release_ff) == false) {
				return false;
			}
		}

		if (param_obj.count("threshold") == 1) {
			if (param_obj["threshold"].is<double>() == false) {
				cout << "param.threshold must be number!" << endl;
				return false;
			}
			float threshold_db = static_cast<float>(param_obj["threshold"].get<double>());
			float threshold = ToAntiLog(threshold_db);

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->threshold), threshold) == false) {
				return false;
			}
		}

		if (param_obj.count("makeup_gain") == 1) {
			if (param_obj["makeup_gain"].is<double>() == false) {
				cout << "param.makeup_gain must be number!" << endl;
				return false;
			}
			float makeup_gain_db = static_cast<float>(param_obj["makeup_gain"].get<double>());
			float makeup_gain = ToAntiLog(makeup_gain_db);

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->makeup_gain), makeup_gain) == false) {
				return false;
			}
		}

		if (param_obj.count("ratio") == 1) {
			if (param_obj["ratio"].is<double>() == false) {
				cout << "param.ratio must be number!" << endl;
				return false;
			}
			uint32_t ratio = static_cast<uint32_t>(param_obj["ratio"].get<double>());
			if (
				ratio > Comp::COMP_RATIO_INF ||
				ratio < Comp::COMP_RATIO_1   )
			{
				cout << "\"ratio\" out of range! ("
				<< Comp::COMP_RATIO_1 << '~'
				<< Comp::COMP_RATIO_INF << ')' << endl;
			}

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->ratio), ratio) == false) {
				return false;
			}
		}

		if (param_obj.count("knee") == 1) {
			if (param_obj["knee"].is<string>() == false) {
				cout << "param.knee must be string!" << endl;
				return false;
			}
			string knee_str = param_obj["knee"].get<string>();
			uint32_t knee;
			if (knee_str == "hard") {
				knee = Comp::COMP_KNEE_HARD;
			} else if (knee_str == "soft1") {
				knee = Comp::COMP_KNEE_SOFT1;
			} else if (knee_str == "soft2") {
				knee = Comp::COMP_KNEE_SOFT2;
			} else {
				cout << "\"knee\" must be hard, soft1, or soft2!" << endl;
				return false;
			}

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->knee), knee) == false) {
				return false;
			}
		}

		if (param_obj.count("level_type") == 1) {
			if (param_obj["level_type"].is<string>() == false) {
				cout << "param.level_type must be string!" << endl;
				return false;
			}
			string level_type_str = param_obj["level_type"].get<string>();
			uint32_t level_type;
			if (level_type_str == "peak") {
				level_type = Comp::COMP_LEVEL_TYPE_PEAK;
			} else if (level_type_str == "rms") {
				level_type = Comp::COMP_LEVEL_TYPE_RMS;
			} else {
				cout << "\"level_type\" must be peak or rms!" << endl;
				return false;
			}

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->level_type), level_type) == false) {
				return false;
			}
		}

	}
	else if (access_ == "get") {
		// paramがなくてもOK
		if (command_obj_.count("param") == 1) {
			if (command_obj_["param"].is<picojson::object>() == false) {
				cout << "\"param\" must be object!" << endl;
			}
			picojson::object param_obj = command_obj_["param"].get<picojson::object>();

			if (param_obj.count("attack_time") == 1) {
				float attack_ff;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->attack), &attack_ff) == false) {
					return false;
				}
				float attack_time = ForgetFactorToTime(attack_ff, DSP_SAMPLING_FREQ);

				set_object_member(param_obj, "attack_time", attack_time);
			}

			if (param_obj.count("release_time") == 1) {
				float release_ff;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->release), &release_ff) == false) {
					return false;
				}
				float release_time = ForgetFactorToTime(release_ff, DSP_SAMPLING_FREQ);

				set_object_member(param_obj, "release_time", release_time);
			}

			if (param_obj.count("threshold") == 1) {
				float threshold;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->threshold), &threshold) == false) {
					return false;
				}
				float threshold_db = ToDb(threshold);

				set_object_member(param_obj, "threshold", threshold_db);
			}

			if (param_obj.count("makeup_gain") == 1) {
				float makeup_gain;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->makeup_gain), &makeup_gain) == false) {
					return false;
				}
				float makeup_gain_db = ToDb(makeup_gain);

				set_object_member(param_obj, "makeup_gain", makeup_gain_db);
			}

			if (param_obj.count("ratio") == 1) {
				uint32_t ratio;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->ratio), &ratio) == false) {
					return false;
				}
				set_object_member(param_obj, "ratio", ratio);
			}

			if (param_obj.count("knee") == 1) {
				uint32_t knee;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->knee), &knee) == false) {
					return false;
				}
				string knee_str;
				switch (knee) {
					case Comp::COMP_KNEE_HARD:
					knee_str = "hard";
					break;
					case Comp::COMP_KNEE_SOFT1:
					knee_str = "soft1";
					break;
					case Comp::COMP_KNEE_SOFT2:
					knee_str = "soft2";
					break;
					default:
					cout << "knee is broken!(" << knee << ")" << endl;
					return false;
				}
				set_object_member(param_obj, "knee", knee_str);
			}

			if (param_obj.count("level_type") == 1) {
				uint32_t level_type;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->level_type), &level_type) == false) {
					return false;
				}
				string level_type_str;
				switch (level_type) {
					case Comp::COMP_LEVEL_TYPE_PEAK:
					level_type_str = "peak";
					break;
					case Comp::COMP_LEVEL_TYPE_RMS:
					level_type_str = "rms";
					break;
					default:
					cout << "level_type is broken!(" << level_type << ")" << endl;
					return false;
				}
				set_object_member(param_obj, "level_type", level_type_str);
			}
			set_object_member(command_obj_, "param", param_obj);
		}

		// statusがなくてもOK
		if (command_obj_.count("status") == 1) {
			if (command_obj_["status"].is<picojson::object>() == false) {
				cout << "\"status\" must be object!" << endl;
			}
			picojson::object status_obj = command_obj_["status"].get<picojson::object>();

			if (status_obj.count("gain_target") == 1) {
				float gain_target;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->gain_target), &gain_target) == false) {
					return false;
				}
				float gain_target_db = ToDb(gain_target);
				set_object_member(status_obj, "gain_target", gain_target_db);
			}

			if (status_obj.count("gain_current") == 1) {
				float gain_current;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->gain_current), &gain_current) == false) {
					return false;
				}
				float gain_current_db = ToDb(gain_current);
				set_object_member(status_obj, "gain_current", gain_current_db);
			}

			if (status_obj.count("peak") == 1) {
				float peak;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->peak), &peak) == false) {
					return false;
				}
				float peak_db = PowerToDb(peak);
				set_object_member(status_obj, "peak", peak_db);
			}

			if (status_obj.count("peak_release") == 1) {
				float peak_release;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->peak_release), &peak_release) == false) {
					return false;
				}
				float peak_db = PowerToDb(peak_release);
				set_object_member(status_obj, "peak_release", peak_db);
			}

			if (status_obj.count("rms") == 1) {
				float rms;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->rms), &rms) == false) {
					return false;
				}
				float rms_db = PowerToDb(rms);
				set_object_member(status_obj, "rms", rms_db);
			}
			set_object_member(command_obj_, "status", status_obj);
		}
	}
	else {
		cout << "invalid accessor!\n" << endl;
		return false;
	}
	command_obj_["result"] = static_cast<picojson::value>(true);
	return true;
}

} // namespace command
