#include <iostream>
#include <stdint.h>
#include <picojson_misc.h>
#include "command_delay.h"
#include <dsp_lib/include/dsp/delay.h>
#include <spi/spi_api.h>
#include <control/misc.h>

using namespace std;

namespace control {

bool
CommandDelay(
	picojson::object				&command_obj_,
	std::string						access_,
	dsp_lib::DspManager::NetList	netlist_
)
{
	uint32_t config_base	= reinterpret_cast<uint32_t>(netlist_.config);
	uint32_t param_base		= reinterpret_cast<uint32_t>(netlist_.param);
	//uint32_t status_base	= reinterpret_cast<uint32_t>(netlist_.status);

	dsp_lib::Delay::Config*	config_offset	= 0x00000000;
	dsp_lib::Delay::Param*	param_offset = 0x00000000;
	//dsp_lib::Delay::Status*	status_offset	= 0x00000000;

	if (access_ == "set") {
		// paramがないとエラー
		picojson::object param_obj;
		if (get_object_member(command_obj_, "param", param_obj) == false) {
			cout << "\"param\" not found!" << endl;
			return false;
		}

		if (param_obj.count("delay_samples") == 1) {
			if (param_obj["delay_samples"].is<double>() == false) {
				cout << "param.delay_samples must be number!" << endl;
				return false;
			}

			uint32_t delay_samples = static_cast<uint32_t>(param_obj["delay_samples"].get<double>());

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->delay_samples), delay_samples) == false) {
				return false;
			}
		}
	}
	else if (access_ == "get") {
		// paramがなくてもOK
		if (command_obj_.count("config") == 1) {
			if (command_obj_["config"].is<picojson::object>() == false) {
				cout << "\"config\" must be object!" << endl;
			}
			picojson::object config_obj = command_obj_["config"].get<picojson::object>();

			if (config_obj.count("delay_buf_size") == 1) {
				uint32_t delay_buf_size;
				if (ReadSharedRam(config_base + ToDspAddressing(&config_offset->buf_size), &delay_buf_size) == false) {
					cout << "read failed!" << endl;
					return false;
				}

				set_object_member(config_obj, "delay_buf_size", delay_buf_size);
				set_object_member(command_obj_, "config", config_obj);
			}
		}

		if (command_obj_.count("param") == 1) {
			if (command_obj_["param"].is<picojson::object>() == false) {
				cout << "\"param\" must be object!" << endl;
			}
			picojson::object param_obj = command_obj_["param"].get<picojson::object>();

			if (param_obj.count("delay_samples") == 1) {
				uint32_t delay_samples;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->delay_samples), &delay_samples) == false) {
					cout << "read failed!" << endl;
					return false;
				}

				set_object_member(param_obj, "delay_samples", delay_samples);
				set_object_member(command_obj_, "param", param_obj);
			}
		}
	}
	else {
		cout << "invalid accessor!\n" << endl;
		return false;
	}
	command_obj_["result"] = static_cast<picojson::value>(true);
	return true;
}

} // namespace command
