#include <iostream>
#include <stdint.h>
#include <picojson_misc.h>
#include "command_gate.h"
#include <dsp_lib/include/dsp/gate.h>
#include <dsp_lib/include/misc/smoothing.h>
#include <spi/spi_api.h>
#include <control/misc.h>

using namespace std;
using namespace dsp_lib;

namespace control {

bool
CommandGate(
	picojson::object				&command_obj_,
	std::string						access_,
	dsp_lib::DspManager::NetList	netlist_
)
{
	//uint32_t config_base	= reinterpret_cast<uint32_t>(netlist_.config);
	uint32_t param_base		= reinterpret_cast<uint32_t>(netlist_.param);
	uint32_t status_base	= reinterpret_cast<uint32_t>(netlist_.status);

	//Gate::Config*	config_offset	= 0x00000000;
	Gate::Param*	param_offset = 0x00000000;
	Gate::Status*	status_offset	= 0x00000000;

	if (access_ == "set") {
		// paramがないとエラー
		picojson::object param_obj;
		if (get_object_member(command_obj_, "param", param_obj) == false) {
			cout << "\"param\" not found!" << endl;
			return false;
		}

		if (param_obj.count("attack_time") == 1) {
			if (param_obj["attack_time"].is<double>() == false) {
				cout << "param.attack_time must be number!" << endl;
				return false;
			}
			float attack_time = static_cast<float>(param_obj["attack_time"].get<double>());
			uint32_t attack_sample;
			TimeToSample(attack_time, &attack_sample);

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->attack), attack_sample) == false) {
				return false;
			}
		}

		if (param_obj.count("release_time") == 1) {
			if (param_obj["release_time"].is<double>() == false) {
				cout << "param.release_time must be number!" << endl;
				return false;
			}
			float release_time = static_cast<float>(param_obj["release_time"].get<double>());
			uint32_t release_sample;
			TimeToSample(release_time, &release_sample);

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->release), release_sample) == false) {
				return false;
			}
		}

		if (param_obj.count("hold_time") == 1) {
			if (param_obj["hold_time"].is<double>() == false) {
				cout << "param.hold_time must be number!" << endl;
				return false;
			}
			float hold_time = static_cast<float>(param_obj["hold_time"].get<double>());
			uint32_t hold_block;
			TimeToBlock(hold_time, &hold_block);

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->hold), hold_block) == false) {
				return false;
			}
		}

		if (param_obj.count("threshold") == 1) {
			if (param_obj["threshold"].is<double>() == false) {
				cout << "param.threshold must be number!" << endl;
				return false;
			}
			float threshold_db = static_cast<float>(param_obj["threshold"].get<double>());

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->threshold), threshold_db) == false) {
				return false;
			}
		}

		if (param_obj.count("hysteresis") == 1) {
			if (param_obj["hysteresis"].is<double>() == false) {
				cout << "param.hysteresis must be number!" << endl;
				return false;
			}
			float hysteresis_db = static_cast<float>(param_obj["hysteresis"].get<double>());

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->hysteresis), hysteresis_db) == false) {
				return false;
			}
		}

		if (param_obj.count("depth") == 1) {
			if (param_obj["depth"].is<double>() == false) {
				cout << "param.depth must be number!" << endl;
				return false;
			}
			float depth_db = static_cast<float>(param_obj["depth"].get<double>());

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->depth), depth_db) == false) {
				return false;
			}
		}

		if (param_obj.count("level_type") == 1) {
			if (param_obj["level_type"].is<string>() == false) {
				cout << "param.level_type must be string!" << endl;
				return false;
			}
			string level_type_str = param_obj["level_type"].get<string>();
			uint32_t level_type;
			if (level_type_str == "peak") {
				level_type = Gate::GATE_LEVEL_TYPE_PEAK;
			} else if (level_type_str == "rms") {
				level_type = Gate::GATE_LEVEL_TYPE_RMS;
			} else {
				cout << "\"level_type\" must be peak or rms!" << endl;
				return false;
			}

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->level_type), level_type) == false) {
				return false;
			}
		}

	}
	else if (access_ == "get") {
		// paramがなくてもOK
		if (command_obj_.count("param") == 1) {
			if (command_obj_["param"].is<picojson::object>() == false) {
				cout << "\"param\" must be object!" << endl;
			}
			picojson::object param_obj = command_obj_["param"].get<picojson::object>();

			if (param_obj.count("attack_time") == 1) {
				uint32_t attack_sample;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->attack), &attack_sample) == false) {
					return false;
				}
				float attack_time;
				SampleToTime(attack_sample, &attack_time);

				set_object_member(param_obj, "attack_time", attack_time);
			}

			if (param_obj.count("release_time") == 1) {
				uint32_t release_sample;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->release), &release_sample) == false) {
					return false;
				}
				float release_time;
				SampleToTime(release_sample, &release_time);

				set_object_member(param_obj, "release_time", release_time);
			}

			if (param_obj.count("hold_time") == 1) {
				uint32_t hold_block;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->hold), &hold_block) == false) {
					return false;
				}
				float hold_time;
				BlockToTime(hold_block, &hold_time);

				set_object_member(param_obj, "hold_time", hold_time);
			}

			if (param_obj.count("threshold") == 1) {
				float threshold_db;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->threshold), &threshold_db) == false) {
					return false;
				}

				set_object_member(param_obj, "threshold", threshold_db);
			}

			if (param_obj.count("hysteresis") == 1) {
				float hysteresis_db;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->hysteresis), &hysteresis_db) == false) {
					return false;
				}

				set_object_member(param_obj, "hysteresis", hysteresis_db);
			}

			if (param_obj.count("depth") == 1) {
				float depth_db;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->depth), &depth_db) == false) {
					return false;
				}

				set_object_member(param_obj, "depth", depth_db);
			}

			if (param_obj.count("level_type") == 1) {
				uint32_t level_type;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->level_type), &level_type) == false) {
					return false;
				}
				string level_type_str;
				switch (level_type) {
					case Gate::GATE_LEVEL_TYPE_PEAK:
					level_type_str = "peak";
					break;
					case Gate::GATE_LEVEL_TYPE_RMS:
					level_type_str = "rms";
					break;
					default:
					cout << "current level_type is broken!(" << level_type << ")" << endl;
					return false;
				}
				set_object_member(param_obj, "level_type", level_type_str);
			}
			set_object_member(command_obj_, "param", param_obj);
		}

		// statusがなくてもOK
		if (command_obj_.count("status") == 1) {
			if (command_obj_["status"].is<picojson::object>() == false) {
				cout << "\"status\" must be object!" << endl;
			}
			picojson::object status_obj = command_obj_["status"].get<picojson::object>();

			if (status_obj.count("gain_target") == 1) {
				float gain_target_db;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->gain_target), &gain_target_db) == false) {
					return false;
				}
				set_object_member(status_obj, "gain_target", gain_target_db);
			}

			if (status_obj.count("gain_current") == 1) {
				float gain_current_db;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->gain_current), &gain_current_db) == false) {
					return false;
				}
				set_object_member(status_obj, "gain_current", gain_current_db);
			}

			if (status_obj.count("peak") == 1) {
				float peak;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->peak), &peak) == false) {
					return false;
				}
				float peak_db = PowerToDb(peak);
				set_object_member(status_obj, "peak", peak_db);
			}

			if (status_obj.count("peak_release") == 1) {
				float peak_release;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->peak_release), &peak_release) == false) {
					return false;
				}
				float peak_db = PowerToDb(peak_release);
				set_object_member(status_obj, "peak_release", peak_db);
			}

			if (status_obj.count("rms") == 1) {
				float rms;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->rms), &rms) == false) {
					return false;
				}
				float rms_db = PowerToDb(rms);
				set_object_member(status_obj, "rms", rms_db);
			}

			if (status_obj.count("gate_status") == 1) {
				uint32_t gate_status;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->gate_status), &gate_status) == false) {
					return false;
				}
				string gate_status_str;
				switch(gate_status) {
				case Gate::GATE_STATUS_CLOSE:
					gate_status_str = "close";
					break;
				case Gate::GATE_STATUS_HOLD:
					gate_status_str = "hold";
					break;
				case Gate::GATE_STATUS_OPEN:
					gate_status_str = "open";
					break;
				default:
					cout << "gate_status is broken!(" << gate_status << ")" << endl;
					return false;
				}
				set_object_member(status_obj, "gate_status", gate_status_str);
			}

			set_object_member(command_obj_, "status", status_obj);
		}
	}
	else {
		cout << "invalid accessor!\n" << endl;
		return false;
	}
	command_obj_["result"] = static_cast<picojson::value>(true);
	return true;
}

} // namespace command
