#include <iostream>
#include <stdint.h>
#include <picojson_misc.h>
#include <control/misc.h>
#include "command_iir.h"
#include <dsp_lib/include/dsp/iir.h>
#include <dsp_lib/misc/biquad/fdesign.h>
#include <dsp_lib/misc/biquad/fdesign_prototypes.h>
#include <spi/spi_api.h>

// #define ENABLE_OUTPUT_IIR_SET_COEF_INFO

using namespace std;

namespace {
	const uint32_t SIZE_OF_IIR_COEF = 5;
}

namespace control {

bool
CommandIir(
	picojson::object				&command_obj_,
	std::string						access_,
	dsp_lib::DspManager::NetList	netlist_
)
{
	uint32_t config_base	= reinterpret_cast<uint32_t>(netlist_.config);
	uint32_t param_base		= reinterpret_cast<uint32_t>(netlist_.param);
	//uint32_t status_base	= reinterpret_cast<uint32_t>(netlist_.status);

	dsp_lib::Iir::Config*	config_offset	= 0x00000000;
	dsp_lib::Iir::Param*	param_offset	= 0x00000000;
	//dsp_lib::Iir::Status*	status_offset	= 0x00000000;

	uint32_t	coef_base;
	dsp_lib::Iir::IirCoef*		coef_offset = 0x00000000;

	if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->coef), &coef_base) == false) {
		return false;
	}

	uint32_t	coef_update_req;
	uint32_t	coef_update_req_base;
	uint32_t*	coef_update_req_offset = 0x00000000;

	if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->coef_update_req), &coef_update_req_base) == false) {
		return false;
	}

	dsp_lib::Iir::Config	config;

	if (ReadSharedRam(config_base + ToDspAddressing(&config_offset->number_of_stage), &config.number_of_stage) == false) {
		return false;
	}

	if (access_ == "set") {
		picojson::object param_obj;

		if (get_object_member(command_obj_, "param", param_obj) == false) {
			cout << "invalid command!" << endl;
			return false;
		}


		uint32_t	stage;
		string	type;
		float	frequency;
		float	q;
		float	gain;

		if (
			get_object_member(param_obj, "stage",		stage)		== false ||
			get_object_member(param_obj, "type",		type)		== false ||
			get_object_member(param_obj, "frequency",	frequency)	== false ||
			get_object_member(param_obj, "q",			q)			== false ||
			get_object_member(param_obj, "gain",		gain)		== false )
		{
			cout << "invalid \"param\"!" << endl;
			return false;
		}

		if (stage >= config.number_of_stage) {
			cout <<
				"param.stage(" + to_string(stage) +
				") is larger than number_of_stage(" + to_string(config.number_of_stage) + ")!" << endl;
			return false;
		}

		if (ReadSharedRam(coef_update_req_base + ToDspAddressing(&coef_update_req_offset[stage]), &coef_update_req) == false) {
			return false;
		}
		if (coef_update_req != 0) {
			cout << "coef update busy!" << endl;
			return false;
		}

		uint32_t	type_id;
		float		coef[SIZE_OF_IIR_COEF];

		if (type == "through") {
			type_id = dsp_lib::FILTER_TYPE_THROUGH;
		} else if (type == "peq") {
			type_id = dsp_lib::FILTER_TYPE_PEQ;
		} else if (type == "hpf6") {
			type_id = dsp_lib::FILTER_TYPE_HPF6;
		} else if (type == "lpf6") {
			type_id = dsp_lib::FILTER_TYPE_LPF6;
		} else if (type == "hpf12") {
			type_id = dsp_lib::FILTER_TYPE_HPF12;
		} else if (type == "lpf12") {
			type_id = dsp_lib::FILTER_TYPE_LPF12;
		} else if (type == "low_shelving") {
			type_id = dsp_lib::FILTER_TYPE_LOW_SHELVING;
		} else if (type == "high_shelving") {
			type_id = dsp_lib::FILTER_TYPE_HIGH_SHELVING;
		} else if (type == "all_pass") {
			type_id = dsp_lib::FILTER_TYPE_ALL_PASS;
		} else if (type == "notch") {
			type_id = dsp_lib::FILTER_TYPE_NOTCH;
		} else if (type == "horn_eq") {
			type_id = dsp_lib::FILTER_TYPE_HORN_EQ;
		} else {
			cout << "invalid filter type!" << endl;
			return false;
		}
		dsp_lib::Fdesign(type_id, frequency, gain, q, /*alphaindex*/0, /*eqon*/1, coef);
		
		// 係数をADI用に変換
		float		coef_adi[SIZE_OF_IIR_COEF];
		ConvertIirCoefToAdi(coef, coef_adi);

		// DSP側を変更したらコメント解除
		//if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->type_id), type_id) == false) {
		//	return false;
		//}
		//if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->frequency), frequency) == false) {
		//	return false;
		//}
		//if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->q), q) == false) {
		//	return false;
		//}
		//if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->gain), gain) == false) {
		//	return false;
		//}


		for (uint32_t coef_no = 0; coef_no < SIZE_OF_IIR_COEF; coef_no++) {
			if (WriteSharedRam(coef_base + ToDspAddressing(&coef_offset[stage][coef_no]), coef_adi[coef_no]) == false) {
				return false;
			}
		}

		coef_update_req = 1;
		if (WriteSharedRam(coef_update_req_base + ToDspAddressing(&coef_update_req_offset[stage]), &coef_update_req) == false) {
			return false;
		}
#ifdef ENABLE_OUTPUT_IIR_SET_COEF_INFO
		picojson::array coef_array(SIZE_OF_IIR_COEF);
		for (uint32_t coef_no = 0; coef_no < SIZE_OF_IIR_COEF; coef_no++) {
			coef_array[coef_no] = get_number_value(coef[coef_no]);
		}

		set_object_member(param_obj, "coef", coef_array);
		set_object_member(command_obj_, "param", param_obj);
#endif
	}
	else if (access_ == "get") {
		picojson::object param_obj;

		if (get_object_member(command_obj_, "param", param_obj) == false) {
			cout << "invalid command!" << endl;
			return false;
		}

		uint32_t	stage;
		picojson::null dummy_null;
		if (
			get_object_member(param_obj, "stage",		stage)		== false ||
			get_object_member(param_obj, "type",		dummy_null)	== false ||
			get_object_member(param_obj, "frequency",	dummy_null)	== false ||
			get_object_member(param_obj, "q",			dummy_null)	== false ||
			get_object_member(param_obj, "gain",		dummy_null)	== false )
		{
			cout << "\"param\" is invalid!" << endl;
			return false;
		}

		if (stage >= config.number_of_stage) {
			cout <<
				"param.stage(" + to_string(stage) +
				") is larger than number_of_stage(" + to_string(config.number_of_stage) + ")!" << endl;
			return false;
		}


		// DSP側を変更したらコメント解除
		//string		type;
		//uint32_t	type_id;
		//float		frequency;
		//float		q;
		//float		gain;
		//if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->type_id), &type_id) == false) {
		//	return false;
		//}
		//if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->frequency), &frequency) == false) {
		//	return false;
		//}
		//if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->q), &q) == false) {
		//	return false;
		//}
		//if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->gain), &gain) == false) {
		//	return false;
		//}
		//switch (type_id) {
		//case dsp_lib::DspManager::DSP_MODULE_ID_NOP:
		//	type = "nop";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_THROUGH:
		//	type = "through";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_GAIN:
		//	type = "gain";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_MATRIX:
		//	type = "matrix";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_IIR:
		//	type = "iir";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_PEAK:
		//	type = "peak";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_RMS:
		//	type = "rms";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_COMP:
		//	type = "comp";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_GATE:
		//	type = "gate";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_DELAY:
		//	type = "delay";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_AEC:
		//	type = "aec";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_NC:
		//	type = "nc";
		//	break;
		//case dsp_lib::DspManager::DSP_MODULE_ID_MICROPHONEARRAY:
		//	type = "microphonearray";
		//	break;
		//default:
		//	cout << "DSP's type_id is invalid!" << endl;
		//	return false;
		//	break;
		//}
		//set_object_member(param_obj, "type", type);
		//set_object_member(param_obj, "frequency", frequency);
		//set_object_member(param_obj, "q", q);
		//set_object_member(param_obj, "gain", gain);

		//set_object_memberの動作を確認後削除予定
		//command_obj_["param"].get<picojson::object>().erase("type");
		//command_obj_["param"].get<picojson::object>()["type"] = static_cast<picojson::value>(type);
		//command_obj_["param"].get<picojson::object>().erase("frequency");
		//command_obj_["param"].get<picojson::object>()["frequency"] = get_number_value(frequency);

		float coef_adi[SIZE_OF_IIR_COEF];
		for (uint32_t coef_no = 0; coef_no < SIZE_OF_IIR_COEF; coef_no++) {
			if (ReadSharedRam(coef_base + ToDspAddressing(&coef_offset[stage][coef_no]), &coef_adi[coef_no]) == false) {
				return false;
			}
		}

		float coef[SIZE_OF_IIR_COEF];
		ConvertIirCoefFromAdi(coef_adi, coef);

		picojson::array coef_array(SIZE_OF_IIR_COEF);
		for (uint32_t coef_no = 0; coef_no < SIZE_OF_IIR_COEF; coef_no++) {
			coef_array[coef_no] = get_number_value(coef[coef_no]);
		}


		set_object_member(param_obj, "coef", coef_array);
		set_object_member(command_obj_, "param", param_obj);

	} else {
		cout << "invalid accessor!\n" << endl;
		return false;
	}
	set_object_member(command_obj_, "result", true);
	return true;
}

} // namespace command
