#include <iostream>
#include <stdint.h>
#include <picojson_misc.h>
#include "command_matrix.h"
#include <dsp_lib/include/dsp/matrix.h>
#include <spi/spi_api.h>
#include <control/misc.h>

using namespace std;

namespace control {

uint32_t
GetMatrixIndex(
	dsp_lib::Matrix::Config		&config_,
	uint32_t					input_no_,
	uint32_t					output_no_
)
{
	uint32_t number_of_input	= config_.number_of_input;
	//uint32_t number_of_output	= config_.number_of_output;
	uint32_t index				= input_no_	+ output_no_ * number_of_input;
	return index;
}

bool
CommandMatrix(
	picojson::object				&command_obj_,
	std::string						access_,
	dsp_lib::DspManager::NetList	netlist_
)
{
	uint32_t config_base	= reinterpret_cast<uint32_t>(netlist_.config);
	uint32_t param_base		= reinterpret_cast<uint32_t>(netlist_.param);
	//uint32_t status_base	= reinterpret_cast<uint32_t>(netlist_.status);

	dsp_lib::Matrix::Config*	config_offset	= 0x00000000;
	dsp_lib::Matrix::Param*		param_offset	= 0x00000000;
	//dsp_lib::Matrix::Status*	status_offset	= 0x00000000;

	dsp_lib::Matrix::Config	config;

	if (ReadSharedRam(config_base + ToDspAddressing(&config_offset->number_of_input), &config.number_of_input) == false) {
		return false;
	}
	if (ReadSharedRam(config_base + ToDspAddressing(&config_offset->number_of_output), &config.number_of_output) == false) {
		return false;
	}

	uint32_t	gain_target_base;
	float*		gain_target_offset = 0x00000000;

	if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->gain_target), &gain_target_base) == false) {
		return false;
	}

	if (access_ == "set") {
		if (object_member_check(command_obj_, "param", picojson::object_type) == false) {
			cout << "invalid command!" << endl;
			return false;
		}

		picojson::object param_obj = command_obj_["param"].get<picojson::object>();

		if (param_obj.count("gain") == 1) {
			if (param_obj["gain"].is<picojson::object>() == false) {
				cout << "param.gain must be object" << endl;
				return false;
			}

			picojson::object gain_obj = param_obj["gain"].get<picojson::object>();

			if (
				object_member_check(gain_obj, "input_ch", picojson::number_type) == false ||
				object_member_check(gain_obj, "output_ch", picojson::number_type) == false ||
				object_member_check(gain_obj, "gain", picojson::number_type) == false)
			{
				cout << "param.gain: invalid command!" << endl;
				return false;
			}

			uint32_t input_ch = static_cast<uint32_t>(gain_obj["input_ch"].get<double>());
			uint32_t output_ch = static_cast<uint32_t>(gain_obj["output_ch"].get<double>());
			float gain = ToAntiLog(static_cast<float>(gain_obj["gain"].get<double>()));

			if (input_ch >= config.number_of_input) {
				cout <<
					"param.gain.input_ch(" + to_string(input_ch) +
					") is larger than number_of_input(" + to_string(config.number_of_input) + ")!" << endl;
				return false;
			}
			if (output_ch >= config.number_of_output) {
				cout <<
					"param.gain.output_ch(" + to_string(output_ch) +
					") is larger than number_of_output(" + to_string(config.number_of_output) + ")!" << endl;
				return false;
			}

			uint32_t	gain_target_index = GetMatrixIndex(config, input_ch, output_ch);

			if (WriteSharedRam(gain_target_base + ToDspAddressing(&gain_target_offset[gain_target_index]), gain) == false) {
				return false;
			}
		}
	}
	else if (access_ == "get") {
		if (object_member_check(command_obj_, "param", picojson::object_type) == false) {
			cout << "invalid command!" << endl;
			return false;
		}

		picojson::object param_obj = command_obj_["param"].get<picojson::object>();

		if (param_obj.count("gain") == 1) {
			if (param_obj["gain"].is<picojson::object>() == true) {
				picojson::object gain_obj = param_obj["gain"].get<picojson::object>();
				if (
					object_member_check(gain_obj, "input_ch", picojson::number_type) == false ||
					object_member_check(gain_obj, "output_ch", picojson::number_type) == false)
				{
					cout << "param.gain: invalid command!" << endl;
					return false;
				}

				uint32_t	input_ch = static_cast<uint32_t>(gain_obj["input_ch"].get<double>());
				uint32_t	output_ch = static_cast<uint32_t>(gain_obj["output_ch"].get<double>());
				float		gain;

				if (input_ch >= config.number_of_input) {
					cout <<
						"param.gain.input_ch(" + to_string(input_ch) +
						") is larger than number_of_input(" + to_string(config.number_of_input) + ")!" << endl;
					return false;
				}
				if (output_ch >= config.number_of_output) {
					cout <<
						"MATRIX: param.gain.output_ch(" + to_string(output_ch) +
						") is larger than number_of_output(" + to_string(config.number_of_output) + ")!" << endl;
					return false;
				}

				uint32_t	gain_target_index = GetMatrixIndex(config, input_ch, output_ch);


				if (ReadSharedRam(gain_target_base + ToDspAddressing(&gain_target_offset[gain_target_index]), &gain) == false) {
					return false;
				}
				float gain_db = ToDb(gain);

				gain_obj.erase("gain");
				gain_obj["gain"] = get_number_value(gain_db);
				command_obj_["param"].get<picojson::object>().erase("gain");
				command_obj_["param"].get<picojson::object>()["gain"] = static_cast<picojson::value>(gain_obj);

			}
			else if (param_obj["gain"].is<picojson::array>() == true) {
				// array はあとで対応する
				cout << "MATRIX: gain must be object or array!" << endl;
				return false;
			}
			else {
				cout << "MATRIX: gain must be object or array!" << endl;
				return false;
			}
		}
	}
	else {
		cout << "invalid accessor!\n" << endl;
		return false;
	}
	command_obj_["result"] = static_cast<picojson::value>(true);
	return true;
}

} // namespace command
