#include <sys/time.h>

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <picojson_misc.h>
#include <spi/spi_api.h>
#include <shared/shared_ram.h>
#include "execute_command.h"
#include <control/misc.h>
#include <control/command/command_gain.h>
#include <control/command/command_matrix.h>
#include <control/command/command_iir.h>
#include <control/command/command_peak.h>
#include <control/command/command_rms.h>
#include <control/command/command_comp.h>
#include <control/command/command_gate.h>
#include <control/command/command_delay.h>
#include <control/command/command_aec.h>
#include <control/command/command_nc.h>
#include <control/command/command_microphonearray.h>

namespace control {

void
PrintCommandNumber(uint32_t command_no_)
{
	cout << "command[" << command_no_ << "]: ";
}

void
PrintCommandError(
	picojson::object &command_obj_,
	uint32_t command_no_, 
	string msg_
)
{
	PrintCommandNumber(command_no_);
	cout << msg_ << endl;
	disp_value(command_obj_);
}

bool
ExecuteCommand(
	char*							result_file_name_,
	picojson::array					command_array_,
	dsp_lib::DspManager::NetList*	&netlists_,
	uint32_t						number_of_netlist
)
{
	struct timeval s, e;

	//gettimeofday(&s, NULL);

	ofstream ofs(result_file_name_);
	
	//gettimeofday(&e, NULL);
	//printf("ofstream ofs(resultfile) time = %lf\n", (e.tv_sec - s.tv_sec) + (e.tv_usec - s.tv_usec)*1.0E-6);
	
	if (!ofs) {
		cout << "file: " << result_file_name_ << " cannot be opened!" << endl;
		return false;
	}

	picojson::array		command_return_array;

	// command_arrayを走査して、各command_objに従いパラメータ設定・取得する
	for (uint32_t command_no = 0; command_no < command_array_.size(); command_no++) {

		if (command_array_[command_no].is<picojson::object>() == false ){
			PrintCommandNumber(command_no);
			cout << "command must be object!" <<endl;
			disp_value(command_array_[command_no]);
			return false;
		}
		picojson::object	&command_obj_ref = command_array_[command_no].get<picojson::object>();	// エラー時の表示用
		picojson::object	command_obj = command_array_[command_no].get<picojson::object>();		// 書き換えて出力用

		// commandオブジェクトのメンバの存在と型を確認
		if (
			object_member_check(command_obj, "access",		picojson::string_type) == false ||
			object_member_check(command_obj, "block_no",	picojson::number_type) == false	||
			object_member_check(command_obj, "type",		picojson::string_type) == false	)
		{
			PrintCommandError(command_obj, command_no, "invalid command!");
			return false;
		}

		std::string		access		= command_obj["access"].get<string>();
		uint32_t		block_no	= static_cast<uint32_t>(command_obj["block_no"].get<double>());
		std::string		type		= command_obj["type"].get<string>();

		if (block_no >= number_of_netlist) {
			PrintCommandError(command_obj, command_no, 
				"block_no(" + to_string(block_no) + ") is larger than number of netlist(" + to_string(number_of_netlist) + ")!");
			return false;
		}

		dsp_lib::DspManager::NetList netlist = netlists_[block_no];

		
		switch (netlist.type) {
		case dsp_lib::DspManager::DSP_MODULE_ID_NOP:
			if (type != "nop") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"nop\", but command is \"" + type + "\"");
				return false;
			}
			// 何もしない
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_THROUGH:
			if (type != "through") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"through\", but command is \"" + type + "\"");
				return false;
			}
			// 何もしない
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_GAIN:
			/*if (type != "gain") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"gain\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandGain(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_MATRIX:
			/*if (type != "matrix") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"matrix\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandMatrix(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_IIR:
			/*if (type != "iir") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"iir\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandIir(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_PEAK:
			/*if (type != "peak") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"peak\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandPeak(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_RMS:
			/*if (type != "rms") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"rms\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandRms(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_COMP:
			/*if (type != "comp") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"comp\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandComp(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_GATE:
			/*if (type != "gate") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"gate\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandGate(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}			
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_DELAY:
			/*if (type != "delay") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"delay\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandDelay(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}			
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_AEC:
			/*if (type != "aec") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"aec\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandAec(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_NC:
			/*if (type != "nc") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"nc\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandNc(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		case dsp_lib::DspManager::DSP_MODULE_ID_MICROPHONEARRAY:
			/*if (type != "microphonearray") {
				PrintCommandError(command_obj_ref, command_no,
					"netlist[" + to_string(block_no) + "] is \"microphonearray\", but command is \"" + type + "\"");
				return false;
			}*/
			if (CommandMicrophonearray(command_obj, access, netlist) == false) {
				PrintCommandError(command_obj, command_no, type + ": invalid comannd!");
				return false;
			}
			break;
		default:
			PrintCommandError(command_obj_ref, command_no,
				"netlist[" + to_string(block_no) + "]: Unknown dsp type!");
			return false;
			break;
		}

		command_return_array.push_back(static_cast<picojson::value>(command_obj));

	}

	ofs << picojson::value(command_return_array).serialize(true);

	return true;
}

} // namespace command
