#include <cmath>
#include <shared/shared_ram.h>
#include "misc.h"

namespace control {

float
PowerToDb(float power_)
{
	return 10.0f * log10f(power_);
}

float
ToDb(float antilog_)
{
	return 20.0f * log10f(antilog_);
}

float
ToAntiLog(float db_)
{
	return powf(10, db_ * 0.05f);
}

uint32_t
ToDspAddressing(void* cpu_addressing_)
{
	uint32_t dsp_addressing = reinterpret_cast<uint32_t>(cpu_addressing_)/SHAREDRAM_ADDRESS_UNIT_BYTE;
	return dsp_addressing;
}

void
ConvertIirCoefToAdi(
		const float * before_,
		float * after_
)
{
	// for ADI

	// [before]
	//	    	b0 + b1*z^-1 + b2*z^-2
	// H(z) =	-----------------------
	//    		1 + a1*z^-1 + a2*z^-2

	// [after]
	//	    	b0 + b1*z^-1 + b2*z^-2
	// H(z) =	-----------------------
	//    		1 - a1*z^-1 - a2*z^-2

	after_[0] = -before_[4];	// a2
	after_[1] = -before_[3];	// a1
	after_[2] = before_[2];		// b2
	after_[3] = before_[1];		// b1
	after_[4] = before_[0];		// b0
}

void
ConvertIirCoefFromAdi(
		const float * before_,
		float * after_
)
{
	// for ADI

	// [before]
	//	    	b0 + b1*z^-1 + b2*z^-2
	// H(z) =	-----------------------
	//    		1 - a1*z^-1 - a2*z^-2

	// [after]
	//	    	b0 + b1*z^-1 + b2*z^-2
	// H(z) =	-----------------------
	//    		1 + a1*z^-1 + a2*z^-2

	after_[0] = before_[4];		// b0
	after_[1] = before_[3];		// b1
	after_[2] = before_[2];		// b2
	after_[3] = -before_[1];	// a1
	after_[4] = -before_[0];	// a2
}

} // namespace control
