/*
 *	fir.h
 */

//********************************************************************************
#pragma once

//********************************************************************************
#include <stdint.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/misc/null_check.h>

//********************************************************************************
namespace dsp_lib {

//********************************************************************************
class Fir : public DspModule {
	public:
		//----------------------------------------
		typedef struct {
			uint32_t	taps;			//<! must be even and at least 8
		} Config;

		typedef struct {
			float pm *	coeff;
		} Param;

		typedef struct {
		} Status;

		//----------------------------------------
		Fir(
			Config*			config_,
			Param*			param_,
			Status*			/*status_*/
		);

		static bool Configure(
			Config*			config_,
			uint32_t		taps_
		)
		{
			if (NullCheck(config_) == false) {
				return false;
			}
			if ((taps_<8) || (taps_&0x1)!=0) {
				return false;
			}

			config_->taps = taps_;

			return true;
		}

		static bool Attach(
			Param*		param_,
			float pm *	coeff_
		)
		{
			if (NullCheck(param_, coeff_) == false) {
				return false;
			}

			param_->coeff	= coeff_;

			return true;
		}

		bool Ready();

		bool Execute(
			const float*	ibuf_,
			float*			obuf_
		);

		bool Task();

	protected:
		//----------------------------------------
		Param*			m_param;

		uint32_t		m_taps;
		float*			m_buf;
		uint32_t		m_pt;
};

//********************************************************************************
}	// namespace dsp_lib
