/*
 *	microphonearray.h
 */

//********************************************************************************
#pragma once

//********************************************************************************
#include <stdint.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/misc/null_check.h>
#include "bf.h"
#include "doa.h"

//********************************************************************************
namespace dsp_lib {

//********************************************************************************
class Microphonearray : public DspModule {
	public:
		//----------------------------------------
		enum MICROPHONEARRAY_BF_UPDATE_MODE {
			MICROPHONEARRAY_BF_UPDATE_MODE_MANUAL = 0,
			MICROPHONEARRAY_BF_UPDATE_MODE_SEMIAUTO,
			MICROPHONEARRAY_BF_UPDATE_MODE_AUTO
		};

		//----------------------------------------
		typedef struct {
			Doa::Config*	doa;
			Bf::Config*		bf;
		} Config;

		typedef struct {
			uint32_t	update_mode;						//!<
			float		dir;								//!< direction [deg] for manual ctrl
			float		dis;								//!< distance [m]    for manual ctrl
			Doa::Param	doa;
			Bf::Param	bf;
		} Param;

		typedef struct {
			Doa::Status	doa;
			Bf::Status	bf;
		} Status;

		//----------------------------------------
		Microphonearray(
			Config*			config_,
			Param*			param_,
			Status*			status_
		);

		static bool Configure(
			Config*			config_,
			Doa::Config*	doa_,
			Bf::Config*		bf_
		)
		{
			if (NullCheck(config_, doa_, bf_) == false) {
				return false;
			}

			config_->doa = doa_;
			config_->bf= bf_;

			return true;
		}

		static bool Attach()
		{
			return true;
		}

		bool Ready();

		bool Task();

		bool Execute(
			const float*	input_,
			float*			output_
		);

	protected:
		//----------------------------------------
		typedef struct {
			Doa*		doa;
			Bf*			bf;
		} Data;

		//----------------------------------------
		bool		m_ready;

		Param*		m_param;
		Status*		m_status;

		Data		m_data;
};

//********************************************************************************
}	// namespace dsp_lib
