#pragma once
#include <string>
#include <picojson.h>

using namespace std;

bool
object_member_check(
	picojson::object obj_,
	string member_name_,
	int member_type_
);

template<class TYPE>
void
disp_value(TYPE &json_value_) {
	std::cout << static_cast<picojson::value>(json_value_).serialize(true) << std::endl;
}

template<class FLOATING_TYPE>
picojson::value
get_number_value(FLOATING_TYPE data_)
{
	picojson::value number_value;
	if (isnan(data_) == true) {
		number_value = static_cast<picojson::value>(string("NaN"));
	}
	else if (isinf(data_) == true) {
		number_value = static_cast<picojson::value>(string("Infinity"));
	}
	else {
		number_value = static_cast<picojson::value>(static_cast<double>(data_));
	}

	return number_value;
}

void
erase_number_value(
	picojson::object	&obj_,
	string				member_name_
);

bool
check_number_value(
	picojson::object obj_,
	string member_name_
);

template<class TYPE>
bool
get_object_member(
	picojson::object obj_,
	string member_name_,
	TYPE &return_member_
)
{
	// メンバの存在を確認
	if (obj_.count(member_name_) != 1) {
		cout << "object must have \"" << member_name_ << "\" !" << endl;
		return false;
	}

	// メンバの型を確認
	if (obj_[member_name_].is<TYPE>() == false) {
		cout << member_name_ + ": invalid type!" << endl;
		return false;
	}
	return_member_ = obj_[member_name_].get<TYPE>();

	return true;
}

// ------ ここからテンプレート特殊化 ------
// 特殊化はinlineにしないと複数ファイルからincludeしたとき定義が重複する
template<>
inline bool
get_object_member(
	picojson::object obj_,
	string member_name_,
	picojson::null &return_member_
)
{
	// メンバの存在を確認
	if (obj_.count(member_name_) != 1) {
		cout << "object must have \"" << member_name_ << "\" !" << endl;
		return false;
	}

	// メンバの型を確認
	if (obj_[member_name_].is<picojson::null>() == false) {
		cout << member_name_ + ": invalid type!" << endl;
		return false;
	}

	//nullなのでreturn_member_をそのまま返す
	return true;
}

// 数値型はdoubleのみなので、テンプレートの特殊化で対応する
template<>
inline bool
get_object_member(
	picojson::object obj_,
	string member_name_,
	float &return_member_
)
{
	if (check_number_value(obj_, member_name_) == false) {
		return false;
	}
	return_member_ = static_cast<float>(obj_[member_name_].get<double>());
	return true;
}

template<>
inline bool
get_object_member(
	picojson::object obj_,
	string member_name_,
	int32_t &return_member_
)
{
	if (check_number_value(obj_, member_name_) == false) {
		return false;
	}
	return_member_ = static_cast<int32_t>(obj_[member_name_].get<double>());
	return true;
}

template<>
inline bool
get_object_member(
	picojson::object obj_,
	string member_name_,
	uint32_t &return_member_
)
{
	if (check_number_value(obj_, member_name_) == false) {
		return false;
	}
	return_member_ = static_cast<uint32_t>(obj_[member_name_].get<double>());
	return true;
}

// ------ テンプレート特殊化ここまで ------


template<class TYPE>
bool
set_object_member(
	picojson::object &obj_,
	string member_name_,
	TYPE member_value_
)
{
	// メンバが存在していれば削除
	if (obj_.count(member_name_) == 1) {
		obj_.erase(member_name_);
	}

	// メンバを書き込み
	obj_[member_name_] = static_cast<picojson::value>(member_value_);

	return true;
}

// ------ ここからテンプレート特殊化 ------
template<>
inline bool
set_object_member(
	picojson::object &obj_,
	string member_name_,
	float member_value_
)
{
	erase_number_value(obj_, member_name_);

	// メンバを書き込み
	obj_[member_name_] = get_number_value(member_value_);

	return true;
}

template<>
inline bool
set_object_member(
	picojson::object &obj_,
	string member_name_,
	double member_value_
)
{
	erase_number_value(obj_, member_name_);

	// メンバを書き込み
	obj_[member_name_] = get_number_value(member_value_);

	return true;
}

template<>
inline bool
set_object_member(
	picojson::object &obj_,
	string member_name_,
	int32_t member_value_
)
{
	erase_number_value(obj_, member_name_);

	// メンバを書き込み
	obj_[member_name_] = static_cast<picojson::value>(static_cast<double>(member_value_));

	return true;
}

template<>
inline bool
set_object_member(
	picojson::object &obj_,
	string member_name_,
	uint32_t member_value_
)
{
	erase_number_value(obj_, member_name_);

	// メンバを書き込み
	obj_[member_name_] = static_cast<picojson::value>(static_cast<double>(member_value_));

	return true;
}
// ------ テンプレート特殊化ここまで ------
