import sys
import traceback
import os.path
import subprocess as sp
import json
import time
import os
import fcntl
import math
import re   # regular expression
import sys

from enum import IntEnum

#--------------------------------------------------------------------------------------------
# DSP related
DSPdir = "/home/toa/TOA_AM-CF1/DSP"
DSPdir_org = "/home_org/toa/TOA_AM-CF1/DSP"

lists_dspupdater_files = DSPdir + "/DSP_Updater_Files.json"
dsp_update_exe = "/home/toa/TOA_AM-CF1/tools/Dspupdater/dspupdater" #DSPアップデート実行ファイル

dsp_update_log = [] #dspアップデートログ
dsp_update_log_path = "/home/toa/dsp_update.log" #dspアップデートログ出力先
dsp_update_log_path_org = "/home_org/toa/dsp_update.log" #dspアップデートログ出力先

UPDATE_FAILED_flag = "/home/toa/TOA_AM-CF1/FAIL_UPDATE_FLAG"
UPDATE_FAILED_flag_org = "/home_org/toa/TOA_AM-CF1/FAIL_UPDATE_FLAG"

MAX_VER_DIGIT_NUM = 4   # max digit number of version

# read lists_dspupdater_files and return its contents strings
# TODO: To tell the truth, we should use '.value' to get the value of DspUpdFiles.BIN.value
#    However, if we use as index number such as test[DspUpdFiles.BIN], the value (0) is used.
#    So we do not change the codes.
class DspUpdFiles(IntEnum):
    BIN = 0
    SUM = 1
    VER = 2

# Update success/failed status to show alert on browser
class DspUpdSts(IntEnum):
    SUCCESS = 0
    FAILED = 1

# DSP_Updater_Files.json's id number
OBJ_VER_TMP = 0
OBJ_VER_FINAL = 1

# for JudgeDspVerUpgrade
UPGRADE = 1
EQUAL = 0
DOWNGRADE_ELSE = -1

# -----------------------------------------
class STATUS_UPDATE(IntEnum):
    STATUS_NORMAL_BOOT          = 0
    STATUS_SUCCESS_UPDATE       = 1
    STATUS_FAILURE_CPU_UPDATE   = 2
    STATUS_FAILURE_DSP_UPDATE   = 3
    STATUS_DSP_1_0_0_0          = 4
    STATUS_UPDATE_START         = 5
 
UPDATE_STATUS_FILE = '/home/toa/update_status'
UPDATE_STATUS_FILE_ORG = '/home_org/toa/update_status'

 # update status
 # status_num: STATUS_UPDATE(Enum)
def Update_Status_Rewrite_int(status_num):
    try:
        with open(UPDATE_STATUS_FILE,'w') as f:
            f.write(str(status_num))    # status_num is STATUS_UPDATE(Enum)
        os.chmod(UPDATE_STATUS_FILE, 0o777)
        MountSyncCmmd_toRO("cp -a " + UPDATE_STATUS_FILE + " " + UPDATE_STATUS_FILE_ORG)
    except:
        DspUpdateLog("update status rewrite failed", True)
        DspUpdateLog(traceback.format_exc(), True)

def Update_Status_Rewrite(status_num):
    Update_Status_Rewrite_int(status_num.value)


def Update_Status_Read():
    try:
        sts_file = None
        with open(UPDATE_STATUS_FILE,'r') as f:
            sts_file = f.read()
        return int(sts_file)
    except:
        DspUpdateLog("update status read failed", True)
        DspUpdateLog(traceback.format_exc(), True)
        return -1
#--------------------------------------------------------------------------------------------

# when want to write ROM data with command, use this to remount,rw, sync and then remount
# Test: OK (mount, home_org) 2019/06/21 sato
def MountSyncCmmd_toRO(command):
    sp.call("mount -o remount,rw /", shell=True)
    sp.call(command, shell=True) 
    os.sync()
    sp.call("mount -o remount,ro /", shell=True)

# Test: OK (mount, home_org) 2019/06/21 sato
def MountSyncCmmd_toRW(command):
    sp.call("mount -o remount,rw /", shell=True)
    sp.call(command, shell=True) 
    os.sync()

# update failed flag making
# Test: OK (mount, home_org) 2019/06/21 sato
def FlagUpDspUpdateFailed():
    sp.call("touch " + UPDATE_FAILED_flag, shell=True)
    MountSyncCmmd_toRO("touch " + UPDATE_FAILED_flag_org)
    
# Test: OK (version can be obtained, if remove dsp_version.json before open None will be printed) 2019/06/21 sato
def GetDspVersion():
    ret = None
    try:
        #DSPバージョン取得、一時ファイル保存
        dsp_version_fname = "/home/toa/TOA_AM-CF1/Application/data/config/dsp_version.json"
        dsp_version_getter = "/home/toa/TOA_AM-CF1/dspversion/get_dspversion"
        print("exec: " + dsp_version_getter + " " + dsp_version_fname)
        sp.call(dsp_version_getter + " " + dsp_version_fname, shell=True)

        # ローカルJSONファイルの読み込み
        with open(dsp_version_fname, 'r') as f:
            DSP_version = json.load(f)
            print(DSP_version)
            #ret = DSP_version['dsp_version']
            ret = DSP_version.get('dsp_version')
    except:
        ret = None

    return ret

# get new dsp version
# Test: OK (version can be obtained, if remove dsp_version.json before open None will be printed) 2019/06/21 sato
def GetNewDspVersion(new_ver_file):
    ret = None

    try:
        with open(new_ver_file,'r') as f:
            new_dsp_ver = json.load(f)
            ret = new_dsp_ver.get('dsp_version')
    except:
        DspUpdateLog("failed to read " + new_ver_file, True)
        ret = None
    
    return ret

# Test: OK (if something happens in try, then led will blink)
def DspUpdateError(message):
    try:
        DspUpdateLog('DSP Update Error occured.' + message,True)

        sp.call("rm -rf " + DSPdir, shell=True) #関連フォルダごと消す。DSPアップデートしない状態となる。(戻り値チェック未実装)
        MountSyncCmmd_toRO("rm -rf " + DSPdir_org)

        Update_Status_Rewrite(STATUS_UPDATE.STATUS_FAILURE_DSP_UPDATE)

        print("DspUpdateError() before reboot\n")
        sp.call("reboot", shell=True)

    except:
        #消したはずのフォルダがある。フォルダ消えない問題の際に突入。無限再起動防止。LED赤点滅。
        print("test")
        while (1):
            sp.call("i2cset -y 1 0x3a 0xff", shell=True)
            time.sleep(1)
            sp.call("i2cset -y 1 0x3a 0xbf", shell=True)
            time.sleep(1)


# Test: OK (dsp_update.log can be saved) 2019/06/21 sato
def DspUpdateLog(_log_string,dump_flg):
    #引数1:ログに書き込む変数または文字列 (リストでも良い)
    #引数2:ファイル出力するかしないか。Trueなら出力する。
    try:
        global dsp_update_log
        log_string = str(_log_string)

        dsp_update_log.append(log_string)
        print(log_string)

        #ログの権限は777に。次回アップデート開始時に削除する。
        if dump_flg == True:
            with open(dsp_update_log_path, 'a') as f:
                for i in range(len(dsp_update_log)):
                    f.write('\n')
                    f.write(dsp_update_log[i])
            dsp_update_log = [] #ファイルに書き終わったら空にする。
            sp.call("chmod 777 " + dsp_update_log_path, shell=True) #権限777
            MountSyncCmmd_toRO("cp -a " + dsp_update_log_path + " " + dsp_update_log_path_org)
    except:
        traceback.print_exc()
# -----------------------------------------

def ReadDspUpdaterFiles(object_version, list_binsumver):
    ret = False
    try:
        if os.path.exists(lists_dspupdater_files):
            with open(lists_dspupdater_files, 'r') as f:
                dsp_updater_files = json.load(f)

            list_binsumver[DspUpdFiles.BIN] = dsp_updater_files[object_version]['bin'] #binary file
            list_binsumver[DspUpdFiles.SUM] = dsp_updater_files[object_version]['sum'] #Check sum
            list_binsumver[DspUpdFiles.VER] = dsp_updater_files[object_version]['ver'] #version
            print(list_binsumver)
            ret = True

        else:
            DspUpdateLog(lists_dspupdater_files + " is not found",True)
            ret = False
    except:
        DspUpdateLog("failed to read " + lists_dspupdater_files, True)
        ret = False

    return ret

# check dsp updater files exist or not
def ChkDspUpdaterFiles(list_binsumver):
    if  os.path.exists(list_binsumver[DspUpdFiles.BIN]) and \
        os.path.exists(list_binsumver[DspUpdFiles.SUM]) and \
        os.path.exists(list_binsumver[DspUpdFiles.VER]):
        return True
    else:
        return False

# Update Execution
def DspUpdateExec(dsp_version, object_version):
    dsp_update_flag = False
    list_binsumver = [0]*3

    if ReadDspUpdaterFiles(object_version, list_binsumver) == True:
        #アップデータファイルリストより取得したbin,sum,verがあるかどうか
        if ChkDspUpdaterFiles(list_binsumver) == True:
            print("dsp_update_bin dsp_update_sum dsp_update_ver exist") #DEBUG DSP_UP
            #アップデート先のdspバージョンファイルを読む
            new_dsp_ver = GetNewDspVersion(list_binsumver[DspUpdFiles.VER])
            if new_dsp_ver is None:
                return False
            DspUpdateLog('ver.' + dsp_version + ' -> ' + 'ver.' + new_dsp_ver, True)

            cmd = "sudo " + dsp_update_exe + " " + list_binsumver[DspUpdFiles.BIN] + " " + list_binsumver[DspUpdFiles.SUM]
            updateRESULT = sp.call(cmd, shell=True) #DSP Update
            if updateRESULT != 0:
                DspUpdateLog('DSP Update Failed.',True)
                dsp_update_flag = False

            else:
                DspUpdateLog('DSP Update Success.',True)
                dsp_update_flag = True
                    
        else:
            #ファイル欠損があった場合はアップデートせず
            DspUpdateLog("dsp_update_bin dsp_update_sum dsp_update_ver does not exist",True)
            dsp_update_flag = False
    else:
        DspUpdateLog("failed to read " + lists_dspupdater_files,True)
        dsp_update_flag = False

    return dsp_update_flag

# finalization of dsp update
# if parameter "version" is 1.0.0.0 (not None), update status is already changed to STATUS_DSP_1_0_0_0
def DspUpdateEnd(status, version = None):
    DspUpdateLog("DspUpdateEnd. remove DSP directory",True)
    sp.call("rm -rf " + DSPdir, shell=True) #at first, Popen was used.
    MountSyncCmmd_toRO("rm -rf " + DSPdir_org)

    # Update success flag to success_flag
    if status == DspUpdSts.SUCCESS:
        DspUpdateLog("status: success. reboot",True)
        Update_Status_Rewrite(STATUS_UPDATE.STATUS_SUCCESS_UPDATE)
    else:
        DspUpdateLog("status: failed. reboot",True)
        if version is None:
            # if version is 1.0.0.0, update status was written to STATUS_DSP_1_0_0_0
            Update_Status_Rewrite(STATUS_UPDATE.STATUS_FAILURE_DSP_UPDATE)
        
    sp.call("reboot", shell=True)


def StartupDsp():
    DspUpdateLog("run startup.out",True)
    sp.run(['./startup.out','/home/toa/TOA_AM-CF1/tmp/block_cf1_init.json','netlist.bin'])
    DspUpdateLog("done startup.out",True)
    
# same as TOA_AM-CF1/Driver/STARTUP
def ResetDsp():
    DspUpdateLog("DSP Reset",True)
    sp.call('gpio -g mode 7 out', shell=True)
    sp.call('gpio -g write 7 0', shell=True)
    sp.call('sleep 1', shell=True)
    sp.call('gpio -g write 7 1', shell=True)

    sp.call('sleep 5', shell=True)  # if sleep 1, spi packet ack error has occured
    DspUpdateLog("DSP Reset finished",True)

# Dsp version upgrade or not
# return 0: same version, > 0: dsp_ver < new_dsp_ver, -1: else
def JudgeDspVerUpgrade(dsp_version, new_dsp_ver, max_ver_digit_num):
    ret = DOWNGRADE_ELSE

    # split with non digit number (expect like 10.0.105.30 (can handle 10.0.104.30_test format))
    dsp_ver_arr = re.split(r'[^0-9]+', dsp_version, max_ver_digit_num)
    new_dsp_ver_arr = re.split(r'[^0-9]+', new_dsp_ver, max_ver_digit_num)
    #print("dsp_ver_arr: ")
    #print(dsp_ver_arr)
    #print("new_dsp_ver_arr: ")
    #print(new_dsp_ver_arr)

    # length of ver array should be equal to or higher than max_ver_digit_num
    if len(dsp_ver_arr) >= max_ver_digit_num and len(new_dsp_ver_arr) >= max_ver_digit_num:
        # judge from the left digit to right digit
        for i in range(max_ver_digit_num):
            if int(dsp_ver_arr[i]) < int(new_dsp_ver_arr[i]):
                ret = UPGRADE
                break
            elif int(dsp_ver_arr[i]) > int(new_dsp_ver_arr[i]):
                ret = DOWNGRADE_ELSE
                break
            else:
                ret = EQUAL
    else:
        ret = DOWNGRADE_ELSE

    return ret


def stacktrace():
    t, v, tb = sys.exc_info()
    stacks = traceback.format_exception(t,v,tb)
    for stack in stacks:
        DspUpdateLog (str(stack),True)

# DSP_Update() is too complicated and has many dependencies, 
# so I made a new function.     Sato
def DspUpdate():
    try:
        dsp_version = GetDspVersion()
        if dsp_version is None:
            DspUpdateLog("dsp_version is None",True)
            DspUpdateError("1")
        else:
            DspUpdateLog("dsp_version: " + dsp_version,True)

            # Compare present dsp version and new dsp version
            list_binsumver = [0]*3
            flg = ReadDspUpdaterFiles(OBJ_VER_FINAL, list_binsumver)
            if flg:
                new_dsp_ver = GetNewDspVersion(list_binsumver[DspUpdFiles.VER])
                if new_dsp_ver is None:
                    DspUpdateLog("dsp_version is None",True)
                    DspUpdateError("11")

                judgever = JudgeDspVerUpgrade(dsp_version, new_dsp_ver, MAX_VER_DIGIT_NUM)
                if judgever == EQUAL:
                    DspUpdateLog("New dsp version is the same as present one",True)
                    DspUpdateEnd(DspUpdSts.SUCCESS)
                elif judgever == DOWNGRADE_ELSE:
                    DspUpdateLog("New dsp version " + new_dsp_ver + " is lower than present one, or others",True)
                    DspUpdateEnd(DspUpdSts.SUCCESS)
                else:
                    DspUpdateLog("New dsp version is " + new_dsp_ver,True)
            else:
                DspUpdateLog("New dsp version read error",True)
                DspUpdateError("2")
                    
            # new dsp version is different from present one
            if dsp_version == "0.1.0.0":
                result = DspUpdateExec(dsp_version, OBJ_VER_FINAL)    # update to object version
                if result == False:
                    DspUpdateLog("DspUpdateExec(dsp_version, OBJ_VER_FINAL) failed",True)
                    DspUpdateError("3")
                else:
                    DspUpdateEnd(DspUpdSts.SUCCESS)

            elif dsp_version == "1.0.0.0":
                Update_Status_Rewrite(STATUS_UPDATE.STATUS_DSP_1_0_0_0)
                DspUpdateEnd(DspUpdSts.FAILED, "1.0.0.0")

            elif dsp_version == "1.1.0.0":
                list_binsumver_tmp = [0]*3
                flg2 = ReadDspUpdaterFiles(OBJ_VER_TMP, list_binsumver_tmp)
                # lists_dspupdater_files can be read and bin, sum, ver of tmp and not_tmp are available
                if flg and flg2  and ChkDspUpdaterFiles(list_binsumver_tmp) and ChkDspUpdaterFiles(list_binsumver):
                    StartupDsp()
                    result = DspUpdateExec(dsp_version, OBJ_VER_TMP)   # update to tmp version
                    if result == False:
                        DspUpdateLog("DspUpdateExec(dsp_version, OBJ_VER_TMP) failed",True)
                        DspUpdateError("4")
                    else:
                        ResetDsp()  # same as STARTUP's dsp reset
                        dsp_version = GetDspVersion() 
                        if dsp_version is None:
                            DspUpdateLog("dsp_version is None",True)
                            DspUpdateError("5")
                        else:
                            result = DspUpdateExec(dsp_version, OBJ_VER_FINAL)
                            if result == False:
                                DspUpdateLog("DspUpdateExec(dsp_version, OBJ_VER_FINAL) failed",True)
                                DspUpdateError("6")
                            else:
                                DspUpdateEnd(DspUpdSts.SUCCESS)
                else:
                    DspUpdateLog("lists_dspupdater_files cannot be read or bin, sum, ver of tmp and not_tmp are unavailable",True)
                    DspUpdateError("7")

            elif dsp_version == "0.0.0.0":
                Update_Status_Rewrite(STATUS_UPDATE.STATUS_DSP_1_0_0_0)
                DspUpdateEnd(DspUpdSts.FAILED, "1.0.0.0")

            else:
                DspUpdateLog("dsp_version: else",True)
                StartupDsp()
                result = DspUpdateExec(dsp_version, OBJ_VER_FINAL)    # update to object version
                if result == False:
                    DspUpdateLog("DspUpdateExec(dsp_version, OBJ_VER_FINAL) failed",True)
                    DspUpdateError("8")
                else:
                    DspUpdateEnd(DspUpdSts.SUCCESS)
    except:
        stacktrace()
        DspUpdateError("9")


# Is_DSP_Update() is too complicated and has many dependencies, 
# so I made a new function.     Sato
def IsDspUpdate():
    dsp_update_flag = False

    if os.path.exists(DSPdir):
        dsp_update_flag = True
    else:
        dsp_update_flag = False
    
    # save dsp version json file
    dsp_version = GetDspVersion() #ライブラリからDSPバージョンを取得し、ファイル出力
    print(dsp_version)

    return dsp_update_flag

if __name__ == '__main__':
    # Test of JudgeDspVerUpgrade
    print("Test of JudgeDspVerUpgrade")
    testdata = [
        [ "10.3.3.0", "1.3.3.0", 4, DOWNGRADE_ELSE, "downgrade test: " ],
        [ "10.3.3.0", "2.3.3.0", 4, DOWNGRADE_ELSE, "downgrade test: " ],
        [ "1.30.3.0", "1.4.2.0", 4, DOWNGRADE_ELSE, "downgrade test: " ],
        [ "1.5.30.0", "1.5.4.0", 4, DOWNGRADE_ELSE, "downgrade test: " ],
        [ "1.5.2.200", "1.5.2.3", 4, DOWNGRADE_ELSE, "downgrade test: " ],
        [ "1.3.3.0", "1.3.3.0", 4, EQUAL, "equal test: " ],
        [ "10.30.300.40", "10.30.300.40", 4, EQUAL, "equal test: " ],
        [ "1.3.3.0", "10.3.3.0", 4, UPGRADE, "upgrade test: " ],
        [ "2.3.3.0", "10.3.3.0", 4, UPGRADE, "upgrade test: " ],
        [ "1.4.2.0", "1.30.3.0", 4, UPGRADE, "upgrade test: " ],
        [ "1.5.4.0", "1.5.30.0", 4, UPGRADE, "upgrade test: " ],
        [ "1.5.2.3", "1.5.2.200", 4, UPGRADE, "upgrade test: " ],
        [ "1.500.200.300", "500.30.3.200", 4, UPGRADE, "upgrade test: " ],
        [ "1.3.4.0_test", "1.3.20.0_fd2", 4, UPGRADE, "include _ test: " ],
        [ "1.3.0", "1.3.0", 4, DOWNGRADE_ELSE, "fewer digit num test: " ],
    ]
    for data in testdata:
        ret = JudgeDspVerUpgrade( data[0], data[1], data[2] )
        if ret == data[3]:
            print (data[4]+"OK")
        else:
            print (data[4]+"NG")

    #MountSyncCmmd_toRO("touch /home_org/toa/testasds")
    #MountSyncCmmd_toRW("touch /home_org/toa/dsfrw")
    #FlagUpDspUpdateFailed()
    print(GetDspVersion())
    print(GetNewDspVersion("/home/toa/TOA_AM-CF1/Application/data/config/dsp_version.json"))
    print("tet")
    list_binsumver = [0]*3
    ReadDspUpdaterFiles(OBJ_VER_TMP, list_binsumver)
