//======================================================================
//
//	fdesign.cpp
//
//
//======================================================================
#include <dsp_lib/misc/biquad/fdesign.h>
#include <dsp_lib/misc/biquad/fdesign_prototypes.h>
#include <dsp_lib/misc/sizeof_array.h>

//======================================================================
namespace dsp_lib {

//======================================================================
#define	COEFF_FREQUENCY_SCALE			(0.0001f)
#define	COEFF_FREQUENCY_1HZ				(10000)
#define	COEFF_GAIN_SCALE				(0.1f)

//======================================================================
#define	COEFF_FILTER_ALPHA				(1.0f)
#define	COEFF_XOVER_VQ_ALPHA			COEFF_FILTER_ALPHA

/*--------------------------------------------------------------------------------*/
#define	COEFF_XOVER_LPF_BS12_ALPHA		(1.2742f)
#define	COEFF_XOVER_HPF_BS12_ALPHA		(1.0f / COEFF_XOVER_LPF_BS12_ALPHA)

/*--------------------------------------------------------------------------------*/
#define	COEFF_XOVER_LPF_BS18_ALPHA1		(1.32475f)
#define	COEFF_XOVER_LPF_BS18_ALPHA2		(1.44993f)

#define	COEFF_XOVER_HPF_BS18_ALPHA1		(1.0f / COEFF_XOVER_LPF_BS18_ALPHA1)
#define	COEFF_XOVER_HPF_BS18_ALPHA2		(1.0f / COEFF_XOVER_LPF_BS18_ALPHA2)

/*--------------------------------------------------------------------------------*/
#define	COEFF_XOVER_LPF_BS24_ALPHA1		(1.43241f)
#define	COEFF_XOVER_LPF_BS24_ALPHA2		(1.60594f)

#define	COEFF_XOVER_HPF_BS24_ALPHA1		(1.0f / COEFF_XOVER_LPF_BS24_ALPHA1)
#define	COEFF_XOVER_HPF_BS24_ALPHA2		(1.0f / COEFF_XOVER_LPF_BS24_ALPHA2)

//======================================================================
const float
m_lpf_alpha_table[] = {
	COEFF_FILTER_ALPHA,				// 0
	COEFF_XOVER_VQ_ALPHA,			// 1
	COEFF_XOVER_LPF_BS12_ALPHA,		// 2
	COEFF_XOVER_LPF_BS18_ALPHA1,	// 3
	COEFF_XOVER_LPF_BS24_ALPHA1,	// 4
	COEFF_XOVER_LPF_BS18_ALPHA2,	// 5
	COEFF_XOVER_LPF_BS24_ALPHA2,	// 6
};

const int	m_lpf_alpha_table_size = SIZEOF_ARRAY(m_lpf_alpha_table);

//---------------------------------------------------------------------
const float
m_hpf_alpha_table[] = {
	COEFF_FILTER_ALPHA,				// 0
	COEFF_XOVER_VQ_ALPHA,			// 1
	COEFF_XOVER_HPF_BS12_ALPHA,		// 2
	COEFF_XOVER_HPF_BS18_ALPHA1,	// 3
	COEFF_XOVER_HPF_BS24_ALPHA1,	// 4
	COEFF_XOVER_HPF_BS18_ALPHA2,	// 5
	COEFF_XOVER_HPF_BS24_ALPHA2,	// 6
};
const int	m_hpf_alpha_table_size = SIZEOF_ARRAY(m_hpf_alpha_table);

//======================================================================
void
GetLPFAlpha(
	const int32_t		alphaindex_,
	float			&alpha_
)
{
	int	no	= alphaindex_;

	if (no < 0) {
		no = 0;
	}
	if (m_lpf_alpha_table_size <= no) {
		no = m_lpf_alpha_table_size-1;
	}
	alpha_ = m_lpf_alpha_table[no];
}

void
GetHPFAlpha(
	const int32_t		alphaindex_,
	float			&alpha_
)
{
	int	no	= alphaindex_;

	if (no < 0) {
		no = 0;
	}
	if (m_hpf_alpha_table_size <= no) {
		no = m_hpf_alpha_table_size-1;
	}
	alpha_ = m_hpf_alpha_table[no];
}

#if 1
void
GetHonEQGainNo(
	const int32_t		gain_,
	int				&gain_no_
)
{
	gain_no_ = gain_/5;
}
#endif

void
Fdesign(
	int32_t	filtertype_,
	float	f,
	float	gain,
	float	q,
	int32_t	alphaindex_,
	int32_t	eqon_,
	float	*coeff_
)
{
	//BOOST_ASSERT(coeff_);

	float	alpha;
	int		bypass;
	int  no ,gain_;
	
	gain_ = static_cast<int>(gain);

	bypass	= !eqon_;


	switch (filtertype_) {
	case FILTER_TYPE_PEQ :
		Fdesign_PEQ(f, q, gain, bypass, coeff_);
		break;

	case FILTER_TYPE_HPF6 :
		GetHPFAlpha(alphaindex_, alpha);
		Fdesign_XOverHPF6(f, alpha, bypass, coeff_);
		break;
#if 1
	case FILTER_TYPE_HPF12 :
		GetHPFAlpha(alphaindex_, alpha);
		Fdesign_XOverHPF12(f, q, alpha, bypass, coeff_);
		break;

	case FILTER_TYPE_LPF6 :
		GetLPFAlpha(alphaindex_, alpha);
		Fdesign_XOverLPF6(f, alpha, bypass, coeff_);
		break;
	case FILTER_TYPE_LPF12 :
		GetLPFAlpha(alphaindex_, alpha);
		Fdesign_XOverLPF12(f, q, alpha, bypass, coeff_);
		break;

	case FILTER_TYPE_HIGH_SHELVING :
		Fdesign_HighShelving(f, gain, bypass, coeff_);
		break;
		
	case FILTER_TYPE_LOW_SHELVING :
		Fdesign_LowShelving(f, gain, bypass, coeff_);
		break;

	case FILTER_TYPE_NOTCH :
		Fdesign_NotchFilter(f, q, bypass, coeff_);
		break;

	case FILTER_TYPE_ALL_PASS :
		Fdesign_APF(f, q, bypass, coeff_);
		break;

	case FILTER_TYPE_HORN_EQ :
		GetHonEQGainNo(gain_, no);
		Fdesign_HornEQ(no, bypass, coeff_);
		break;
#endif
	case FILTER_TYPE_THROUGH :
	default :
		Fdesign_Through(coeff_);
		break;
	}
}

} // namespace dsp_lib
//======================================================================
