/************************************************************************
 *
 *	coeff_bpf.c
 *
 *	20021225	Tsuyoshi Ogawa
 *
 ************************************************************************/
#include "fdesign_prototypes.h"
#include "fdesign_const.h"


#include <cmath>

#ifndef	NULL
#include <stdio.h>
#endif

namespace dsp_lib {

/*============================================================*/
void
Fdesign_BPF_CheckParameter(
	float*	f_,
	float*	q_
)
{
	Fdesign_PEQ_CheckParameter(f_, q_, NULL);
}

/*============================================================*/
void
Fdesign_BPF(
	float	f_,
	float	q_,
	int		bypass_,
	float*	coeff_
)
{
	/*------------------------------------------------------------*/
	float	f		= f_;
	float	q		= q_;

	float	pi		= M_PI;		/* 円周率 */
	float	fs		= M_FS;		/* サンプリング周波数 */


	float	b0, b1, b2, a1, a2;	/* 係数 */

	float	bb;					/* 分母 */
	float	tx, txtx, tq, k;
	float	var_dm;

	/*------------------------------------------------------------*/
	/* 1. 初期設定,入力値の整合性チェック, 制限値セット */
	Fdesign_BPF_CheckParameter(&f, &q);

	/*------------------------------------------------------------*/
	if (bypass_) {
		Fdesign_Through(coeff_);
	}
	else {
		/*------------------------------------------------------------*/
		/* 2. 共通項の計算 */
		var_dm		= pi * f / fs ;

		k		= 1.0f;
		tx		= tanf(var_dm) ;
		tq		= tx / q;
		txtx	= tx*tx;

		/*------------------------------------------------------------*/
		/* 3. 係数算出 */
		var_dm	= k / q * tx;
		bb	= 1.0f + tq + txtx;

		b0	= var_dm					/ bb;
		b1	= 0.0f;
		b2	= -b0;
		a1	= 2.0f * (1.0f - txtx)	/ bb;;
		a2	= (-1.0f + tq - txtx)	/ bb;

		/*------------------------------------------------------------*/
		if (coeff_) {
			coeff_[0] = b0;
			coeff_[1] = b1;
			coeff_[2] = b2;
			coeff_[3] = -a1;
			coeff_[4] = -a2;
		}
		/*------------------------------------------------------------*/
	}
	/*------------------------------------------------------------*/
}

/*============================================================*/
} // namespace dsp_lib
