/************************************************************************
 *
 *	coeff_horneq.c
 *
 *	20020920	Tsuyoshi Ogawa
 *
 ************************************************************************/
#include "fdesign_prototypes.h"
#include "fdesign_const.h"


#include <cmath>
#include <dsp_lib/misc/sizeof_array.h>

namespace dsp_lib {

/*============================================================*/
const float g_coeff_horn_eq_frequency[] = {
	21989.0f,	//  0.5dB
	21248.0f,	//  1.0dB
	20736.0f,	//  1.5dB
	20345.0f,	//  2.0dB
	20034.0f,	//  2.5dB
	19779.0f,	// 	3.0dB
	19566.0f,	// 	3.5dB
	19387.0f,	// 	4.0dB
	19234.0f,	// 	4.5dB
	19103.0f,	// 	5.0dB
	18989.0f,	// 	5.5dB
	18891.0f,	// 	6.0dB
	18806.0f,	// 	6.5dB
	18731.0f,	// 	7.0dB
	18666.0f,	// 	7.5dB
	18608.0f,	// 	8.0dB
	18558.0f,	// 	8.5dB
	18514.0f,	// 	9.0dB
	18474.0f,	// 	9.5dB
	18440.0f,	// 10.0dB
	18409.0f,	// 10.5dB
	18382.0f,	// 11.0dB
	18358.0f,	// 11.5dB
	18337.0f,	// 12.0dB
	18318.0f,	// 12.5dB
	18301.0f,	// 13.0dB
	18287.0f,	// 13.5dB
	18273.0f,	// 14.0dB
	18262.0f,	// 14.5dB
	18251.0f,	// 15.0dB
	18242.0f,	// 15.5dB
	18234.0f,	// 16.0dB
	18226.0f,	// 16.5dB
	18220.0f,	// 17.0dB
	18214.0f,	// 17.5dB
	18209.0f,	// 18.0dB
};
const int	g_number_of_coeff_horn_eq_frequency = SIZEOF_ARRAY(g_coeff_horn_eq_frequency);

/*============================================================*/
void
Fdesign_HornEQ_CheckParameter(
	int*	gain_no_
)
{
	if (gain_no_) {
		if (*gain_no_ < 0) {
			*gain_no_ = 0;
		}
		else if (g_number_of_coeff_horn_eq_frequency < *gain_no_) {
			*gain_no_  = g_number_of_coeff_horn_eq_frequency;
		}
	}
}

/*============================================================*/
void
Fdesign_HornEQ(
	int		gain_no_,
	int		bypass_,
	float*	coeff_
)
{
	/*------------------------------------------------------------*/
	int		gain_no	= gain_no_;

	float	pi		= M_PI;		/* 円周率 */
	float	fs		= M_FS;		/* サンプリング周波数 */

	float	b0, b1, b2, a1, a2;	/* 係数 */

	float	f, b, k;
	float	var_dm;

	float	gain;

	/*------------------------------------------------------------*/
	/* 1. 初期設定,入力値の整合性チェック, 制限値セット */
	Fdesign_HornEQ_CheckParameter(&gain_no);

	/*------------------------------------------------------------*/
	if ((gain_no <= 0) || bypass_) {
		Fdesign_Through(coeff_);
	}
	else {
		/*------------------------------------------------------------*/
		f		= g_coeff_horn_eq_frequency[gain_no - 1];
		gain	= (float)gain_no * 0.5f;

		var_dm		= 2.0f * pi * f / fs;

		b		= (sinf(var_dm) - 1.0f) / cosf(var_dm);
		k		= powf(10.0f, (gain + 1.6f) / 20.0f);


		/*------------------------------------------------------------*/
		/* 3. 係数算出 */
		b0	= (1.0f + b + (1.0f - b) * k) * 0.5f;
		b1	= (1.0f + b - (1.0f - b) * k) * 0.5f;
		b2	= 0.0f;
		a1	= -b;
		a2	= 0.0f;

		/*------------------------------------------------------------*/
		if (coeff_) {
			coeff_[0] = b0;
			coeff_[1] = b1;
			coeff_[2] = b2;
			coeff_[3] = -a1;
			coeff_[4] = -a2;
		}
		/*------------------------------------------------------------*/
	}
}

/*============================================================*/
} // namespace dsp_lib
