/************************************************************************
 *
 *	fdesign_xover_hpf6.c
 *
 ************************************************************************/
#include <cmath>
#include <dsp_lib/misc/biquad/fdesign_const.h>
#include <dsp_lib/misc/biquad/fdesign_prototypes.h>

//======================================================================
namespace dsp_lib {

/*============================================================*/
void
Fdesign_XOverHPF6_CheckParameter(
	float	*f_,
	float	*alpha_
)
{
	Fdesign_XOverLPF6_CheckParameter(f_, alpha_);
}

/*============================================================*/
void
Fdesign_XOverHPF6(
	float	f_,
	float	alpha_,
	int		bypass_,
	float*	coeff_
)
{
	/*------------------------------------------------------------*/
	float	f		= f_;
	float	alpha	= alpha_;

	float	pi		= M_PI;			// 円周率
	float	fs		= M_FS;			// サンプリング周波数

	float	b0, b1, b2, a1, a2;	// 係数

	float	tx;
	float	_dm;

	/*------------------------------------------------------------*/
	/* 1. 初期設定,入力値の整合性チェック, 制限値セット*/
	Fdesign_XOverHPF6_CheckParameter(&f, &alpha);

	/*------------------------------------------------------------*/
	if (bypass_) {
		Fdesign_Through(coeff_);
	}
	else {
		/*------------------------------------------------------------*/
		_dm	= pi * f / fs;
		tx	= tanf(_dm) * alpha;

		b0	= 1.0f / (1.0f + tx);
		b1	= -b0;
		b2	= 0.0f;

		a1	= (1.0f - tx) / (1.0f + tx);
		a2	= 0.0f;

		/*------------------------------------------------------------*/
		if (coeff_) {
			coeff_[0] = b0;
			coeff_[1] = b1;
			coeff_[2] = b2;
			coeff_[3] = -a1;
			coeff_[4] = -a2;
		}
		/*------------------------------------------------------------*/
	}
}

} // namespace dsp_lib
//======================================================================
