/*
 * dsp_config.cpp
 *
 *  Created on: 2016/10/20
 *      Author: toa29548
 */

#include <iostream>
#include <algorithm>
#include <picojson_misc.h>
#include <dsp_lib/include/dsp/dsp_manager.h>
#include <stddef.h>

// TI
//#include <dsp_lib/substitute/TI_OMAPL138_app.h>
//#define AUDIO_BLOCK_SIZE		(384)

// ADI
//#include <dsp_lib/substitute/ADI_ADSP21489_app.h>
//#include <analog_audio.h>

// RaspberryPi
#include <dsp_lib/substitute/RaspberryPi_app.h>
#include <audio.h>
#include "access_sharedram.h"
#include "simulated_malloc.h"

#include <shared/map_no.h>
#include "map.h"
#include "dsp_config.h"
#include "read_input_file.h"
#include <dsp_lib/include/dsp/dsp_manager.h>

using namespace dsp_lib;
using namespace shared;
using namespace std;

namespace dsp_ctrl {

//#pragma DATA_SECTION(".shram_cpu_alloc");
uint32_t			g_net_list_count = 0;

bool
AddNetList(
		uint32_t	type_,
		int32_t		input_,
		int32_t		output_,
		void*		config_,
		void*		param_,
		void*		status_
)
{
	uint32_t no = g_net_list_count;

	if (no >= g_dsp_manager_config->net_list_size) {
		return false;
	}

	if (DspManager::Setup(
			g_dsp_manager_config,
			no,
			type_,
			input_,
			output_,
			config_,
			param_,
			status_) == false) {
		return false;
	}

	g_net_list_count = no + 1;

	return true;
}

void
PrintBlockNumber(uint32_t block_no_)
{
	cout << "block[" << block_no_ << "]: ";
}

void
PrintBlockError(
	picojson::object &block_obj_,
	uint32_t block_no_, 
	string msg_
)
{
	PrintBlockNumber(block_no_);
	cout << msg_ << endl;
	disp_value(block_obj_);
}

void
PrintConfigurationError(
	picojson::object &block_obj_,
	uint32_t block_no_
)
{
	PrintBlockError(block_obj_, block_no_, "configuration failed!");
}

void
PrintAttachError(
	picojson::object &block_obj_,
	uint32_t block_no_
)
{
	PrintBlockError(block_obj_, block_no_, "attach failed!");
}

void
PrintMemoryError(
	picojson::object &block_obj_,
	uint32_t block_no_
)
{
	PrintBlockError(block_obj_, block_no_, "memory error!");
}

bool
ConfigurationDspManager(picojson::array& block_array_)
{
	using namespace std;

	/* コンフィギュレーション初期化 */
	if (DspManager::Configure(
			g_dsp_manager_config,
			g_dsp_manager_net_list,
			g_number_of_netlist,
			NUMBER_OF_EXTERNAL_IO
			) == false ) {
		return false;
	}

	/* コンフィギュレーション設定 */
	int32_t	number_of_io = NUMBER_OF_EXTERNAL_IO;

	for (uint32_t block_no = 0; block_no < g_number_of_netlist; block_no++) {
		if (block_array_[block_no].is<picojson::object>() == false) {
			PrintBlockNumber(block_no);
			cout << "block must be object!" << endl;
			return false;
		}
		picojson::object	block_obj = block_array_[block_no].get<picojson::object>();

		// block_objがtype, input, output, configsをキーとして持っていることを確認
		{
			if (block_obj.count("type") != 1) {
				PrintBlockError(block_obj, block_no, "block object must have type!");
				return false;
			}
			if (block_obj.count("ibuf_no") != 1) {
				PrintBlockError(block_obj, block_no, "block object must have ibuf_no!");
				return false;
			}
			if (block_obj.count("obuf_no") != 1) {
				PrintBlockError(block_obj, block_no, "block object must have obuf_no!");
				return false;
			}
			if (block_obj.count("configs") != 1) {
				PrintBlockError(block_obj, block_no, "block object must have configs!");
				return false;
			}
		}
		// type, input, output, configsの型チェック
		if (block_obj["type"].is<string>() == false) {
			PrintBlockError(block_obj, block_no, "type must be string!");
			return false;
		}
		if (block_obj["ibuf_no"].is<double>() == false) {
			PrintBlockError(block_obj, block_no, "ibuf_no must be number!");
			return false;
		}
		if (block_obj["obuf_no"].is<double>() == false) {
			PrintBlockError(block_obj, block_no, "obuf_no must be number!");
			return false;
		}
		if (block_obj["configs"].is<picojson::object>() == false) {
			PrintBlockError(block_obj, block_no, "configs must be object!");
			return false;
		}

		// ブロック情報の取り込み

		string				type	= block_obj["type"].get<string>();
		int32_t				ibuf_no	= static_cast<uint32_t>(block_obj["ibuf_no"].get<double>());	// jsonの数値はdouble
		int32_t				obuf_no	= static_cast<uint32_t>(block_obj["obuf_no"].get<double>());	// jsonの数値はdouble
		picojson::object	configs	= block_obj["configs"].get<picojson::object>();

		uint32_t	type_id;
		//int32_t	ibuf_no	= 0;
		//int32_t	obuf_no	= 0;
		void*		config	= NULL;
		void*		param	= NULL;
		void*		status	= NULL;

		int32_t		max_used_buf_no;

		if (type == "nop") {
			type_id	= DspManager::DSP_MODULE_ID_NOP;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config = NULL;
			param  = NULL;
			status = NULL;

			if (Nop::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Nop::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}

			max_used_buf_no = 0;

		} else if (type == "through") {
			type_id	= DspManager::DSP_MODULE_ID_THROUGH;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config = NULL;
			param  = NULL;
			status = NULL;

			if (Through::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Through::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}

			max_used_buf_no = std::max(ibuf_no, obuf_no);

		} else if (type == "gain") {
			type_id	= DspManager::DSP_MODULE_ID_GAIN;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config	= NULL;
			param	= MALLOC_SHAREDRAM_DM(Gain::Param, 1);
			status	= NULL;

			if (NullCheck(param) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}
			if (Gain::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Gain::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}

			max_used_buf_no = std::max(ibuf_no, obuf_no);

		} else if (type == "matrix") {
			type_id	= DspManager::DSP_MODULE_ID_MATRIX;

			if (configs.count("number_of_input") != 1) {
				PrintBlockError(block_obj, block_no, "configs must have number_of_input!");
				return false;
			}
			if (configs.count("number_of_output") != 1) {
				PrintBlockError(block_obj, block_no, "configs must have number_of_output!");
				return false;
			}

			if (configs["number_of_input"].is<double>() == false) {
				PrintBlockError(block_obj, block_no, "configs.number_of_input must be number!");
				return false;
			}
			if (configs["number_of_output"].is<double>() == false) {
				PrintBlockError(block_obj, block_no, "configs.number_of_output must be number!");
				return false;
			}

			uint32_t number_of_input	= static_cast<uint32_t>(configs["number_of_input"].get<double>());
			uint32_t number_of_output	= static_cast<uint32_t>(configs["number_of_output"].get<double>());

			float* param_gain_target = MALLOC_SHAREDRAM_DM(float, number_of_input * number_of_output);

			config	= MALLOC_SHAREDRAM_DM(Matrix::Config, 1);
			param	= MALLOC_SHAREDRAM_DM(Matrix::Param, 1);
			status	= NULL;

			if (NullCheck(config, param, param_gain_target) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}
			if (Matrix::Configure(
				reinterpret_cast<Matrix::Config*>(config),
				number_of_input,
				number_of_output
				) == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Matrix::Attach(
				reinterpret_cast<Matrix::Param*>(param),
				GetSharedRamPointer(param_gain_target)
				) == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = std::max(
				ibuf_no + number_of_input - 1,
				obuf_no + number_of_output - 1
				);

		} else if (type == "iir") {
			type_id	= DspManager::DSP_MODULE_ID_IIR;

			if (configs.count("number_of_stage") != 1) {
				PrintBlockError(block_obj, block_no, "configs must have number_of_stage!");
				return false;
			}

			if (configs["number_of_stage"].is<double>() == false) {
				PrintBlockError(block_obj, block_no, "configs.number_of_stage must be number!");
				return false;
			}

			uint32_t number_of_stage	= static_cast<uint32_t>(configs["number_of_stage"].get<double>());

			Iir::IirCoef* pm param_coef = MALLOC_SHAREDRAM_PM(Iir::IirCoef pm, number_of_stage);
			uint32_t* param_coef_update_req = MALLOC_SHAREDRAM_DM(uint32_t, number_of_stage);

			config	= MALLOC_SHAREDRAM_DM(Iir::Config, 1);
			param	= MALLOC_SHAREDRAM_DM(Iir::Param, 1);
			status	= NULL;

			if (NullCheck(config, param, param_coef, param_coef_update_req) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			if (Iir::Configure(
				reinterpret_cast<Iir::Config*>(config),
				number_of_stage
				) == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Iir::Attach(
				reinterpret_cast<Iir::Param*>(param),
				GetSharedRamPointer(param_coef),
				GetSharedRamPointer(param_coef_update_req)
				) == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = std::max(ibuf_no, obuf_no);

		} else if (type == "peak") {
			type_id	= DspManager::DSP_MODULE_ID_PEAK;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config	= NULL;
			param	= NULL;
			status	= MALLOC_SHAREDRAM_DM(Peak::Status, 1);

			if (NullCheck(status) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			if (Peak::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Peak::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = ibuf_no;

		} else if (type == "rms") {
			type_id	= DspManager::DSP_MODULE_ID_RMS;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config	= NULL;
			param	= NULL;
			status	= MALLOC_SHAREDRAM_DM(Rms::Status, 1);

			if (NullCheck(status) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			if (Rms::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Rms::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = ibuf_no;

		} else if (type == "comp") {
			type_id	= DspManager::DSP_MODULE_ID_COMP;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config	= NULL;
			param	= MALLOC_SHAREDRAM_DM(Comp::Param, 1);
			status	= MALLOC_SHAREDRAM_DM(Comp::Status, 1);

			if (NullCheck(param, status) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			if (Comp::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Comp::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = std::max(ibuf_no, obuf_no);

		} else if (type == "gate") {
			type_id	= DspManager::DSP_MODULE_ID_GATE;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config	= NULL;
			param	= MALLOC_SHAREDRAM_DM(Gate::Param, 1);
			status	= MALLOC_SHAREDRAM_DM(Gate::Status, 1);

			if (NullCheck(param, status) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			if (Gate::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Gate::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = std::max(ibuf_no, obuf_no);

		} else if (type == "delay") {
			type_id	= DspManager::DSP_MODULE_ID_DELAY;

			if (configs.count("delay_buf_size") != 1) {
				PrintBlockError(block_obj, block_no, "configs must have delay_buf_size!");
				return false;
			}

			if (configs["delay_buf_size"].is<double>() == false) {
				PrintBlockError(block_obj, block_no, "configs.delay_buf_size must be number!");
				return false;
			}

			uint32_t delay_buf_size	= static_cast<uint32_t>(configs["delay_buf_size"].get<double>());

			config	= MALLOC_SHAREDRAM_DM(Delay::Config, 1);
			param	= MALLOC_SHAREDRAM_DM(Delay::Param, 1);
			status	= NULL;

			if (NullCheck(config, param) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			if (Delay::Configure(
				reinterpret_cast<Delay::Config*>(config), 
				delay_buf_size
				) == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Delay::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = std::max(ibuf_no, obuf_no);

		} else if (type == "aec") {
			type_id	= DspManager::DSP_MODULE_ID_AEC;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config	= NULL;
			param	= MALLOC_SHAREDRAM_DM(Aec::Param, 1);
			status	= MALLOC_SHAREDRAM_DM(Aec::Status, 1);

			if (NullCheck(param) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			if (Aec::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Aec::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = std::max(ibuf_no + 1, obuf_no + 1);

		} else if (type == "nc") {
			type_id	= DspManager::DSP_MODULE_ID_NC;

			if (configs.empty() == false) {
				PrintBlockError(block_obj, block_no, "configs must be empty!");
				return false;
			}

			config	= NULL;
			param	= MALLOC_SHAREDRAM_DM(Nc::Param, 1);
			status	= NULL;

			if (NullCheck(param) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			if (Nc::Configure() == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Nc::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = std::max(ibuf_no, obuf_no);

		} else if (type == "microphonearray") {
			type_id	= DspManager::DSP_MODULE_ID_MICROPHONEARRAY;

			// configsオブジェクトのキーを確認
			if (configs.count("number_of_input") != 1) {
				PrintBlockError(block_obj, block_no, "configs must have number_of_input!");
				return false;
			}
			if (configs.count("sensor_pos_y") != 1) {
				PrintBlockError(block_obj, block_no, "configs must have sensor_pos_y!");
				return false;
			}
			if (configs.count("doa_sensor_ch") != 1) {
				PrintBlockError(block_obj, block_no, "configs must have doa_sensor_ch!");
				return false;
			}
			if (configs.count("bf_subarray_ch") != 1) {
				PrintBlockError(block_obj, block_no, "configs must have bf_subarray_ch!");
				return false;
			}

			// configs.number_of_inputの型を確認
			if (configs["number_of_input"].is<double>() == false) {
				PrintBlockError(block_obj, block_no, "configs.number_of_input must be number!");
				return false;
			}

			// configs.sensor_pos_yの型を確認
			if (configs["sensor_pos_y"].is<picojson::array>() == false) {
				PrintBlockError(block_obj, block_no, "configs.sensor_pos_y must be array!");
				return false;
			}
			if (configs["sensor_pos_y"].get<picojson::array>()[0].is<double>() == false) {
				PrintBlockError(block_obj, block_no, "configs.sensor_pos_y element must be number!");
				return false;
			}
			if (configs["sensor_pos_y"].get<picojson::array>().size() != configs["number_of_input"].get<double>()) {
				PrintBlockError(block_obj, block_no, "configs.sensor_pos_y must have 'number_of_input' elements!");
				return false;
			}

			// configs.doa_sensor_chの型を確認
			if (configs["doa_sensor_ch"].is<picojson::array>() == false) {
				PrintBlockError(block_obj, block_no, "configs.doa_sensor_ch must be array!");
				return false;
			}
			if (configs["doa_sensor_ch"].get<picojson::array>()[0].is<double>() == false) {
				PrintBlockError(block_obj, block_no, "configs.doa_sensor_ch element must be number!");
				return false;
			}
			if (configs["doa_sensor_ch"].get<picojson::array>().size() != Doa::DOA_NUMBER_OF_SENSOR) {
				PrintBlockError(
					block_obj, block_no, 
					string("configs.doa_sensor_ch must have ") + to_string(Doa::DOA_NUMBER_OF_SENSOR) + string(" elements!")
				);
				return false;
			}

			// configs.bf_subarray_chの型を確認
			if (configs["bf_subarray_ch"].is<picojson::array>() == false) {
				PrintBlockError(block_obj, block_no, "configs.bf_subarray_ch must be array!");
				return false;
			}
			if (configs["bf_subarray_ch"].get<picojson::array>()[0].is<picojson::array>() == false) {
				PrintBlockError(block_obj, block_no, "configs.bf_subarray_ch element must be array!");
				return false;
			}
			if (configs["bf_subarray_ch"].get<picojson::array>().size() != configs["number_of_input"].get<double>() - 1) {
				PrintBlockError(block_obj, block_no, "configs.bf_subarray_ch element must have 'number_of_input' -1 element!");
				return false;
			}
			if (configs["bf_subarray_ch"].get<picojson::array>()[0].get<picojson::array>()[0].is<double>() == false) {
				PrintBlockError(block_obj, block_no, "configs.bf_subarray_ch element's element must be number!");
				return false;
			}
			if (configs["bf_subarray_ch"].get<picojson::array>()[0].get<picojson::array>().size() != Bf::BF_SIZE_OF_SUBARRAY) {
				PrintBlockError(
					block_obj, block_no, 
					string("configs.bf_subarray_ch element's element must have ") + to_string(Bf::BF_SIZE_OF_SUBARRAY) + string(" elements!")
				);
				return false;
			}


			int32_t			number_of_input		= static_cast<int32_t>(configs["number_of_input"].get<double>());
			picojson::array &sensor_pos_y		= configs["sensor_pos_y"].get<picojson::array>();
			picojson::array &doa_sensor_ch		= configs["doa_sensor_ch"].get<picojson::array>();
			picojson::array &bf_subarray_ch		= configs["bf_subarray_ch"].get<picojson::array>();

			config	= MALLOC_SHAREDRAM_DM(Microphonearray::Config, 1);
			param	= MALLOC_SHAREDRAM_DM(Microphonearray::Param, 1);
			status	= MALLOC_SHAREDRAM_DM(Microphonearray::Status, 1);

			if (NullCheck(config, param, status) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			Microphonearray::Config*	microphonearray_config			= MALLOC_SHAREDRAM_DM(Microphonearray::Config,	1);
			Microphonearray::Param*		microphonearray_param			= MALLOC_SHAREDRAM_DM(Microphonearray::Param,	1);
			Microphonearray::Status*	microphonearray_status			= MALLOC_SHAREDRAM_DM(Microphonearray::Status,	1);
			float*						microphonearray_sensor_pos_y	= MALLOC_SHAREDRAM_DM(float, number_of_input);
			Doa::Config*				doa_config						= MALLOC_SHAREDRAM_DM(Doa::Config,				1);
			Bf::Config*					bf_config						= MALLOC_SHAREDRAM_DM(Bf::Config,				1);
			Bf::SubarrayCh*				bf_config_subarray_ch			= MALLOC_SHAREDRAM_DM(Bf::SubarrayCh,			1);

			if (NullCheck(
				microphonearray_config,
				microphonearray_param,
				microphonearray_status,
				microphonearray_sensor_pos_y,
				doa_config,
				bf_config,
				bf_config_subarray_ch
				) == false) {
				PrintMemoryError(block_obj, block_no);
				return false;
			}

			// アレイを構成するマイクロホン素子のy座標を設定
			for (int32_t input_no = 0; input_no < number_of_input; input_no++) {
				microphonearray_sensor_pos_y[input_no] = static_cast<float>(sensor_pos_y[input_no].get<double>());
			}

			// 音源位置推定に使用するマイクロホン素子の番号と、そのy座標を設定
			uint32_t	doa_config_sensor_ch[Doa::DOA_NUMBER_OF_SENSOR];
			float		doa_config_sensor_pos_y[Doa::DOA_NUMBER_OF_SENSOR];

			for (int32_t sensor_no = 0; sensor_no < Doa::DOA_NUMBER_OF_SENSOR; sensor_no++) {
				int32_t selected_sensor = static_cast<int32_t>(doa_sensor_ch[sensor_no].get<double>());

				if ((selected_sensor < 0) || (selected_sensor >= number_of_input)) {
					PrintBlockError(block_obj, block_no, "Microphonearray:Doa invalid value!");
					return false;
				}

				doa_config_sensor_ch[sensor_no]		= selected_sensor;
				doa_config_sensor_pos_y[sensor_no]	= microphonearray_sensor_pos_y[doa_config_sensor_ch[sensor_no]];
			}

			// ビームフォーミングに使用するサブアレイを構成するマイクロホン素子の番号を設定
			for (int subarray_no = 0; subarray_no < number_of_input - 1; subarray_no++) {
				for (int sensor_no = 0; sensor_no < Bf::BF_SIZE_OF_SUBARRAY; sensor_no++) {
					int selected_sensor	= static_cast<int>(
													bf_subarray_ch[subarray_no]
													.get<picojson::array>()[sensor_no]
													.get<double>());

					if (/*allow negative value*/ selected_sensor >= number_of_input) {
						PrintBlockError(block_obj, block_no, "Microphonearray:Bf invalid value!");
						return false;
					}

					bf_config_subarray_ch[subarray_no][sensor_no] = selected_sensor;
				}
			}

			if (Doa::Configure(
				doa_config,
				doa_config_sensor_ch,
				doa_config_sensor_pos_y
				) == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Doa::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}

			if (Bf::Configure(
				bf_config,
				number_of_input,
				GetSharedRamPointer(microphonearray_sensor_pos_y),
				GetSharedRamPointer(bf_config_subarray_ch)
				) == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Bf::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}

			if (Microphonearray::Configure(
				reinterpret_cast<Microphonearray::Config*>(config),
				GetSharedRamPointer(doa_config),
				GetSharedRamPointer(bf_config)
				) == false) {
				PrintConfigurationError(block_obj, block_no);
				return false;
			}
			if (Microphonearray::Attach() == false) {
				PrintAttachError(block_obj, block_no);
				return false;
			}
			max_used_buf_no = std::max(ibuf_no + number_of_input - 1, obuf_no);

		} else {
			PrintBlockError(block_obj, block_no, "invalid dsp type!");
			return false;
		}

		if (AddNetList(
			type_id,
			ibuf_no,
			obuf_no,
			config,
			param,
			status
			) == false) {
			PrintBlockError(block_obj, block_no, "Failed to add netlist!");
			return false;
		}
		number_of_io = std::max(number_of_io, max_used_buf_no + 1);
	}
	g_dsp_manager_config->buffer_size = number_of_io;
	return true;
}

bool
CreateDspConfiguration(picojson::array& block_cfg_)
{
	/* ネットリスト初期化 */
	if (ConfigurationDspManager(block_cfg_) == false) {
		PrintHeapInfo();
		return false;
	}
	PrintHeapInfo();
	return true;
}

} // namespace dsp_ctrl
