//==========================================================================
// aec.h

//--------------------------------------------------------------------------
#pragma once

//==========================================================================
#include <stdint.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/dsp/iir.h>
#include <dsp_lib/include/dsp/microphonearray.h>
#include <dsp_lib/include/misc/dsp_const.h>

//==========================================================================
namespace dsp_lib {

	//==========================================================================
	class Aec : public DspModule {
	public:
		//----------------------------------------
		typedef struct {
		} Config;
		typedef struct {
			uint32_t 	cear_aec;				// <! aec on/off
			float		param_get_flag;
			float		param_set_flag;
			float		mu_[8];
			float		errmu_limit;
			float		rcv_dly_;
			float		nlp_ratio0;
			float		nlp_ratio1;
			float		nlp_enhance0;
			float		nlp_enhance1;
			float		nlp_alpha0;
			float		nlp_alpha1;
			float		nlp_alpha2;
			float		nlp_step;
			float		drc_compTh;
			float		drc_compRatio;
			float		drc_compGain;
			float		drc_boostTh;
			float		drc_boostRatio;
			float		drc_boostGain;
			float		drc_AT;
			float		drc_RT;
			Microphonearray::Status*	microphonearray;
			uint32_t	mic_mute;
			float		drc_gain;
			float		cng_noise;
			float		bmInput_fixedGain;
			float		nlp_fixedGain;
			float		drc_fixedGain;
		} Param;
		typedef struct {
			float		focus_x;
			float		focus_y;
		} Status;

		//----------------------------------------
		Aec(
			Config*	/*config_*/,
			Param*	param_,
			Status* status_
		);
		static bool Configure()
		{
			return true;
		}
		static bool Attach()
		{
			return true;
		}
		bool Ready();
		bool Execute(
			const float*	input_,								// <! rcv i, snd i
			float*			output_								// <! rcv o, snd o
		);
		bool Task();

	protected:
		//----------------------------------------
		static const int	AEC_BLOCK_SIZE = DSP_BLOCK_SIZE / DSP_SRC_RATE;

		//----------------------------------------
		void Initialize();
		void ExecuteCtrl();
		void ExecuteThrough(
			float(*i_)[AEC_BLOCK_SIZE],
			float(*o_)[AEC_BLOCK_SIZE]
		);

		//----------------------------------------
		bool			m_ready;
		Param*			m_param;
		Status*			m_status;
		float 			mic_mute_gain;
		float 			bf_gain;
		void SetParam();
		void GetParam();
	};

	//==========================================================================
}	// namespace dsp_lib
