/*
 * dsp_delay.h
 *
 *  Created on: 2014/07/02
 *      Author: toa29548
 */

#pragma once

#include <stdint.h>
#include <dsp_lib/include/misc/null_check.h>
#include <dsp_lib/include/misc/misc.h>
#include <dsp_lib/include/dsp/dsp_module.h>

namespace dsp_lib {

class Delay : public DspModule {
public:
	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------
	struct Config{
		uint32_t	buf_size;			//!< バッファサイズ, 2のべき乗, >=DSP_BLOCK_SIZE
	};
	// 例
	// buf_size	= 131072 の場合,	2^17[samples], 2730[msec] @FS=48kHz
	// buf_size	= 1024 の場合,	2^10[samples], 21.3[msec] @FS=48kHz

	struct Param {
		uint32_t	delay_samples;		//!< 遅延量[samples], buf_size - 1
	};
	struct Status{};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------
	/*!
	 * Delayコンフィギュレーションの初期化
	 * @note
	 * @param config_ コンフィギュレーション構造体
	 * @param number_of_input_ Matrixの入力チャンネル数
	 * @param number_of_output_ Matrixの出力チャンネル数
	 */
	static bool
	Configure(
	    Config*    config_,
	    uint32_t   buf_size_
	)
	{
		if (NullCheck(config_) == false) {
			return false;
		}
		if (IsPowerOf2(buf_size_) != 1) {
			return false;
		}
		config_->buf_size = buf_size_;

		return true;
	}

	static bool Attach(void) {
		return true;
	}

	Delay(
			Config*	config_,
			Param*	param_,
			Status*	/*status_*/
	);

	bool Ready(void);

	bool Execute(
			const float*	input_,
				  float*	output_
	);
	bool Task(void);

protected:
	Config*		m_config;
	Param*		m_param;
//	Status*		m_status;

	uint32_t	m_buf_size;			//!< Config構造体から取り込む

	float		*m_cbuf;			//!< リングバッファ
	uint32_t	m_cbuf_idx;			//!< リングバッファのインデックス

};

} //namespace dsp_lib
