/*
 * dsp_peak.h
 *
 *  Created on: 2016/09/29
 *      Author: toa29548
 */

#pragma once

#include <stdint.h>
#include <dsp_lib/include/dsp/dsp_module.h>

namespace dsp_lib {

class Peak : public DspModule {
public:
	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------
	struct Config{};
	struct Param{};
	struct Status{
		float		peak;			//!< 直近5msecのpeak, 単位:振幅の2乗
		float		peak_release;	//!< peakと同様に上昇するが、ゆっくりと下降する, 単位:振幅の2乗
	};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------
	static bool Configure(void) {
		return true;
	};
	static bool Attach(void) {
		return true;
	}

	Peak(
			Config*	/*config_*/,
			Param*	/*param_*/,
			Status*	status_
	);

	bool Ready(void);

	bool Execute(
			const float*	input_,
				  float*	/*output_*/
	);
	bool Task(void);

protected:
	// coeff_peak_release : powf(powf(10.0f, -x [dB] /10.0f), 5.0f/1000.0f)
	#define	PEAK_RELEASE_40DB_SEC	(0.9549925860214360f)
	#define	PEAK_RELEASE_30DB_SEC	(0.9660508789898130f)
	#define	PEAK_RELEASE_20DB_SEC	(0.9772372209558110f)
	#define	PEAK_RELEASE_10DB_SEC	(0.9885530946569390f)
	#define	PEAK_SCALING_COEFF		(1.0f)		// スケーリング用係数

	static const float	PEAK_RELEASE_DEFAULT;
	static const float	PEAK_RELEASE_CALC_PERIOD_SEC;	//!< peak_releaseの計算周期[秒]
	static const float	PEAK_HOLD_TIME_SEC;				//!< peakのホールドタイム[秒]

	static const uint32_t	PEAK_RELEASE_CALC_PERIOD;
	static const uint32_t	PEAK_HOLD_BLOCK;

	static int m_count_distributor;		//!< 更新周期カウンタの初期値をインスタンスごとに変えるために使う

	void execute_peak_release(void);
	void execute_peak_detect(
			const float*	input_
	);


//	Config*		m_config;
//	Param*		m_param;
	Status*		m_status;

	float 		m_peak;				//!< peak_currentを5msecピークホールドしたレベル, 単位:振幅の2乗
	float 		m_peak_current;		//!< 現在のリセット周期内の最大レベル, 単位:振幅の2乗
	float 		m_peak_release;		//!< peak_releaseのワークエリア, 単位:振幅の2乗
	uint32_t	m_release_count;	//!< 更新判断用カウンタ
	uint32_t	m_hold_count;		//!< ホールドタイム計測用カウンタ

};

} //namespace dsp_lib
