/*
 *	dsp_rms.h
 *
 */
#pragma once

#include <cmath>
#include <stdint.h>
#include <dsp_lib/include/dsp/dsp_module.h>

namespace dsp_lib {

class Rms : public DspModule {
public:
	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------
	struct Config{};
	struct Param{};
	struct Status {
		float	rms;									//!< DSP_BLOCK_SIZE*SIZE_OF_RMS_BLOCK*SIZE_OF_RMS_SUMサンプルの2乗平均, 平方根は取っていない
	};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------
	static bool Configure(void) {
		return true;
	};
	static bool Attach(void) {
		return true;
	}

	Rms(
			Config*	/*config_*/,
			Param*	/*param_*/,
			Status*	status_
	);

	bool Ready(void);

	bool Execute(
			const float*	input_,
				  float*	/*output_*/
	);
	bool Task(void);

protected:
	static const int	RMS_UPDATE_PERIOD_MSEC	= 5;	//!< RMSの更新周期の設計値
	static const int	RMS_FRAME_LENGTH_MSEC	= 100;	//!< RMSの計算フレーム長の設計値
	static const int	SQUARE_SUM_BUF_LEN		= RMS_FRAME_LENGTH_MSEC / RMS_UPDATE_PERIOD_MSEC + 1;	//!< ワーストケースを考えて+1

	static const float	SINE_WAVE_SCALING;				//!< FULLSCALEのsinのときに1.0にするための係数
	static const float	RMS_SCALING_COEF;				//!< RMSのスケーリング係数

	static const int	SIZE_OF_RMS_BLOCK;
	static const int	SIZE_OF_RMS_SUM;
	static const float	RMS_MEAN_COEF;

	static const float	RMS_UPDATE_PERIOD_MSEC_ACTUAL;			//!< RMSの更新周期の実際の値
	static const float	RMS_FRAME_LENGTH_MSEC_ACTUAL;			//!< RMSの計算フレーム長の実際の値

	static int m_count_distributor;		//!< 更新周期カウンタの初期値をインスタンスごとに変えるために使う

	void execute_rms_summing(
			const float*	input_
	);
	void execute_rms_update(void);

//	Config*		m_config;
//	Param*		m_param;
	Status*		m_status;

	float		m_square_sum_buf[SQUARE_SUM_BUF_LEN];	//!< RMS更新周期分の2乗和を貯めるリングバッファ
	uint32_t	m_square_sum_buf_idx;					//!< リングバッファ内のインデックス
	float		m_square_sum;							//!< 割り込みの度に2乗和を足し込んで、RMS更新周期分（DSP_BLOCK_SIZE*SIZE_OF_RMS_BLOCKサンプル）の2乗和にする
	uint32_t	m_count;								//!< 更新周期を測るカウンタ
};

} //namespace dsp_lib
