/*
 * smoothing.h
 *
 *  Created on: 2016/11/28
 *      Author: toa29548
 */

#pragma once

#include <dsp_lib/include/misc/dsp_const.h>
#include <stdint.h>

namespace dsp_lib {

/*!
 * 時間を忘却係数に変換する
 * @param time_				アタックタイム or リリースタイム[sec]
 * @param update_frequency_	データの更新周波数。サンプル更新の場合はサンプリング周波数を, 割込みごとに更新する場合は割込み周波数を指定する
 */
inline float
TimeToForgetFactor(
	const float time_,
	const float	update_frequency_
)
{
	float forget_factor = 1.0f - time_/(time_+(1.0f/(update_frequency_)));	// 増幅時の忘却係数
	return forget_factor;
}


/*!
 * 時間をサンプル数に変換する
 * @param time_				時間[sec]
 * @param samples_			サンプル[samples]
 */
inline void
TimeToSample(
	const float time_,
	uint32_t*	samples_
)
{
	*samples_ = time_ * DSP_SAMPLING_FREQ;
}

/*!
 * 時間をブロック数に変換する
 * @param time_				時間[sec]
 * @param blocks_			ブロック[blocks]
 */
inline void
TimeToBlock(
	const float time_,
	uint32_t*	blocks_
)
{
	*blocks_ = time_ * DSP_BLOCK_FREQ;
}

} //namespace dsp_lib
