#!/bin/sh

### This script returns :
###   0: On success.
###   1: Error has been occured.

MYSELF_DIR=$(dirname "${0}")
ENVIRONMENT_FILE="${MYSELF_DIR}"
DSP_OLD_VERSION="/home/toa/TOA_AM-CF1/Application/data/config/dsp_version.json" # DART Model

## DSP UpdateList modify
DSP_UPDATE_LISTFILE_MODIFY_PY="${ENVIRONMENT_FILE}/dsp_update_list_modify.py"

## DSP UPDATE FILES LIST
DSP_UPDATE_FILELIST="${ENVIRONMENT_FILE}/DSP_Updater_Files.json"

## DSP UPDATE FILES update Phase 1##
DSP_MODULE_BIN_TMP="${ENVIRONMENT_FILE}/AMCF1_DSPmodule_tmp.bin"
DSP_CHKSUM_FILE_TMP="${ENVIRONMENT_FILE}/AMCF1_DSPmodule_tmp.sum"
DSP_NEW_VERSION_TMP="${ENVIRONMENT_FILE}/new_dsp_version_tmp.json"

## DSP UPDATE FILES update Phase 2## (if necessary)
DSP_MODULE_BIN="${ENVIRONMENT_FILE}/AMCF1_DSPmodule.bin"
DSP_CHKSUM_FILE="${ENVIRONMENT_FILE}/AMCF1_DSPmodule.sum"
DSP_NEW_VERSION="${ENVIRONMENT_FILE}/new_dsp_version.json"

TO_COPY_DIR="/mnt/data/DSP"

## Check version file##
Check_Compare_DSPVersion () {
  RESULT_DIFF=1
  LBL_VERSION='dsp_version'
  
  #json read (DSP version file is dsp_version tag only.)
  _new_version=$(cat ${DSP_NEW_VERSION} | sed 's/^\[{\(.*\)}]$/\1/' | grep ${LBL_VERSION} | cut -d ':' -f 2- | tr -d '}' | tr -d ' "' | tr -d '\n' )
  _old_version=$(cat ${DSP_OLD_VERSION} | sed 's/^\[{\(.*\)}]$/\1/' | grep ${LBL_VERSION} | cut -d ':' -f 2- | tr -d '}' | tr -d ' "' | tr -d '\n' )

  echo ${_new_version}
  echo ${_old_version}

  if [ ${_new_version} = ${_old_version} ]; then
    RESULT_DIFF=0
  else
    RESULT_DIFF=1
  fi
}

## Check version file and modify DSP Update List file##
Check_DSPVersion_and_Modif_ListFile () {
  RESULT_DIFF=1
  LBL_VERSION='dsp_version'
  
  #json read (DSP version file is dsp_version tag only.)
  _old_version=$(cat ${DSP_OLD_VERSION} | sed 's/^\[{\(.*\)}]$/\1/' | grep ${LBL_VERSION} | cut -d ':' -f 2- | tr -d '}' | tr -d ' "' | tr -d '\n' )

  echo ${_old_version}

  if [ ${_old_version} != '1.1.0.0' ]; then
    # IF DSP version is not 1.1.0.0
    python3 ${DSP_UPDATE_LISTFILE_MODIFY_PY}
  else
    # IF DSP version is 1.1.0.0
    
    # Update Phase 1
    # COPY DSP TMP FW 
    if [ ! -f ${DSP_MODULE_BIN_TMP} ]; then exit 1; fi
    if [ ! -f ${DSP_CHKSUM_FILE_TMP} ]; then exit 1; fi
    if [ ! -f ${DSP_NEW_VERSION_TMP} ]; then exit 1; fi

    cp -rfp ${DSP_MODULE_BIN_TMP} ${TO_COPY_DIR}/
    cp -rfp ${DSP_CHKSUM_FILE_TMP} ${TO_COPY_DIR}/
    cp -rfp ${DSP_NEW_VERSION_TMP} ${TO_COPY_DIR}/
  fi
}

##-----------------------------------------------------------------
## DSP Update START ##

## If DSP_NOW_VERSION does not exist, force to update dsp
if [ ! -f ${DSP_OLD_VERSION} ]; then
  echo 'Not found now dsp version file.'
else
  if [ ! -f ${DSP_NEW_VERSION} ]; then exit 1; fi

  ## SAME:0 DIFF:1
  Check_Compare_DSPVersion
  if [ ${RESULT_DIFF} -eq 0 ]; then
    echo 'DSP version is same.'
    exit 0
  fi
fi

# DSP update list Modify
if [ ! -f ${DSP_UPDATE_LISTFILE_MODIFY_PY} ]; then exit 1; fi

## DSP Update directory Ready ##
rm -rf ${TO_COPY_DIR}
mkdir -m 777 ${TO_COPY_DIR}
# Copy DSP UPDATE FILES #

Check_DSPVersion_and_Modif_ListFile

# Update Phase 2
if [ ! -f ${DSP_MODULE_BIN} ]; then exit 1; fi
if [ ! -f ${DSP_CHKSUM_FILE} ]; then exit 1; fi
if [ ! -f ${DSP_NEW_VERSION} ]; then exit 1; fi
if [ ! -f ${DSP_UPDATE_FILELIST} ]; then exit 1; fi

cp -rfp ${DSP_MODULE_BIN} ${TO_COPY_DIR}/
cp -rfp ${DSP_CHKSUM_FILE} ${TO_COPY_DIR}/
cp -rfp ${DSP_NEW_VERSION} ${TO_COPY_DIR}/

# Update file list
cp -rfp ${DSP_UPDATE_FILELIST} ${TO_COPY_DIR}/
# Permission change
chmod 777 ${TO_COPY_DIR}/*

exit 0
