/*************************************
 *  BM64Control.h
 *  
 *  History
 *  2017/09/28  1st release
 *************************************/

//serial port
#define BM64_SERIAL_PORT                "/dev/serial0"

//serial port settings
#define BM64_SERIAL_IN_FLAGS            (0)
#define BM64_SERIAL_OUT_FLAGS           (0)
#define BM64_SERIAL_CTRL_FLAGS          (CLOCAL | CREAD | CS8 | B115200)
#define BM64_SERIAL_LOCAL_FLAGS         (0)
#define BM64_SERIAL_TIMEOUT             (1)         //100 ms order

//buffer size
#define BM64_RECV_BUFF_SIZE             (1024 * 2)
#define BM64_SEND_BUFF_SIZE             (1024 * 2)
#define BM64_WORK_BUFF_SIZE             (256)
#define BM64_DEBUG_MSG_SIZE             (256) 

//time out
#define BM64_SEND_COMMAND_TIMEOUT       (3)         //1 sec order

//status
#define BM64_TRUE                       (1)
#define BM64_FALSE                      (0)

//status  
#define BM64_STATUS_SUCCESS             (0)
#define BM64_STATUS_FALSE               (-1)
#define BM64_STATUS_UNSUCCESSFUL        (-2)
#define BM64_STATUS_DEVICE_ERROR        (-3)     
#define BM64_STATUS_INVALID_CSUM        (-4) 
#define BM64_STATUS_NOT_INITIALIZE      (-5)
#define BM64_STATUS_WRITE_ERROR         (-6)
#define BM64_STATUS_READ_ERROR          (-7)
#define BM64_STATUS_DISALLOW            (-8)
#define BM64_STATUS_UNKNOWN             (-9)
#define BM64_STATUS_PARAM_ERROR         (-10)
#define BM64_STATUS_BTM_BUSY            (-11)
#define BM64_STATUS_BTM_FULL            (-12)
#define BM64_STATUS_SEND_BUSY           (-13)
#define BM64_STATUS_NOT_RESPONSE        (-14)    

//Debug Flags
#define BM64_DBG_ERROR                  (0x1)
#define BM64_DBG_WARNING                (0x2)
#define BM64_DBG_CAUTION                (0x4)
#define BM64_DBG_COMMENT                (0x8)
#define BM64_DBG_FUNC_ENTER             (0x10)
#define BM64_DBG_FUNC_EXIT              (0x20)
#define BM64_DBG_LOCK                   (0x40)
#define BM64_DBG_EVENT                  (0x80)
#define BM64_DBG_COMMAND                (0x100)
#define BM64_DBG_ACK                    (0x200)
#define BM64_DBG_CALLBACK               (0x400)
#define BM64_DBG_SEND                   (0x800)
#define BM64_DBG_RECV                   (0x1000)
#define BM64_DBG_DUMP                   (0x2000)
#define BM64_DBG_ALLOC                  (0x4000)

//packet format
#define BM64_INDEX_SYNC_WORD            (0)
#define BM64_INDEX_OPCODE               (3)
#define BM64_INDEX_COMMAND_OPCODE       (4)
#define BM64_INDEX_STATUS               (5)

#define BM64_SYNC_WORD                  (0xAA)
#define BM64_PACKET_APPENDIX            (4)         //harder + length + crc

#define BM64_EVENT_ACK_SIZE             (6)
#define BM64_MMI_ACTION_SIZE            (7)

//event opcode
#define BM64_OPCODE_EV_COMMAND_ACK      (0x0)

//command ack status
#define BM64_ACK_COMPLETE               (0)
#define BM64_ACK_DISALLOW               (1)
#define BM64_ACK_UNKNOWN_COMMAND        (2)
#define BM64_ACK_PARAM_ERROR            (3)
#define BM64_ACK_BUSY                   (4)
#define BM64_ACK_MEMORY_FULL            (5)
#define BM64_ACK_WRITE_ERROR            (6)
#define BM64_ACK_NOT_RESPONSE           (7)

typedef struct _BM64_SEND_DATA_INFO
{ 
    unsigned int Request; 
    unsigned int Length; 
    unsigned int Complete; 
    unsigned char OpCode; 
    unsigned char Status;
} BM64_SEND_DATA_INFO, *PBM64_SEND_DATA_INFO; 

//callback
typedef int (*LibBM64_Callback)( unsigned char *pBuffer, int Length ); 

#ifdef __cplusplus
extern "C" 
{
#endif

//APIs
extern int LibBM64_Init( void ); 
extern int LibBM64_Deinit( void ); 
extern int LibBM64_EnterPairingMode( void ); 
extern int LibBM64_ExitPairingMode( void );
extern int LibBM64_LinkLastDevice( void );
extern int LibBM64_DisconnectAllLink( void ); 
extern int LibBM64_SendUserCommand( unsigned char *pBuffer, unsigned int Lentgh ); 
extern int LibBM64_CallbackSet( LibBM64_Callback pCallback ); 
extern void Bm64DPrintf( unsigned int Flags, const char *Fmt, ... );


#ifdef __cplusplus
}
#endif
