#pragma once

#define GPIO_HEART_BEAT			6
#define GPIO_CONTROL_OUT1   22
#define GPIO_CONTROL_OUT2   27
#define GPIO_CONTROL_IN1    26
#define GPIO_CONTROL_IN2    16
#define GPIO_PC_VOL_UP      13
#define GPIO_PC_VOL_DOWN    12
#define GPIO_PC_VOL_MUTE    17
#define GPIO_PWR_DOWN_DET   23
#define GPIO_USB_AUDIO_ST   4
#define GPIO_ADDA_RESET     24
#define GPIO_DSP_RESET      7

#define GPIO_MOSI			10
#define GPIO_MISO			9

#define MIN_COUNT_PWSW      40
#define MAX_COUNT_PWSW      80
#define MAX_COUNT_SHUTDOWN  200
#define MAX_COUNT_CTRL_IN   2   // temp
#define MAX_CTRL_IO_NUMBER  2

//#define MAX_COUNT_SWITCH    10 // 0.63s
//#define MAX_COUNT_SWITCH    20 // 1.31s
//#define MAX_COUNT_SWITCH    30 // 2.31s
//#define MAX_COUNT_SWITCH    40 // 3.84s


typedef struct {
	uint32_t prev;
	uint32_t current;
    uint32_t temp[2];
    uint32_t index;
	uint32_t flag;
	uint32_t count;
} st_inputGPIO;

enum {
    INPUT0_POWER_SWITCH,	//Second PCBからなくなる
    INPUT1_RESET_SWITCH,	//Second PCBからなくなる
    INPUT2_CONTROL_INPUT1,
    INPUT3_CONTROL_INPUT2,
    MAX_INPUT_NUMBER
};

enum {
    CFG_LEVEL,
    CFG_PULSE
};

enum {
    CIN_FUNC_NONE,
    CIN_FUNC_MUTE_PULSE,
    CIN_FUNC_MUTE_LEVEL,
    CIN_FUNC_VOL_UP,
    CIN_FUNC_VOL_DOWN,
    CIN_FUNC_PRESET1,
    CIN_FUNC_PRESET2,
    CIN_FUNC_MAX_NUMBER
};

enum {
    COUT_FUNC_NONE,
    COUT_FUNC_MUTE_STATUS,
    COUT_FUNC_SYNC_INPUT1,
    COUT_FUNC_SYNC_INPUT2,
    COUT_FUNC_MAX_NUMBER
};

void Initialize_GPIO(void);
void Initialize_GPIOStatus(void);
void Initialize_CtrlInputConfig(uint32_t i_cfg1, uint32_t i_cfg2);
void Initialize_CtrlOutputConfig(uint32_t i_cfg1, uint32_t i_cfg2);
uint32_t Is_CtrlInputCfgPulse(uint32_t i_num);
uint32_t Get_CtrlInputCfg(uint32_t i_num);
uint32_t Get_ControlInput1(void);
uint32_t Get_ControlInput2(void);
uint32_t Get_PowerDownDetect(void);
uint32_t Get_UsbAudioStatus(void);
void Set_ControlOutput1(uint32_t i_ctrl);
void Set_ControlOutput2(uint32_t i_ctrl);
void Set_ControlOutputIndividual(uint32_t i_num, uint32_t i_ctrl);
void Set_PC_VolumeUp(uint32_t i_ctrl);
void Set_PC_VolumeDown(uint32_t i_ctrl);
void Set_PC_VolumeMute(uint32_t i_ctrl);
void Set_ADDA_Reset(uint32_t i_ctrl);
void Set_DSP_Reset(uint32_t i_ctrl);
void Set_gpioMuteStatus(uint32_t i_status);
uint32_t Get_gpioMuteStatus(void);
uint32_t Is_ShutdownMode(void);
uint32_t Is_StandbyMode(void);
uint32_t Is_PowerSwitchStatusChange(void);
uint32_t Is_ResetSwitchStatusChange(void);
uint32_t Is_ControlInput1StatusChange(void);
uint32_t Is_ControlInput2StatusChange(void);
uint32_t Read_PowerSwitchStatus(void);
uint32_t Read_ResetSwitchStatus(void);
uint32_t Read_ControlInput1Status(void);
uint32_t Read_ControlInput2Status(void);
uint32_t Debug_GetCount(void);
int32_t CinScan(void);
int32_t KeyScan(void);
void Update_CtrlOutput(void);
