/*--------------------------------------------------------------------------*/
/*																			*/
/*	bluetooth_control.c (Rev 1.00)											*/
/*																			*/
/*	1.概要																	*/
/*		BM64制御処理														*/
/*																			*/
/*	2.改版履歴																*/
/*																			*/
/*																			*/
/*--------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdint.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/file.h>
#include "BM64Control.h"
#include "i2c_control.h"
#include "CtimerLib.h"
#include "bluetooth_control.h"

#if 1
// 時間計測デバッグ用
#include <sys/time.h>
#endif


#define BT_MODESTATUS_NONE			(0)
#define BT_MODESTATUS_IDLE			(1)
#define BT_MODESTATUS_PAIRING		(2)
#define BT_MODESTATUS_CONNECTED		(4)
#define BT_MODESTATUS_RECONNECT		(5)

#define LED_OFF						(0)
#define LED_ON						(1)

#define SW_OFF						(0)
#define SW_ON						(1)

#define BT_PATH_MAX					(1024)	/* ファイルパス長					*/
#define BT_DEVICE_NAME_MAX			(255)	/* デバイス名称の最大文字数			*/

#define BT_KEY_LOCK					(1)
#define BT_KEY_UNLOCK				(0)

/* 外部制御プロトコルのコマンド(CommStatusConverter.h)と値を揃えること */
#define	BT_CONTROL_INVALID			(0)		// 制御指示: 指示なし
#define	BT_CONTROL_ON				(1)		// 制御指示: ON
#define	BT_CONTROL_OFF				(2)		// 制御指示: OFF

#define BT_REM_PROC_FLAG_OFF		(0)		// 外部制御プロトコルによるBT制御済フラグOFF
#define BT_REM_PROC_FLAG_ON			(1)		// 外部制御プロトコルによるBT制御済フラグON

//#define BM64_DEBUG


int32_t	BTpairing;							/* ペアリングフラグ 		*/
int32_t	BTestablished;						/* 接続確立フラグ 			*/
int32_t	BTdisconnected;						/* 切断フラグ 				*/
//int32_t BTevent_mute;						/* ミュート 				*/
int32_t BTInitError=0;						/* 初期化での通信失敗		*/
static uint8_t BTDeviceNamePath[BT_PATH_MAX];
											/* デバイス名称のファイルパス	*/
int32_t SCO_ready;							/* SCO link connected	*/


static void BTDPrintf( const char *Fmt, ... );
static void BTDPrintfLog( const char *Fmt, ... );
//static void LibBT_resetMuteEven(void);
static int32_t LibBT_IsStandbyMode( uint32_t i_state );
static int32_t LibBT_getKeyEvent(void);
static void LibBT_ControlLED( int32_t BTmode );
static void LibBT_Error(void);
static int32_t EventCallback( uint8_t *pBuffer, int32_t Length );
static int32_t LibBT_setIdleMode(void);
static int32_t LibBT_enterPairingMode(void);
static void LibBT_stopBM64(void);
static void LibBT_startBM64(void);
static int32_t LibBT_getDeviceName(uint8_t* filePath, uint8_t* deviceName);

#if 1
// 時間計測デバッグ用
struct timeval tvStart;
struct timeval tvEnd;

void tttimersub(struct timeval *a, struct timeval *b, struct timeval* result)
{
    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;
    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;
    if ((result)->tv_usec < 0) {
      --((result)->tv_sec);
      (result)->tv_usec += 1000000;
    }
}

void getTimeStart( void )
{
	gettimeofday( &tvStart, NULL );
}


void getTimeEnd( void )
{

	struct timeval tvDiff;
	int32_t   diffS;

	gettimeofday( &tvEnd, NULL );

	tttimersub(&tvEnd, &tvStart, &tvDiff);
	diffS = (tvDiff.tv_sec * 1000) + (tvDiff.tv_usec / 1000);


	if (( tvEnd.tv_usec < tvStart.tv_usec ) && (diffS > 50)){
		printf( "elapsed %ld.%06ld[s]\n", tvEnd.tv_sec - tvStart.tv_sec - 1, tvEnd.tv_usec - tvStart.tv_usec + 1000000 );
	} else if (diffS > 50) {
		printf( "elapsed %ld.%06ld[s]\n", tvEnd.tv_sec - tvStart.tv_sec, tvEnd.tv_usec - tvStart.tv_usec );
	}
}
#endif


static void Read_Local_Device_Name(void)
{
	uint8_t 			pBuffer[2];

	/*------------------------------------------------------*/
	/* BM64 OFF												*/
	/* -----------------------------------------------------*/
    pBuffer[0] = 0x10;
    pBuffer[1] = 0x00;
	LibBM64_SendUserCommand(&pBuffer[0], 2);
}


static void LibBT_Change_Device_Name(void)
{
	uint8_t 			pBuffer[BT_DEVICE_NAME_MAX+1];
	uint8_t				pTmpBuffer[BT_DEVICE_NAME_MAX];
	int32_t				copySize;

	/*------------------------------------------------------*/
	/* BM64 OFF												*/
	/* -----------------------------------------------------*/
	memset(pBuffer, 0x00, sizeof(pBuffer));
    pBuffer[0] = 0x05;

	memset(pTmpBuffer, 0x00, sizeof(pTmpBuffer));
	copySize = LibBT_getDeviceName(BTDeviceNamePath, pTmpBuffer);
	// 正常に取得できた場合
	if (copySize > 0)
	{
		memcpy(&pBuffer[1], pTmpBuffer, copySize);
		LibBM64_SendUserCommand(&pBuffer[0], (copySize+1));
	}
	// 異常が発生した場合にはAM-CF1を設定する
	else
	{
		//41 4d 2d 43 46 31     //AM-CF1
		pBuffer[1] = 0x41;
		pBuffer[2] = 0x4d;
		pBuffer[3] = 0x2d;
		pBuffer[4] = 0x43;
		pBuffer[5] = 0x46;
		pBuffer[6] = 0x31;
		pBuffer[7] = 0x00;

		LibBM64_SendUserCommand(&pBuffer[0], 8);
	}
}

static void LibBT_NR_Setting(void)
{
	uint8_t 			pBuffer[9];

#if 1
#if 0
	pBuffer[0] = 0x1D;
    pBuffer[1] = 0x18;
	LibBM64_SendUserCommand(&pBuffer[0], 2);
	BTDPrintf("DSP_NR_CTRL : ENABLE_Mic_NR\n");
#else
	pBuffer[0] = 0x1D;
    pBuffer[1] = 0x19;
	LibBM64_SendUserCommand(&pBuffer[0], 2);
	BTDPrintf("BTM_Status: DISABLE_Mic_NR\n");
#endif
#endif


#if 1
#if 0
	pBuffer[0] = 0x1D;
    pBuffer[1] = 0x1B;
	LibBM64_SendUserCommand(&pBuffer[0], 2);
	BTDPrintf("BTM_Status: ENABLE_SPK_NR\n");
#else
	pBuffer[0] = 0x1D;
    pBuffer[1] = 0x1C;
	LibBM64_SendUserCommand(&pBuffer[0], 2);
	BTDPrintf("BTM_Status: DISABLE_SPK_NR\n");
#endif
#endif

}

static void LibBT_AEC_Setting(void){
	uint8_t 			pBuffer[9];

#if 1
#if 1
	BTDPrintf("\n!!!BTM_Status: Enable AEC\n");
	pBuffer[0] = 0x02;
    pBuffer[1] = 0x00;
    pBuffer[2] = 0x1e;
	LibBM64_SendUserCommand(&pBuffer[0], 3);
#else
	BTDPrintf("\n!!!BTM_Status: Disalbe AEC\n");
	pBuffer[0] = 0x02;
    pBuffer[1] = 0x00;
    pBuffer[2] = 0x1f;
	LibBM64_SendUserCommand(&pBuffer[0], 3);
#endif
#endif

#if 1
#if 1
	BTDPrintf("\n!!!BTM_Status: Enable AEC RX noise reduction\n");
	pBuffer[0] = 0x02;
    pBuffer[1] = 0x00;
    pBuffer[2] = 0x21;
	LibBM64_SendUserCommand(&pBuffer[0], 3);
#else
	BTDPrintf("\n!!!BTM_Status: Disable AEC RX noise reduction\n");
	pBuffer[0] = 0x02;
    pBuffer[1] = 0x00;
    pBuffer[2] = 0x22;
	LibBM64_SendUserCommand(&pBuffer[0], 3);
#endif
#endif

}


static int32_t LibBT_getDeviceName(uint8_t* filePath, uint8_t* deviceName)
{
	int8_t* find = "\"bt_devicename\"";
	FILE* file;
	int32_t lock = -1;
	uint8_t buff[BT_DEVICE_NAME_MAX];
	int8_t* search;
	int8_t* target;
	uint8_t isFind = -1;
	uint8_t targetSize = 0;

	memset(buff, 0x00, sizeof(buff));

	file = fopen(filePath, "r");
	if (file != NULL) {
		lock = flock(fileno(file), LOCK_SH | LOCK_NB);

		if (lock == -1) {
			fclose(file);
			BTDPrintfLog("LibBT : lock error");
			return -1;
		}
	}
	else {
		BTDPrintfLog("LibBT : open error : %s\n", filePath);
		return -1;
	}

	// "device"行の取得
	while(fgets(buff, sizeof(buff), file) != NULL)
	{
		if (strstr(buff, find) != NULL)
		{
			isFind = 0;
			break;
		}
	}

	flock(fileno(file), LOCK_UN);
	fclose(file);

	// "bt_devicename"行からデバイス名称を取得する
	if (isFind == 0)
	{
		// "bt_devicename"があることを前提にjsonの区切り文字「:」を検索する
		search = strchr(buff, ':');
		if (search == NULL)
		{
			BTDPrintfLog("LibBT : not find = :");
			return -1;
		}
		// 次の文字にアドレスを移動する
		search += 1;

		// デバイス名称の始まりの「"」を探す
		search = strchr(search, '"');
		if (search == NULL)
		{
			BTDPrintfLog("LibBT : not find = /");
			return -1;
		}
		// 次の文字にアドレスを移動する
		search += 1;
		// デバイス名称の文字列の先頭アドレスを取得する
		target = search;

		// デバイス名称の終わりの「"」を探す
		search = strchr(search, '"');
		if (search == NULL)
		{
			BTDPrintfLog("LibBT : not find = /");
			return -1;
		}

		// デバイス名称の文字数を取得する
		targetSize = search - target;

		// デバイス名称を取得した文字数分コピーする
		memcpy(deviceName, target, targetSize);
	}

	return targetSize;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_Init                                                              */
/*                                                                          */
/* 2.Description                                                            */
/*  BlueTooth LIB　initalize				                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t* deviceNamePath		デバイス名称の設定ファイルパス              */
/*  uint32_t pathSize			設定ファイルパスのサイズ		            */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t LibBT_Init(uint8_t* deviceNamePath, uint32_t pathSize)
{
	int32_t ret;
	int32_t ii;
	static int32_t BTinitFlg = 0;						/* 多重起動を制限する	*/
	uint8_t 			pBuffer[8];

    BTDPrintf( "LibBT build:%s Time:%s\n", __DATE__, __TIME__);

    if (BTinitFlg)
    {
        return(BT_STATUS_FALSE);
    };

	/* デバイス名称のファイルパス を記憶する */
	if (pathSize > BT_PATH_MAX)
	{
		BTDPrintf( "LibBT argument error\n");
        return(BT_STATUS_FALSE);
	};
	memset(BTDeviceNamePath, 0x00, sizeof(BTDeviceNamePath));
	memcpy(&BTDeviceNamePath[0], deviceNamePath, pathSize);

	LibCTIMER_Init();

	ret = LibBM64_Init();
	if ( ret != BM64_STATUS_SUCCESS ){
        BTDPrintfLog( "LibBM64_Init exit %d.\n", ret);
    }
	ret = LibBM64_CallbackSet(EventCallback);
	if ( ret != BM64_STATUS_SUCCESS ){
        BTDPrintfLog( "LibBM64_CallbackSet exit %d.\n", ret);
        LibBT_Error();
     }

    //Read_Link_Status
    pBuffer[0] = 0x0d;
    pBuffer[1] = 0x00;
	BTInitError = 1;

	for(ii=0; ii<2; ii++){
		ret = LibBM64_SendUserCommand(&pBuffer[0], 2);
		if(ret == BM64_STATUS_SUCCESS){
			BTInitError = 0;
			break;
		}
	}

	if(BTInitError == 0){
		BTDPrintfLog( "LibBM64_start %d.\n", ret);

        //LibBT_Change_Device_Name(); //起動時は設定不要
        Read_Local_Device_Name();
		LibBT_NR_Setting();
		LibBT_stopBM64();

        //LibBT_startBM64();

		BTestablished  = 0;
		BTpairing      = 0;
		BTdisconnected = 0;
		//BTevent_mute   = 0;
		SCO_ready = 0;

	}
	else{
		BTDPrintfLog( "LibBM64_Error exit %d.\n", ret);
	}

	BTinitFlg = 1;

    return(BT_STATUS_SUCCESS);
}

static void LibBT_QueryDeviceName(void)
{
	uint8_t 			pBuffer[3];

	/*------------------------------------------------------*/
	/* BM64 Query Device Name								*/
	/* -----------------------------------------------------*/
    pBuffer[0] = 0x16;
    pBuffer[1] = 0x00;
    pBuffer[2] = 0x00;
	LibBM64_SendUserCommand(&pBuffer[0], 3);
}

static void LibBT_Read_Link_Status(void)
{
	uint8_t 			pBuffer[2];

	/*------------------------------------------------------*/
	/* BM64 Read Link Status								*/
	/* -----------------------------------------------------*/
    pBuffer[0] = 0x0d;
    pBuffer[1] = 0x00;
	LibBM64_SendUserCommand(&pBuffer[0], 2);
}

static void Read_Paired_Device_Record(void)
{
	uint8_t 			pBuffer[2];

	/*------------------------------------------------------*/
	/* BM64 Read Paired Device Record						*/
	/* -----------------------------------------------------*/
    pBuffer[0] = 0x0E;
    pBuffer[1] = 0x00;
	LibBM64_SendUserCommand(&pBuffer[0], 2);
}

static void Read_BTM_Setting(void)
{
	uint8_t 			pBuffer[3];

	/*------------------------------------------------------*/
	/* BM64 Read_BTM_Setting								*/
	/* -----------------------------------------------------*/
    pBuffer[0] = 0x24;
    pBuffer[1] = 0x02;
    pBuffer[2] = 0x00;
	LibBM64_SendUserCommand(&pBuffer[0], 3);
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_Control                                                           */
/*                                                                          */
/* 2.Description                                                            */
/*  LibBT_Control		(Bluetooth制御)                                     */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint32_t i_state                                                        */
/*    スタンバイモードフラグ                                                */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
uint32_t LibBT_Control( uint32_t i_state, uint32_t setControl, uint32_t keyLockState, uint32_t *bt_remote_processed_flag )
{
	int32_t 			ret;
	static int32_t 		BTsw           = 0;
	static int32_t 		BTswo          = 0;
	static int32_t		BTState = BT_MODESTATUS_IDLE;	/* BT制御状態 */
	static uint32_t		KeyLockStateOld = BT_KEY_LOCK;
	//getTimeStart();
	if(BTInitError == 1)
		return BTState;
	
	*bt_remote_processed_flag = BT_REM_PROC_FLAG_OFF;	//#外部制御プロトコルによるBT処理完了フラグ初期化

	/* キーロック処理 */
	if ( (KeyLockStateOld == BT_KEY_UNLOCK) && (keyLockState == BT_KEY_LOCK) ) {
		/* 強制的に待機中へ移行する */
		BTState = LibBT_setIdleMode();
		LibCTIMER_cancelTimer( "TM_BTON" );
	} else if ( keyLockState == BT_KEY_UNLOCK ) {
		/* BTスイッチ押下状態を取得 */
		BTswo = BTsw; 							/* BTスイッチ押下状態前回値 */
		BTsw  = LibBT_getKeyEvent();
	} else {
		/* 何もしない */
	}
	/* キーロックの前回値を更新する */
	KeyLockStateOld = keyLockState;

	/* スタンバイモード */
	if ( LibBT_IsStandbyMode( i_state ) ) {
		BTState = BT_MODESTATUS_IDLE;
		//getTimeEnd();
		return BTState;
	}

	switch( BTState ) {
		/* -----------------------------------------------------*/
		/* 待機中状態継続監視 									*/
		/* -----------------------------------------------------*/
		case(BT_MODESTATUS_IDLE):
			/* BTスイッチ監視 */
			if ( ( BTswo == SW_OFF ) && ( BTsw == SW_ON ) ) {
				LibCTIMER_setTimer( "TM_BTON", 3 * 1000);	/* BTスイッチONに切り替わったのでON状態監視タイマースタート */
			}
			else if (( BTsw == SW_ON ) && LibCTIMER_getTimeout("TM_BTON")) {	/* ON状態 3秒継続 */

				LibBT_Change_Device_Name();
                LibBT_startBM64();
				ret = LibBT_enterPairingMode();
				if ( ret != BM64_STATUS_SUCCESS ){
					BTDPrintfLog( "LibBM64_EnterPairingMode exit %d.\n", ret);
					LibBT_Error();
					break;
				}
				else {
					BTState = BT_MODESTATUS_PAIRING;
				}
			}
			else if ( setControl != BT_CONTROL_INVALID ) {		/* 制御指示あり */
				if ( setControl == BT_CONTROL_ON ) {
					LibBT_Change_Device_Name();
					LibBT_startBM64();
					ret = LibBT_enterPairingMode();
					if ( ret != BM64_STATUS_SUCCESS ){
						BTDPrintfLog( "LibBM64_EnterPairingMode exit %d.\n", ret);
						LibBT_Error();
						break;
					}
					else {
						BTState = BT_MODESTATUS_PAIRING;
					}
				}
				*bt_remote_processed_flag = BT_REM_PROC_FLAG_ON;	//外部制御プロトコルによるBT処理完了フラグ
			}
			else{
				if ( BTestablished != 0x0000 ){
					BTdisconnected = 0;
					BTState = BT_MODESTATUS_CONNECTED;			/* コールバック 接続確立 */
				}
			}
			break;
		/* -----------------------------------------------------*/
		/* ペアリングモード中状態継続監視 						*/
		/* -----------------------------------------------------*/
		case(BT_MODESTATUS_PAIRING):
			if ( BTestablished != 0x0000 ) {
				BTdisconnected = 0;						//接続中への状態遷移前に、意図しないdisconnectコールバックが発生した際の「接続→再接続→接続」の異常な状態遷移の発生を防ぐ。
				BTState = BT_MODESTATUS_CONNECTED;				/* コールバック 接続確立 	*/
			}
			else if ( ( BTpairing & 0x8000 ) == 0x8000 ) {
				BTState = LibBT_setIdleMode();					/* ペアリング失敗			*/
			}
			else {

				if (LibCTIMER_getTimeout("TM_PAIRING")) {		/* ペアリング中 60秒継続 	*/
					BTState = LibBT_setIdleMode();
				}
				else if ( ( BTswo == SW_OFF ) && ( BTsw == SW_ON ) ) {
					LibCTIMER_setTimer( "TM_BTON", 1 * 1000);	/* BTスイッチONに切り替わったのでON状態監視タイマースタート */
	    		}
				else if ( ( BTsw == SW_ON ) && LibCTIMER_getTimeout("TM_BTON") ) {		/* ON状態 1秒継続	*/
                    #if 0
                    ret = LibBM64_DisconnectAllLink();
    				if ( ret != BM64_STATUS_SUCCESS )
    	            {
    	                BTDPrintfLog( "LibBM64_DisconnectAllLink exit %d.\n", ret);
    	                LibBT_Error();
    	            }
				    ret = LibBM64_ExitPairingMode();
					if ( ret != BM64_STATUS_SUCCESS ){
						BTDPrintfLog( "LibBM64_ExitPairingMode exit %d.\n", ret);
						LibBT_Error();
					}
                    #endif
					BTState = LibBT_setIdleMode();										/* ペアリングを終了 */
				}
				else if ( setControl != BT_CONTROL_INVALID ) {	/* 制御指示あり */
#if 0
					//ペアリング登録中にBluetoothモード設定ONが送られた際に60秒タイマをリセットする場合は有効化してください。
					//動作未検証のため、正しく動作するかは確認してください。
					if ( setControl == BT_CONTROL_ON ) {
						/* ペアリング登録中のタイマをリセット */
						LibCTIMER_setTimer( "TM_PAIRING", 60 * 1000);
					}
					else if ( setControl == BT_CONTROL_OFF ) {
						BTState = LibBT_setIdleMode();
					}
#else
					//ペアリング登録中にBluetoothモード設定ONが送られた際には無視します。
					if ( setControl == BT_CONTROL_OFF ) {
						BTState = LibBT_setIdleMode();
					}
#endif
					*bt_remote_processed_flag = BT_REM_PROC_FLAG_ON;	//外部制御プロトコルによるBT処理完了フラグ
				}
			}
			break;
		/*------------------------------------------------------*/
		/* 接続中状態継続監視 									*/
		/*  ・手動切断はペアリングモード（5分）					*/
		/*  ・サイドパネルスイッチからの切断はOFF状態			*/
		/*------------------------------------------------------*/
		case(BT_MODESTATUS_CONNECTED):
			if ( BTdisconnected != 0x0000 ) {
				LibCTIMER_setTimer( "TM_DISCONNECT", (5 * 60 * 1000));
				BTestablished  = 0;
				BTpairing      = 0;
				BTdisconnected = 0;
				BTState = BT_MODESTATUS_RECONNECT;
				//LibBM64_LinkLastDevice();
			}
			else if ( ( BTswo == SW_OFF ) && ( BTsw == SW_ON ) ) {
				LibCTIMER_setTimer( "TM_BTON", 3 * 1000);			/* BTスイッチONに切り替わったのでON状態監視タイマースタート */
			}
			else if (( BTsw == SW_ON ) && LibCTIMER_getTimeout("TM_BTON")) {	/* ON状態 3秒継続 */
#if 0
                ret = LibBM64_DisconnectAllLink();
				if ( ret != BM64_STATUS_SUCCESS )
	            {
	                BTDPrintfLog( "LibBM64_DisconnectAllLink exit %d.\n", ret);
	                LibBT_Error();
	            }
			    ret = LibBM64_ExitPairingMode();
				if ( ret != BM64_STATUS_SUCCESS )
	            {
	                BTDPrintfLog( "LibBM64_ExitPairingMode exit %d.\n", ret);
	                LibBT_Error();
	            }
#endif
				BTState = LibBT_setIdleMode();									/* ペアリングを終了	*/
			}
			else if ( setControl != BT_CONTROL_INVALID ) {	/* 制御指示あり */
				if ( setControl == BT_CONTROL_OFF ) {
					BTState = LibBT_setIdleMode();
				}
				*bt_remote_processed_flag = BT_REM_PROC_FLAG_ON;	//外部制御プロトコルによるBT処理完了フラグ
			}


			if(SCO_ready == 1){
				LibBT_AEC_Setting();
				SCO_ready = 0;
			}

			break;
		/*------------------------------------------------------*/
		/* 再接続待ち状態継続監視 								*/
		/* -----------------------------------------------------*/
		case(BT_MODESTATUS_RECONNECT):
			if ( BTestablished != 0x0000 ) {
				BTdisconnected = 0;
				BTState = BT_MODESTATUS_CONNECTED;								/* 再接続判定		*/
			}
			else{
				if(LibCTIMER_getTimeout("TM_DISCONNECT")){
					//LibBT_resetMuteEven();
					BTState = LibBT_setIdleMode();								/* ペアリングを終了	*/
				}
				else if ( ( BTswo == SW_OFF ) && ( BTsw == SW_ON ) ) {
					LibCTIMER_setTimer( "TM_BTON", 3 * 1000);					/* BTスイッチONに切り替わったのでON状態監視タイマースタート 	*/
	    		}
				else if ( ( BTsw == SW_ON ) && LibCTIMER_getTimeout("TM_BTON") ) {	/* ON状態 3秒継続 												*/

					LibBT_stopBM64();												/* ペアリングモード移行で自動接続しないよう一度BM64をOFFする	*/

                    LibBT_startBM64();
                    ret = LibBT_enterPairingMode();
					if ( ret != BM64_STATUS_SUCCESS ){
						BTDPrintfLog( "LibBM64_EnterPairingMode exit %d.\n", ret);
						LibBT_Error();
						 BTState = BT_MODESTATUS_IDLE;
					}
					else{
						BTState = BT_MODESTATUS_PAIRING;
					}
				}
				else if ( setControl != BT_CONTROL_INVALID ) {	/* 制御指示あり */
					if ( setControl == BT_CONTROL_ON ) {
						LibBT_stopBM64();												/* ペアリングモード移行で自動接続しないよう一度BM64をOFFする	*/

						LibBT_startBM64();
						ret = LibBT_enterPairingMode();
						if ( ret != BM64_STATUS_SUCCESS ){
							BTDPrintfLog( "LibBM64_EnterPairingMode exit %d.\n", ret);
							LibBT_Error();
							BTState = BT_MODESTATUS_IDLE;
						}
						else{
							BTState = BT_MODESTATUS_PAIRING;
						}
					}
					*bt_remote_processed_flag = BT_REM_PROC_FLAG_ON;	//外部制御プロトコルによるBT処理完了フラグ
				}
			}
			break;

		default:
			BTState = LibBT_setIdleMode();																		/* アイドル状態に遷移 											*/
			LibBT_Error();
			break;
	}
#ifdef BM64_DEBUG
    if ( ( BTswo == SW_OFF ) && ( BTsw == SW_ON ) ) {
        LibCTIMER_setTimer( "TM_CHECK", 100);
    }
    else if (( BTsw == SW_OFF ) && LibCTIMER_getTimeout("TM_CHECK")) {
        BTDPrintf("\nBM64: BTState:");
        switch(BTState){
            case(BT_MODESTATUS_IDLE):
                BTDPrintf("IDLE\n");
                break;
            case(BT_MODESTATUS_PAIRING):
                BTDPrintf("PAIRING\n");
                break;
            case(BT_MODESTATUS_CONNECTED):
                BTDPrintf("CONNECTED\n");
                break;
            case(BT_MODESTATUS_RECONNECT):
                BTDPrintf("RECONNECT\n");
                break;
            default:
                BTDPrintf("[%x]\n",BTState);
        }
        //LibBT_QueryDeviceName();
        LibBT_Read_Link_Status();
        Read_Paired_Device_Record();
        Read_BTM_Setting();
        Read_Local_Device_Name();
    }
#endif
	/* -----------------------------------------------------*/
	/* LED制御 												*/
	/* -----------------------------------------------------*/
	LibBT_ControlLED(BTState);
	//getTimeEnd();

	return BTState;
}

#if 0
/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_getMuteEvent                                                      */
/*                                                                          */
/* 2.Description                                                            */
/*  LibBT_getMuteEven		(muteモードイベントの有無を返す)                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  0:none  1:muteイベントあり                                              */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t LibBT_getMuteEvent(void)
{
	int32_t 		ret;

	ret = BTevent_mute;
	BTevent_mute = 0;

	return ret;
}
#endif

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_DefKeyLock                                                        */
/*  LibBT_DefKeyUnLock                                                      */
/*  LibBT_DefControlInvalid                                                 */
/*  LibBT_DefControlOn                                                      */
/*  LibBT_DefControlOff                                                     */
/*  LibBT_DefModeStatusNone                                                 */
/*  LibBT_DefModeStatusIdle                                                 */
/*  LibBT_DefModeStatusPairing                                              */
/*  LibBT_DefModeStatusConnected                                            */
/*  LibBT_DefModeStatusReconnect                                            */
/*                                                                          */
/* 2.Description                                                            */
/*  各種の定義値を返す　　　　　　                                          */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  LibBT_DefKeyLock               ロック                                   */
/*  LibBT_DefKeyUnLock             解除                                     */
/*  LibBT_DefControlInvalid        指示なし                                 */
/*  LibBT_DefControlOn             ON                                       */
/*  LibBT_DefControlOff            OFF                                      */
/*  LibBT_DefModeStatusNone        Bluetooth状態：不定                      */
/*  LibBT_DefModeStatusIdle        Bluetooth状態：待機中                    */
/*  LibBT_DefModeStatusPairing     Bluetooth状態：ペアリング中              */
/*  LibBT_DefModeStatusConnected   Bluetooth状態：接続中                    */
/*  LibBT_DefModeStatusReconnect   Bluetooth状態：再接続待ち                */
/*                                                                          */
/*--------------------------------------------------------------------------*/
uint32_t LibBT_DefKeyLock(void)
{
	return BT_KEY_LOCK;
}

uint32_t LibBT_DefKeyUnLock(void)
{
	return BT_KEY_UNLOCK;
}

uint32_t LibBT_DefControlInvalid(void)
{
	return BT_CONTROL_INVALID;
}

uint32_t LibBT_DefControlOn(void)
{
	return BT_CONTROL_ON;
}

uint32_t LibBT_DefControlOff(void)
{
	return BT_CONTROL_OFF;
}

uint32_t LibBT_DefModeStatusNone(void)
{
	return BT_MODESTATUS_NONE;
}

uint32_t LibBT_DefModeStatusIdle(void)
{
	return BT_MODESTATUS_IDLE;
}

uint32_t LibBT_DefModeStatusPairing(void)
{
	return BT_MODESTATUS_PAIRING;
}

uint32_t LibBT_DefModeStatusConnected(void)
{
	return BT_MODESTATUS_CONNECTED;
}

uint32_t LibBT_DefModeStatusReconnect(void)
{
	return BT_MODESTATUS_RECONNECT;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_setIdleMode	                                                    */
/*                                                                          */
/* 2.Description                                                            */
/*  アイドル状態への遷移                                                    */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t LibBT_setIdleMode(void)
{
	Ctrl_BluetoothLED( LED_OFF );
	LibBT_stopBM64();

	//LibBT_resetMuteEven();
	BTestablished  = 0;
	BTpairing      = 0;
	BTdisconnected = 0;

	return BT_MODESTATUS_IDLE;
}
/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_enterPairingMode                                                  */
/*                                                                          */
/* 2.Description                                                            */
/*  ペアリングモードに入る                                                  */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  0:none  1:muteイベントあり                                              */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t LibBT_enterPairingMode(void)
{
	int32_t ret;

    #if 1
	ret = LibBM64_EnterPairingMode();
	if ( ret == BM64_STATUS_SUCCESS ){
		/* ペアリング中 */
		LibCTIMER_setTimer( "TM_PAIRING", 60 * 1000);
		BTestablished  = 0;
		BTpairing      = 0;
		BTdisconnected = 0;
	}
    #endif
    #if 0
    LibBM64_LinkLastDevice();
    #endif


	return ret;
}
/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*   LibBT_startBM64	                                                    */
/*                                                                          */
/* 2.Description                                                            */
/* ペアリングモード移行で電源ONするため未使用　※参考処理と					*/
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void LibBT_startBM64(void)
{
	uint8_t    pBuffer[3];

	/*------------------------------------------------------*/
	/* BM64 ON												*/
	/* -----------------------------------------------------*/

    //pBuffer[0] = 0x02;
    //pBuffer[1] = 0x00;
    //pBuffer[2] = 0x51;
	//LibBM64_SendUserCommand(&pBuffer[0], 3);

    pBuffer[0] = 0x02;
    pBuffer[1] = 0x00;
    pBuffer[2] = 0x52;
	LibBM64_SendUserCommand(&pBuffer[0], 3);

}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*   LibBT_stopBM64	                                                    	*/
/*                                                                          */
/* 2.Description                                                            */
/*  BM64　POWEROFF										                    */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void LibBT_stopBM64(void)
{
	uint8_t	pBuffer[3];
#if 1
	/*------------------------------------------------------*/
	/* BM64 OFF												*/
	/* -----------------------------------------------------*/
    pBuffer[0] = 0x02;
    pBuffer[1] = 0x00;
    pBuffer[2] = 0x53;	//power off button press
	LibBM64_SendUserCommand(&pBuffer[0], 3);

//	pBuffer[0] = 0x02;
//	pBuffer[1] = 0x00;
//	pBuffer[2] = 0x54;	//power off button Release
//	LibBM64_SendUserCommand(&pBuffer[0], 3);
#endif
}

#if 0
/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_resetMuteEven                                                     */
/*                                                                          */
/* 2.Description                                                            */
/*  LibBT_resetMuteEven		(muteモードイベントを消す)                      */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  0:none  1:muteイベントあり                                              */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void LibBT_resetMuteEven(void)
{
	BTevent_mute = 0;
}
#endif

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_IsStandbyMode                                                     */
/*                                                                          */
/* 2.Description                                                            */
/*  LibBT_IsStandbyMode		(スタンバイモード判定)                          */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint32_t i_state		スタンバイモードフラグ                          */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t					(0:通常モード、1:スタンバイモード)              */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t LibBT_IsStandbyMode( uint32_t i_state )
{
	int32_t	ret = 0;
	static uint32_t	i_stateo = 0;		/* スタンバイモード前回値 */

	if ( i_state == 1 ) {
		if ( i_stateo != i_state ) {
			//LibBT_resetMuteEven();
			LibBT_stopBM64();
			Ctrl_BluetoothLED( LED_OFF );
			LibCTIMER_cancelTimer( "TM_BTON" );
		}
		ret = 1;
	}
	else{
		if ( i_stateo != i_state ) {
			BTestablished  = 0;
			BTpairing      = 0;
			BTdisconnected = 0;

		}
	}
	i_stateo = i_state;				/* スタンバイモード前回値更新 */

	return(ret);
}


/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_getKeyEvent                                                       */
/*                                                                          */
/* 2.Description                                                            */
/*  LibBT_getKeyEvent		(チャタリングチェック2回)                       */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t LibBT_getKeyEvent(void)
{
	static uint32_t current = 0;
	static uint32_t status  = 0;

	current <<= 1;
	current |= Get_BluetoothSwitch();

	if(status == 0){									/* OFF						*/
														/* off -> on 				*/
		if((current & 0x0003) == 0x0003){				/* 2回チェック				*/
			status = 1;
		}

	}
	else if ((current & 0x0003) == 0x0000){				/* ON -> OFF(連続取得2回)	*/
			status = 0;
	}
	else{

	};

    return status;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_ControlLED                                                        */
/*                                                                          */
/* 2.Description                                                            */
/*  LibBT_ControlLED		(LED制御)                                       */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  IN モジュールステータス                                                 */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void LibBT_ControlLED( int32_t BTmode )
{
	static int32_t BTStateo = 0;

	if ( BTStateo != BTmode ) {
		if (BTmode == BT_MODESTATUS_PAIRING) {					/* ペアリング + 再接続中の点滅開始 	*/
			Ctrl_BluetoothLED( LED_ON );
			LibCTIMER_setTimer( "TM_LEDOFF", 200);
		}
	}

	if ( BTmode == BT_MODESTATUS_CONNECTED ) {
			/* 点灯 接続中 で 点灯 */
			Ctrl_BluetoothLED( LED_ON );
	}

	else if ( BTmode == BT_MODESTATUS_PAIRING ) {				/* ペアリング+再接続中は点滅する 	*/
		/* LED点滅処理 */
		if (LibCTIMER_getTimeout("TM_LEDOFF")) {				/* LED ON状態 200ミリ秒継続 		*/
			/* LED消灯 点灯タイマ開始 */
			Ctrl_BluetoothLED( LED_OFF );
			LibCTIMER_setTimer( "TM_LEDON", 200);
		}

		else if (LibCTIMER_getTimeout("TM_LEDON")) {			/* LED OFF状態 200ミリ秒継続 		*/
			/* LED点灯 消灯タイマ開始 */
			Ctrl_BluetoothLED( LED_ON );
			LibCTIMER_setTimer( "TM_LEDOFF", 200);
		}
		else{

		};
	}
	else {
		/* その他の状態（待機中+再接続待ち）では消灯 */
		Ctrl_BluetoothLED( LED_OFF );
	};

	BTStateo = BTmode;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibBT_Error                                                             */
/*                                                                          */
/* 2.Description                                                            */
/*  エラー発生時にDeinit+Initを実行											*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void LibBT_Error(void)
{

	int32_t ret;

    BTDPrintfLog( "!!! LibBT_Error !!!\n");

	ret = LibBM64_DisconnectAllLink();
	if ( ret != BM64_STATUS_SUCCESS ){
		BTDPrintfLog( "LibBM64_DisconnectAllLink exit %d.\n", ret);
	}

    ret = LibBM64_ExitPairingMode();
	if ( ret != BM64_STATUS_SUCCESS ){
		BTDPrintfLog( "LibBM64_ExitPairingMode exit %d.\n", ret);
	}

    ret = LibBM64_Deinit();
	if ( ret != BM64_STATUS_SUCCESS ){
		BTDPrintfLog( "LibBM64_Deinit exit %d.\n", ret);
	}

	ret = LibBM64_Init();
	if ( ret != BM64_STATUS_SUCCESS ){
		BTDPrintfLog( "LibBM64_Init exit %d.\n", ret);
	}

	ret = LibBM64_CallbackSet(EventCallback);
	if ( ret != BM64_STATUS_SUCCESS ){
		BTDPrintfLog( "LibBM64_CallbackSet exit %d.\n", ret);
		LibBM64_Deinit();
	}
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  EventCallback                                                           */
/*                                                                          */
/* 2.Description                                                            */
/*  BM64Lib call back function				                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t EventCallback( uint8_t *pBuffer, int32_t Length )
{
    int32_t i;

    BTDPrintf("BM64: ");

	if ( pBuffer[0] == 0x01 ) {
        if(pBuffer[1] == 0x00){
            BTDPrintf("BTM_Status: Power OFF state\n");
        }
        else if(pBuffer[1] == 0x01){
            BTDPrintf("BTM_Status: pairing state (discoverable mode)\n");
        }
        else if(pBuffer[1] == 0x02){
            BTDPrintf("BTM_Status: Power ON state -");
            if ( pBuffer[2] == 0 ) {
				BTDPrintf( "Power on\n");
			} else {
				BTDPrintf( "Already power on\n");
			}

        }
        else if(pBuffer[1] == 0x09){
			BTDPrintf("\n\n!!!!!\n\nBTM_Status: SCO link connected\n");
			SCO_ready = 1;
        }
        else if(pBuffer[1] == 0x0D){
            BTDPrintf("BTM_Status: Standard SPP connected\n");
        }
        else if(pBuffer[1] == 0x10){
            BTDPrintf("BTM_Status: iAP connected\n");
        }
        else if(pBuffer[1] == 0x12){
            BTDPrintf("BTM_Status: MAP connected\n");
        }
        else if(pBuffer[1] == 0x13){
            BTDPrintf("BTM_Status: MAP operation forbidden\n");
        }
        else if(pBuffer[1] == 0x15){
			BTDPrintf("BTM_Status: ACL connected\n");
        }
		else if ( pBuffer[1] == 0x03 ) {
			BTpairing |= 0x0001;
			BTDPrintf( "BTM_Status: pairing successful\n");
		}
        else if ( pBuffer[1] == 0x04 ) {
			BTpairing |= 0x8000;
			BTDPrintf( "BTM_Status: pairing fail\n");
		}
        else if ( pBuffer[1] == 0x05 ) {
			if ( pBuffer[2] == 0 ) {
				BTestablished |= 0x0001;
				BTDPrintf( "BTM_Status: HF/HS link established (HSP)\n");
			} else {
				BTestablished |= 0x0002;
				BTDPrintf( "BTM_Status: HF/HS link established (HFP)\n");
			}
		}
        else if ( pBuffer[1] == 0x06 ) {
			BTestablished |= 0x0004;
			BTDPrintf( "BTM_Status: A2DP link established\n");
		}
        else if ( pBuffer[1] == 0x07 ) {
			//BTdisconnected |= 0x0001;
			BTDPrintf( "BTM_Status: HF link disconnected\n");
		}
        else if ( pBuffer[1] == 0x08 ) {
			//BTdisconnected |= 0x0002;
			BTDPrintf( "BTM_Status: A2DP link disconnected\n");
		}
        else if ( pBuffer[1] == 0x0A ) {
			//BTdisconnected |= 0x0004;		/* SCO切断ではDisconnectしない（skype対応）*/
			BTDPrintf( "BTM_Status: SCO link disconnected\n");
		}
        else if ( pBuffer[1] == 0x0B ) {
			BTestablished |= 0x0008;
			BTDPrintf( "BTM_Status: AVRCP link established\n");
		}
        else if ( pBuffer[1] == 0x0C ) {
			//BTdisconnected |= 0x0008;
			BTDPrintf( "BTM_Status: AVRCP link disconnected\n");
		}
        else if ( pBuffer[1] == 0x0E ) {
			//BTdisconnected |= 0x0010;
			BTDPrintf( "BTM_Status: Standard_SPP / iAP disconnected\n");
		}
        else if ( pBuffer[1] == 0x11 ) {
			if ( pBuffer[2] == 0 ) {
				BTdisconnected |= 0x0020;
				BTDPrintf( "BTM_Status: ACL disconnected (disconnection)\n");
			}
            else {
				BTdisconnected |= 0x0040;
				BTDPrintf( "BTM_Status: ACL disconnected (link loss)\n");
			}
		}
        else if ( pBuffer[1] == 0x14 ) {
			//BTdisconnected |= 0x0080;
			BTDPrintf( "BTM_Status: MAP disconnected\n");
		}
        else if ( pBuffer[1] == 0x0F ) {
			BTdisconnected |= 0x0001;
			BTDPrintf( "BTM_Status: Standby state\n");
		}
        else {
			BTDPrintf( "BTM_Status undefined {%x},{%x},{%x}\n", pBuffer[0], pBuffer[1], pBuffer[2]);
		}
	}
    else if ( pBuffer[0] == 0x29 ) {
		BTDPrintf( "AVRCP_ABS_VOL_CTRL: abs_vol {%d}\n", pBuffer[2]);
#if 0
        if ( pBuffer[2] == 0 ) {
			BTevent_mute = 1;
			BTDPrintf( "Mute;debug \n");
		}
#endif
	}

    else if(pBuffer[0] == 0x02){
        BTDPrintf("Call_Status: ");
        switch (pBuffer[2]) {
            case(0):
                BTDPrintf("Idle\n");
                break;
            case(1):
                BTDPrintf("voice dial\n");
                break;
            case(2):
                BTDPrintf("incoming call\n");
                break;
            case(3):
                BTDPrintf("outgoing call\n");
                break;
            case(4):
                BTDPrintf("call active\n");
                break;
            case(5):
                BTDPrintf("a call active with a call waiting\n");
                break;
            case(6):
                BTDPrintf("a call active with a call hold\n");
                break;
        }
    }
    else if(pBuffer[0] == 0x06){
		BTDPrintf("Phone_Max_Battery_Level: Max Battery {%d}\n",pBuffer[2]);
    }
	else if(pBuffer[0] == 0x07){
		BTDPrintf("Phone_Current_Battery_Level: Current Battery {%d}\n",pBuffer[2]);
    }
	else if(pBuffer[0] == 0x08){
		if(pBuffer[2] == 0){
			BTDPrintf("Roaming_Status: non-roaming\n");
        }
		else{
			BTDPrintf("Roaming_Status: roaming\n");
        }
    }
	else if(pBuffer[0] == 0x09){
		BTDPrintf("Phone_Max_Signal_Strength_Level: signal {%d}\n",pBuffer[2]);
    }
	else if(pBuffer[0] == 0x0A){
		BTDPrintf("Phone_Current_Signal_Strength_Level: signal {%d}\n",pBuffer[2]);
    }
	else if(pBuffer[0] == 0x0B){
		if(pBuffer[2] == 0){
			BTDPrintf("Phone_Service_Status: No service available\n");
        }
		else{
			BTDPrintf("Phone_Service_Status: service available\n");
        }
    }
	else if(pBuffer[0] == 0x0C){
		BTDPrintf("BTM_Battery_Status: {%x}{%x}\n",pBuffer[1],pBuffer[2]);
    }
	else if(pBuffer[0] == 0x0D){
		BTDPrintf("BTM_Charging_Status: {%x}{%x}\n",pBuffer[1],pBuffer[2]);
    }
	else if(pBuffer[0] == 0x0F){
		BTDPrintf("Report_HF_Gain_Level: gain {%d}\n",pBuffer[2]);
    }
	else if(pBuffer[0] == 0x10){
		BTDPrintf("EQ_Mode_Indication: mode {%d}\n",pBuffer[1]);
    }
	else if(pBuffer[0] == 0x17){
        BTDPrintf("Read_Linked_Device_Information_Reply: ");
        switch (pBuffer[2]) {
            case(0):
                BTDPrintf("reply device name\n");
                BTDPrintf(": {%s}\n",&pBuffer[3]);
                break;
            case(1):
                BTDPrintf("reply in-band ringtone status\n");
                break;
            case(2):
                BTDPrintf("reply if remote device is a iAP device\n");
                break;
            case(3):
                BTDPrintf("reply remote device supported AVRCP function");
                BTDPrintf(":0x%x ",pBuffer[3]);
                BTDPrintf("bit0:media player status bit1:media player status notification\n");
                break;
            case(4):
                BTDPrintf("reply HF&A2DP gain\n");
                break;
            case(5):
                BTDPrintf("reply Line_In gain\n");
                break;
            case(6):
                BTDPrintf("reply A2DP used codec type\n");
                break;
        }
    }
	else if(pBuffer[0] == 0x1A){
		BTDPrintf("AVC_Specific_Rsp (avc_rsp_payload {%d})\n",pBuffer[2]);
    }
	else if(pBuffer[0] == 0x1B){
		BTDPrintf("BTM_Utility_Req (action_type:{%x} parameter:{%x)\n",pBuffer[1],pBuffer[2]);
	}
    else if(pBuffer[0] == 0x1E){
        BTDPrintf("Read_Link_Status_Reply: ");
        switch (pBuffer[1]) {
            case(0):
                BTDPrintf("Power OFF state");
                break;
            case(1):
                BTDPrintf("pairing state (discoverable mode)");
                break;
            case(2):
                BTDPrintf("standby state");
                break;
            case(3):
                BTDPrintf("Connected state with only HF profile connected");
                break;
            case(4):
                BTDPrintf("Connected state with only A2DP profile connected");
                break;
            case(5):
                BTDPrintf("Connected state with only A2DP profile connected");
                break;
            case(6):
                BTDPrintf("Connected state with multi-profile connected");
                break;
        }
        BTDPrintf(" :0x%x,0x%x,0x%x,0x%x,0x%x,0x%x\n",pBuffer[2],pBuffer[3],pBuffer[4],pBuffer[5],pBuffer[6],pBuffer[7],pBuffer[6],pBuffer[7]);
    }
    else if(pBuffer[0] == 0x1F){
        BTDPrintf("Read_Paired_Device_Record_Reply: ");
        BTDPrintf(" the paired device number[%x]",pBuffer[1]);
        BTDPrintf(" link priority[%x]",pBuffer[2]);
        BTDPrintf(" linked device BD address 0x%x,0x%x,0x%x,0x%x,0x%x,0x%x\n",pBuffer[3],pBuffer[4],pBuffer[5],pBuffer[6],pBuffer[7],pBuffer[8]);
    }
    else if(pBuffer[0] == 0x21){
        BTDPrintf("Read_Local_Device_Name_Reply: ");
        BTDPrintf("name length[%x]",pBuffer[1]);
        i = pBuffer[1];
        pBuffer[i+2]=0x00;
        BTDPrintf(": {%s}\n",&pBuffer[2]);
        //BTDPrintf(" 0x%x,0x%x,0x%x,0x%x,0x%x,0x%x\n",pBuffer[2],pBuffer[3],pBuffer[4],pBuffer[5],pBuffer[6],pBuffer[7]);
    }
    else if(pBuffer[0] == 0x23){
		BTDPrintf("Report_Link_Back_Status: ");
        switch (pBuffer[1]) {
            case(0):
            BTDPrintf("ACL connection");
            break;
            case(1):
            BTDPrintf("HF connection");
            break;
            case(2):
            BTDPrintf("A2DP connection");
            break;
            case(3):
            BTDPrintf("SPP connection");
            break;
            case(4):
            BTDPrintf("under page state");
            break;
        }
        BTDPrintf(": link back status:{%x}\n",pBuffer[2]);
    }
	else if(pBuffer[0] == 0x24){
		if(pBuffer[1] == 0){
			BTDPrintf("Ringtone_Finish_Indicate (ringtone stop)\n");
        }
		else{
			BTDPrintf("Ringtone_Finish_Indicate (ringtone start)\n");
        }
    }
    else if(pBuffer[0] == 0x2D){
        BTDPrintf("REPORT_TYPE_CODEC: ");

        BTDPrintf("sample: ");
        switch (pBuffer[1]) {
            case(0):
                BTDPrintf("8kHz ");
                break;
            case(1):
                BTDPrintf("12kHz ");
                break;
            case(2):
                BTDPrintf("16kHz ");
                break;
            case(3):
                BTDPrintf("24kHz ");
                break;
            case(4):
                BTDPrintf("32kHz ");
                break;
            case(5):
                BTDPrintf("48kHz ");
                break;
            case(6):
                BTDPrintf("44.1kHz ");
                break;
            case(7):
                BTDPrintf("88kHz ");
                break;
            case(8):
                BTDPrintf("96kHz ");
                break;
            case(9):
                BTDPrintf("reserved ");
                break;
        }
        BTDPrintf("mode: ");
        switch (pBuffer[1]) {
            case(0):
                BTDPrintf("no action\n");
                break;
            case(1):
                BTDPrintf("prepare\n");
                break;
            case(2):
                BTDPrintf("audio in mode\n");
                break;
            case(3):
                BTDPrintf("PCM mode\n");
                break;
            case(4):
                BTDPrintf("A2DP decode mode\n");
                break;
            case(5):
                BTDPrintf("A2DP encode mode\n");
                break;
            case(6):
                BTDPrintf("SCO(HF)\n");
                break;
            case(7):
                BTDPrintf("tone\n");
                break;
            case(8):
                BTDPrintf("aux(voice prompt) tone\n");
                break;
            case(9):
                BTDPrintf("reserved\n");
                break;
        }
    }
    else if(pBuffer[0] == 0x2E){
        BTDPrintf("REPORT_TYPE_BTM_SETTING (0x%x,0x%x)\n",pBuffer[1],pBuffer[2]);
    }
    else if(pBuffer[0] == 0x30){
        BTDPrintf("DSP_Runtime_Program (Type {%d})\n",pBuffer[1]);
    }
    else if(pBuffer[0] == 0x32){
        BTDPrintf("LE_Signaling_Event (SubEvent Type {%d})\n",pBuffer[1]);
    }
    else {
		BTDPrintf( "undefined {%x},{%x},{%x}\n", pBuffer[0], pBuffer[1], pBuffer[2]);
	}
	return 0;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  BTDPrintf	　デバッグコンソール出力                                    */
/*  BTDPrintLog	  デバッグログ出力		                                    */
/*                                                                          */
/* 2.Description                                                            */
/*  BTDPrintf	　デバッグコンソール出力                                    */
/*  BTDPrintLog	  デバッグログ出力		                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void BTDPrintf( const char *Fmt, ... )
{
#ifdef BM64_DEBUG
    int8_t str[128];
    va_list ap;

	va_start( ap, Fmt );
	vsprintf( str, Fmt, ap );
	va_end( ap );
	printf( str );
#endif
}

static void BTDPrintfLog( const char *Fmt, ... )
{
#if 0
    int8_t str[512];
    int8_t date[64];
    va_list ap;

	time_t t = time(NULL);
	strftime( date, sizeof(date), "%Y-%m-%d %H:%M:%S", localtime(&t) );

	va_start( ap, Fmt );
	vsprintf( str, Fmt, ap );
	va_end( ap );
	printf( "%s - %s", date, str );

	FILE* file = fopen("error.log", "a");

	if ( file != NULL ) {
		fprintf( file,  "%s - %s", date, str );
		fclose( file );
	}

//#else
    int8_t str[512];
    va_list ap;

	va_start( ap, Fmt );
	vsprintf( str, Fmt, ap );
	va_end( ap );
	printf( str );
#endif
}
