#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include "LibCF1CommCtrl.h"
#include "CommStatusConverter.h"
#include "UtilityforDebug.h"

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  ConvertCommStatusApp2Lib		                       	                */
/*                                                                          */
/* 2.Description                                                            */
/*  機器状態構造体 値変換処理				                                */
/*	（AM-CF1アプリ→CommCtrlライブラリ）									*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*	const stCommStatus*	src		変換元構造体ポインタ						*/
/*	      stCommStatus*	dst		変換先構造体ポインタ						*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
void ConvertCommStatusApp2Lib(const stCommStatus* src, stCommStatus* dst)
{
	dst->gain = src->gain;
	dst->muteMic = src->muteMic;
	dst->muteSpk = src->muteSpk;
	dst->bluetoothMode = src->bluetoothMode;
	dst->standby = src->standby;
	// 以下の通り、読み替えが必要となる。
	// DSPのx座標 = 外部制御プロトコルのy座標(奥行座標)
	// DSPのy座標 = 外部制御プロトコルのx座標(幅方向座標)
	dst->focus_x = src->focus_y;
	dst->focus_y = src->focus_x;
	dst->focus_mode = src->focus_mode;
	dst->focus_direction = src->focus_direction;
	dst->focus_distance = src->focus_distance;
	dst->focus_unit = src->focus_unit;
	// PRESET変換
	switch (src->preset) {
	case 0x10:
		dst->preset = 0;
		break;
	case 0x11:
		dst->preset = 1;
		break;
	default:
																DBGPrintf("[%s] preset is invalid value(0x%02x).\n", __FUNCTION__, src->preset);
		dst->preset = 0;
		break;
	}
	// 自動ステイタス通知で応答するために情報を取得する
	dst->isPresetAutoResponse = src->isPresetAutoResponse;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  ConvertCommStatusLib2App		                       	                */
/*                                                                          */
/* 2.Description                                                            */
/*  機器状態構造体 値変換処理				                                */
/*	（CommCtrlライブラリ→AM-CF1アプリ）									*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*	const stCommStatus*		src		変換元構造体ポインタ					*/
/*	const stCommStatusFlg*	flgs	機器状態フラグ構造体ポインタ			*/
/*	      stCommStatus*		dst		変換先構造体ポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
void ConvertCommStatusLib2App(const stCommStatus* src, const stCommStatusFlg* flgs, stCommStatus* dst)
{
	// GAIN変換
	if (flgs->gainUpdated) {
		dst->gain = src->gain;
	} else {
		dst->gain = GAIN_INVALID;
	}

	// MUTE(MIC)変換
	if (flgs->muteMicUpdated) {
		dst->muteMic = ((src->muteMic == 1) ? MUTE_ON : MUTE_OFF);
	} else {
		dst->muteMic = MUTE_INVALID;
	}

	// MUTE(SPK)変換
	if (flgs->muteSpkUpdated) {
		dst->muteSpk = ((src->muteSpk == 1) ? MUTE_ON : MUTE_OFF);
	}
	else {
		dst->muteSpk = MUTE_INVALID;
	}

	// BLUETOOTH変換
	if (flgs->bluetoothUpdated) {
		dst->bluetoothMode = ((src->bluetoothMode == 1) ? BLUETOOTH_ON : BLUETOOTH_OFF);
	}
	else {
		dst->bluetoothMode = BLUETOOTH_INVALID;
	}

	// STANDBY変換
	if (flgs->standbyUpdated) {
		dst->standby = ((src->standby == 1) ? STANDBY_ON : STANDBY_OFF);
	}
	else {
		dst->standby = STANDBY_INVALID;
	}

	// FOCUS変換
	if (flgs->focusSetUpdated) {
		dst->focus_mode = ((src->focus_mode == 0) ? FOCUS_SET_AUTO : FOCUS_SET_MANUAL);
		dst->focus_direction = src->focus_direction;
		dst->focus_distance = src->focus_distance;
		dst->focus_unit = src->focus_unit;
	}
	else {
		dst->focus_mode = FOCUS_SET_INVALID;
	}

	// PRESET変換
	if (flgs->presetUpdated) {
		dst->preset = ((src->preset == 0) ? PRESET_1 : PRESET_2);
	}
	else {
		dst->preset = PRESET_INVALID;
	}
}
