import json
import collections as cl
import re
import sys
import subprocess as sp
import copy
import os, time
import numpy as np

from hello.send_command import send_command
import hello.def_block_no as BlockNo
from hello.param_set_get_flag import Wait_Param_Get_Flag
from hello.param_set_get_flag import Wait_Param_Set_Flag

# Countermeasure for continuous hits of APPLY button
g_mic_level_flag = False
g_aec_param_flag = False
s_aec_param_flag = False
g_version_flag = False

#--------------------------------------------------------------------#
def Get_Mic_Level(data,thisNAME_PATH):
    global g_mic_level_flag
    if g_mic_level_flag==True:
        #print('G_LEVEL ->',g_mic_level_flag) #kura DEBUG
        return
    g_mic_level_flag=True

    PATH = os.path.join(thisNAME_PATH, '../../tmp/')
    netlist_file_name = thisNAME_PATH + 'netlist.bin'
    control_out = thisNAME_PATH + 'control.out'

    command_file_name = PATH + '/command/mic_level.json'
    result_file_name = PATH + '/result/mic_level.json'

    blocknums = [
        BlockNo.RAW_LEV_MIC1,BlockNo.RAW_LEV_MIC2,BlockNo.RAW_LEV_MIC3,BlockNo.RAW_LEV_MIC4,
        BlockNo.RAW_LEV_MIC5,BlockNo.RAW_LEV_MIC6,BlockNo.RAW_LEV_MIC7,BlockNo.RAW_LEV_MIC8
        ]
    # if 'get' then L and R gain ( because L!=R )

    # --- command.jsonを作成 ---
    command = {
        'access': 'get',
        'block_no': None,
        'type': 'peak',
        'status': {'level':None}
        }

    commands = []

    for i in range(0,len(blocknums)):
        commands.append(copy.copy(command))
        commands[i]['block_no'] = blocknums[i]

    drcgain_command =[{
        'access': 'set',
        'block_no': BlockNo.AEC,
        'type': 'aec',
        'param': {'param_get_flag':1},
        'status': {}
        },
        {
        'access': 'get',
        'block_no': BlockNo.AEC,
        'type': 'aec',
        'param': {'drc_gain':None},
        'status': {}
        }]

    commands.extend(drcgain_command)

    #ビーム取得用 command.jsonを作成 ---
    beam_commands = [{
        'access': 'get',
        'block_no': BlockNo.BF,
        'type': 'microphonearray',
        'status': { 'bf' : {'focus_x':None, 'focus_y':None}}
        }]

    commands.extend(beam_commands)

    with open(command_file_name, 'w') as f:
        # 辞書型をjsonとして出力
        json.dump(commands, f, indent=True)

    #print('SEND START')
    # --- control.out を実行 ---
    mic_level = [0,0,0,0,0,0,0,0] #マイク数8 先に0dBを代入
    drc_gain_val = 0
    status_x_val = 0
    status_y_val = 0
    st_time = time.perf_counter()
    cnt = 0
    diff_time = 0
    while diff_time <= 1.0:
        Wait_Param_Get_Flag()
        
        send_command(command_file_name,result_file_name)

        #print('SEND END')
        with open(result_file_name, "r") as file:
            ws_send_rdata = json.load(file)

        #levelは、NaNまたはInfinityを返すことがある
        #1秒のうち1回NaNまたはInfinityが帰った場合、平均値を取得せずにErrorを返す
        for i in range(0,len(blocknums)) :
            if ws_send_rdata[i]["status"]["level"] == "NaN" or ws_send_rdata[i]["status"]["level"] == "Infinity":
                #異常時
                mic_level[i] = ws_send_rdata[i]["status"]["level"]
            else:
                if mic_level[i] == "NaN" or mic_level[i] == "Infinity":
                    #異常時はスキップ
                    continue
                else:
                    #正常時
                    mic_level[i] = mic_level[i] + float(ws_send_rdata[i]["status"]["level"])
            #print(str(float(ws_send_rdata[i]["status"]["level"])))

        drc_gain_val = drc_gain_val + float(ws_send_rdata[9]["param"]["drc_gain"])

        status_x_val = status_x_val + ws_send_rdata[10]["status"]["bf"]["focus_x"]
        status_y_val = status_y_val + ws_send_rdata[10]["status"]["bf"]["focus_y"]
        
        ed_time = time.perf_counter()
        diff_time = ed_time - st_time
        cnt = cnt + 1

    #print('mic level get ->', ed_time - st_time)
    ws_send_data = []
    ws_send_data.append("insp_miclevel")

    #平均値計算
    for i in range(0,len(blocknums)) :
        if mic_level[i] == "NaN" or mic_level[i] == "Infinity":
            #異常時
            ws_send_data.append(str("Error"))
        else:
            #正常時
            ws_send_data.append(str("{0:4.1f}".format(mic_level[i]/cnt)))

    ws_send_data.append(str("{0:4.2f}".format(drc_gain_val/cnt)))

    bf_x = "{0:4.2f}".format(status_x_val/cnt)
    bf_y = "{0:4.2f}".format(status_y_val/cnt)

    ws_send_data.append(str(bf_x))
    ws_send_data.append(str(bf_y))

    ws_send = ",".join(ws_send_data)

    g_mic_level_flag=False

    return ws_send


### inspection ###
def Get_Aec_params(data,thisNAME_PATH):
    global g_aec_param_flag
    if g_aec_param_flag==True:
        #print('G_LEVEL ->',g_mic_level_flag) #kura DEBUG
        return
    g_aec_param_flag=True

    PATH = os.path.join(thisNAME_PATH, '../../tmp/')
    netlist_file_name = thisNAME_PATH + 'netlist.bin'
    control_out = thisNAME_PATH + 'control.out'

    command_file_name = PATH + '/command/get_aec_params.json'
    result_file_name = PATH + '/result/get_aec_params.json'

    FAIL_TIME = 1000
    blocknums = [BlockNo.AEC,BlockNo.AEC]
    # if 'get' then L gain only ( because L=R )

    # --- command.jsonを作成 ---
    command = {
        'access': 'get',
        'block_no': None,
        'type': None,
        'param': {}
    }

    commands = []

    for i in range(0,len(blocknums)):
        commands.append(copy.copy(command))
        commands[i]['block_no'] = blocknums[i]

    commands[0]['type'] = 'aec'
    commands[0]['param'] = {'on':None,'mu_':None,'errmu_limit':None,'rcv_dly_':None,'nlp_ratio0':None,'nlp_ratio1':None,
                            'nlp_enhance0':None,'nlp_enhance1':None,'nlp_alpha0':None,'nlp_alpha1':None,'nlp_alpha2':None,'nlp_step':None,
                            'drc_compTh':None,'drc_compRatio':None,'drc_compGain':None,'drc_boostTh':None,'drc_boostRatio':None,'drc_boostGain':None,
                            'drc_AT':None,'drc_RT':None,'mic_mute':None,'cng_noise':None,'bmInput_fixedGain':None,'nlp_fixedGain':None,'drc_fixedGain':None}
    commands[1]['type'] = 'aec'
    commands[1]['param'] = {'param_set_flag':None}

    with open(command_file_name, 'w') as f:
        # 辞書型をjsonとして出力
        json.dump(commands, f, indent=True)

    Wait_Param_Set_Flag()

    # --- control.out を実行 ---
    if send_command(command_file_name,result_file_name) == -1:
        g_aec_param_flag = False
        ws_send = "dialog,DSP error"
        return ws_send

    with open(result_file_name, "r") as file:
        ws_send_rdata = json.load(file)

    if ws_send_rdata[0]['param']['on'] == True:
        AEC_onoff = "true"
    else:
        AEC_onoff = "false"

    if ws_send_rdata[0]['param']['mic_mute'] == True:
        mic_mute_tf = "true"
    else:
        mic_mute_tf = "false"

    ## 取得したmu_パラメータについて、"/"区切りに変換
    _mu_prms = ["{:.5f}".format(round(i,5)) for i in ws_send_rdata[0]['param']['mu_']]
    mu_prms = "/".join(_mu_prms)

    ws_send_data = []

    ws_send_data.append("insp_get_aec_params") #[0]

    ws_send_data.append(AEC_onoff) #AEC_on/off [1]
    ws_send_data.append(mu_prms) # [2]
    ws_send_data.append("{:.3f}".format(round(float(ws_send_rdata[0]['param']['errmu_limit']),3))) #[3]
    ws_send_data.append(int(ws_send_rdata[0]['param']['rcv_dly_'])) #[4]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[0]['param']['nlp_ratio0']),2))) #[5]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[0]['param']['nlp_ratio1']),2))) #[6]
    ws_send_data.append("{:.1f}".format(round(float(ws_send_rdata[0]['param']['nlp_enhance0']),1))) #[7]
    ws_send_data.append("{:.1f}".format(round(float(ws_send_rdata[0]['param']['nlp_enhance1']),1))) #[8]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[0]['param']['nlp_alpha0']),2))) #[9]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[0]['param']['nlp_alpha1']),2))) #[10]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[0]['param']['nlp_alpha2']),2))) #[11]
    ws_send_data.append(int(ws_send_rdata[0]['param']['nlp_step'])) #[12]
    ws_send_data.append(int(ws_send_rdata[0]['param']['drc_compTh'])) #[13]
    ws_send_data.append(int(ws_send_rdata[0]['param']['drc_compRatio'])) #[14]
    ws_send_data.append(int(ws_send_rdata[0]['param']['drc_compGain'])) #[15]
    ws_send_data.append(int(ws_send_rdata[0]['param']['drc_boostTh'])) #[16]
    ws_send_data.append(int(ws_send_rdata[0]['param']['drc_boostRatio'])) #[17]
    ws_send_data.append(int(ws_send_rdata[0]['param']['drc_boostGain'])) #[18]
    ws_send_data.append(int(ws_send_rdata[0]['param']['drc_AT'])) #[19]
    ws_send_data.append(int(ws_send_rdata[0]['param']['drc_RT'])) #[20]
    ws_send_data.append(mic_mute_tf) #[21]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[0]['param']['cng_noise']),2))) #[22]
    ws_send_data.append("{:.1f}".format(round(float(ws_send_rdata[0]['param']['bmInput_fixedGain']),1))) #[23]
    ws_send_data.append("{:.1f}".format(round(float(ws_send_rdata[0]['param']['nlp_fixedGain']),1))) #[24]
    ws_send_data.append("{:.1f}".format(round(float(ws_send_rdata[0]['param']['drc_fixedGain']),1))) #[25]
    ws_send_data.append(int(ws_send_rdata[1]['param']['param_set_flag'])) #[26]

    ws_send_data_ = [str(j) for j in ws_send_data]

    ws_send = ",".join(ws_send_data_)

    g_aec_param_flag=False
    return ws_send

### inspection ###
def Set_Aec_params(data,thisNAME_PATH):
    global s_aec_param_flag
    if s_aec_param_flag==True:
        #print('G_LEVEL ->',g_mic_level_flag) #kura DEBUG
        return
    s_aec_param_flag=True

    PATH = os.path.join(thisNAME_PATH, '../../tmp/')
    netlist_file_name = thisNAME_PATH + 'netlist.bin'
    control_out = thisNAME_PATH + 'control.out'

    command_file_name = PATH + '/command/set_aec_params.json'
    result_file_name = PATH + '/result/set_aec_params.json'

    FAIL_TIME = 1000
    blocknums = [BlockNo.AEC,BlockNo.AEC,BlockNo.AEC,BlockNo.AEC] #set,get,set(param_set_flag),get(param_set_flag)

    if data[2] == 'true':
        AEC_onoff = True
    else:
        AEC_onoff = False

    if data[22] == 'true':
        AEC_micmute = True
    else:
        AEC_micmute = False

    ## mu_パラメータを抽出し、'[]'削除し、/'で区切り、floatに変換
    _mu_prms_set = data[3].lstrip('[').rstrip(']').split('/') #24番目にmu_が入っている。（json形式にしたい）
    mu_prms_set = [float(i) for i in _mu_prms_set]

    # --- command.jsonを作成 ---
    command = {
        'access': 'get',
        'block_no': None,
        'type': None,
        'param': {}
    }

    commands = []

    #param_set_flagのみ最後に通信することが必須なため、param_set_flag通信専用のset用、get用のコマンド配列作成。param配列内部ではアルファベット順しか順序を保証できない。
    for i in range(0,len(blocknums)):
        commands.append(copy.deepcopy(command))
        commands[i]['block_no'] = blocknums[i]
        commands[i]['type'] = 'aec' #set用途
        
    commands[0]['access'] = 'set' #上記でcommand.jsonを'get'で定義しているので'set'に上書きしている

    commands[0]['param'] = {'on':None,'mu_':None,'errmu_limit':None,'rcv_dly_':None,'nlp_ratio0':None,'nlp_ratio1':None,
                            'nlp_enhance0':None,'nlp_enhance1':None,'nlp_alpha0':None,'nlp_alpha1':None,'nlp_alpha2':None,'nlp_step':None,
                            'drc_compTh':None,'drc_compRatio':None,'drc_compGain':None,'drc_boostTh':None,'drc_boostRatio':None,'drc_boostGain':None,
                            'drc_AT':None,'drc_RT':None,'mic_mute':None,'cng_noise':None,'bmInput_fixedGain':None,'nlp_fixedGain':None,'drc_fixedGain':None}

    commands[0]['param']['on'] = AEC_onoff
    commands[0]['param']['mu_'] = mu_prms_set
    commands[0]['param']['errmu_limit'] = float(data[4])
    commands[0]['param']['rcv_dly_'] = int(data[5])
    commands[0]['param']['nlp_ratio0'] = float(data[6])
    commands[0]['param']['nlp_ratio1'] = float(data[7])
    commands[0]['param']['nlp_enhance0'] = float(data[8])
    commands[0]['param']['nlp_enhance1'] = float(data[9])
    commands[0]['param']['nlp_alpha0'] = float(data[10])
    commands[0]['param']['nlp_alpha1'] = float(data[11])
    commands[0]['param']['nlp_alpha2'] = float(data[12])
    commands[0]['param']['nlp_step'] = int(data[13])
    commands[0]['param']['drc_compTh'] = int(data[14])
    commands[0]['param']['drc_compRatio'] = int(data[15])
    commands[0]['param']['drc_compGain'] = int(data[16])
    commands[0]['param']['drc_boostTh'] = int(data[17])
    commands[0]['param']['drc_boostRatio'] = int(data[18])
    commands[0]['param']['drc_boostGain'] = int(data[19])
    commands[0]['param']['drc_AT'] = int(data[20])
    commands[0]['param']['drc_RT'] = int(data[21])
    commands[0]['param']['mic_mute'] = AEC_micmute
    commands[0]['param']['cng_noise'] = float(data[23])
    commands[0]['param']['bmInput_fixedGain'] = float(data[24])
    commands[0]['param']['nlp_fixedGain'] = float(data[25])
    commands[0]['param']['drc_fixedGain'] = float(data[26])

    
    # mu_に注意。
    commands[1]['param'] = {'on':None,'mu_':None,'errmu_limit':None,'rcv_dly_':None,'nlp_ratio0':None,'nlp_ratio1':None,
                            'nlp_enhance0':None,'nlp_enhance1':None,'nlp_alpha0':None,'nlp_alpha1':None,'nlp_alpha2':None,'nlp_step':None,
                            'drc_compTh':None,'drc_compRatio':None,'drc_compGain':None,'drc_boostTh':None,'drc_boostRatio':None,'drc_boostGain':None,
                            'drc_AT':None,'drc_RT':None,'mic_mute':None,'cng_noise':None,'bmInput_fixedGain':None,'nlp_fixedGain':None,'drc_fixedGain':None}
    
    commands[2]['access'] = 'set' #上記でcommand.jsonを'get'で定義しているので'set'に上書きしている
    commands[2]['param'] = {'param_set_flag':None}
    commands[2]['param']['param_set_flag'] = int(data[27])
    commands[3]['param'] = {'param_set_flag':None}

    with open(command_file_name, 'w') as f:
        # 辞書型をjsonとして出力
        json.dump(commands, f, indent=True)

    Wait_Param_Set_Flag()
    
    # --- control.out を実行 ---
    if send_command(command_file_name,result_file_name) == -1:
        s_aec_param_flag = False
        ws_send = "dialog,DSP error"
        return ws_send

    with open(result_file_name, "r") as file:
        ws_send_rdata = json.load(file)

    if ws_send_rdata[1]['param']['on'] == True:
        AEC_onoff = "true"
    else:
        AEC_onoff = "false"

    if ws_send_rdata[1]['param']['mic_mute'] == True:
        mic_mute_tf = "true"
    else:
        mic_mute_tf = "false"

    ## 取得したmu_パラメータについて、"/"区切りに変換
    _mu_prms = ["{:.5f}".format(round(i,5)) for i in ws_send_rdata[1]['param']['mu_']]
    mu_prms = "/".join(_mu_prms)

    ws_send_data = []

    ws_send_data.append("insp_get_aec_params") #[0]

    ws_send_data.append(AEC_onoff) #AEC_on/off [1]
    ws_send_data.append(mu_prms) # [2]
    ws_send_data.append("{:.3f}".format(round(float(ws_send_rdata[1]['param']['errmu_limit']),3))) #[3]
    ws_send_data.append(int(ws_send_rdata[1]['param']['rcv_dly_'])) #[4]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['nlp_ratio0']),2))) #[5]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['nlp_ratio1']),2))) #[6]
    ws_send_data.append("{:.1f}".format(round(float(ws_send_rdata[1]['param']['nlp_enhance0']),1))) #[7]
    ws_send_data.append("{:.1f}".format(round(float(ws_send_rdata[1]['param']['nlp_enhance1']),1))) #[8]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['nlp_alpha0']),2))) #[9]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['nlp_alpha1']),2))) #[10]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['nlp_alpha2']),2))) #[11]
    ws_send_data.append(int(ws_send_rdata[1]['param']['nlp_step'])) #[12]
    ws_send_data.append(int(ws_send_rdata[1]['param']['drc_compTh'])) #[13]
    ws_send_data.append(int(ws_send_rdata[1]['param']['drc_compRatio'])) #[14]
    ws_send_data.append(int(ws_send_rdata[1]['param']['drc_compGain'])) #[15]
    ws_send_data.append(int(ws_send_rdata[1]['param']['drc_boostTh'])) #[16]
    ws_send_data.append(int(ws_send_rdata[1]['param']['drc_boostRatio'])) #[17]
    ws_send_data.append(int(ws_send_rdata[1]['param']['drc_boostGain'])) #[18]
    ws_send_data.append(int(ws_send_rdata[1]['param']['drc_AT'])) #[19]
    ws_send_data.append(int(ws_send_rdata[1]['param']['drc_RT'])) #[20]
    ws_send_data.append(mic_mute_tf) #[21]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['cng_noise']),2))) #[22]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['bmInput_fixedGain']),1))) #[23]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['nlp_fixedGain']),1))) #[24]
    ws_send_data.append("{:.2f}".format(round(float(ws_send_rdata[1]['param']['drc_fixedGain']),1))) #[25]
    ws_send_data.append(int(ws_send_rdata[3]['param']['param_set_flag'])) #[26]

    ws_send_data_ = [str(j) for j in ws_send_data]

    ws_send = ",".join(ws_send_data_)

    s_aec_param_flag=False
    return ws_send

def Get_Version(data,thisNAME_PATH):
    global g_version_flag
    if g_version_flag==True:
        #print('G_LEVEL ->',g_version_flag) #kura DEBUG
        return
    g_version_flag=True

    CPU_VER_LBL='cpuver'
    DSP_VER_LBL='dspver'
    MAC_ADDR_LBL='macaddr'

    ws_send_data = ['insp_version', CPU_VER_LBL, DSP_VER_LBL, MAC_ADDR_LBL]

    #get cpu version
    version_fname = thisNAME_PATH + 'config/amcf1_version.json'
    with open(version_fname,'r') as file:
        version_rdata = json.load(file)
    ws_send_data[ws_send_data.index(CPU_VER_LBL)] = version_rdata['version_for_developer']  #version

    #get dsp version
    DSP_version_fname = thisNAME_PATH + '../../tmp/dsp_version.json' #/home/toa/TOA_AM-CF1/tmp directory
    
    with open(DSP_version_fname,'r') as file:
        dsp_version_rdata = json.load(file)
    ws_send_data[ws_send_data.index(DSP_VER_LBL)] = dsp_version_rdata['dsp_version']  #version
    

    # MACアドレス取得
    cmd = ["ifconfig | grep enx | awk '{print $5}'"]
    mac_addr = sp.getoutput(cmd)

    ws_send_data[ws_send_data.index(MAC_ADDR_LBL)] = mac_addr

    ws_send = ",".join(ws_send_data)

    g_version_flag=False
    return ws_send
