#!/bin/sh

##set -x

RESULT_NG=0

CHKFILE_DAT=/home/toa/StartupShell/chkfile_dat.tar.xz
CHKFILE_DAT_org=/home_org/toa/StartupShell/chkfile_dat.tar.xz
TEMP_UPDATE_DIR=/tmp/Chkfile

DBGPRINT() {
    echo ${*}
}

Chkfile() {
    DIR_FOR_CHECK=${1}
    DAT_FILE=${2}
    RESULT_FILE=/tmp/chkfile_result.${$}
    rm -f ${RESULT_FILE}

    NUM_OF_FILE=$(cat ${DAT_FILE} | wc -l)
    (cd ${DIR_FOR_CHECK} && md5sum --check ${DAT_FILE}) > ${RESULT_FILE} 2>/dev/null
    NUM_OF_OK=$(egrep ': OK$' ${RESULT_FILE} | wc -l)

    if [ ${NUM_OF_FILE} -eq ${NUM_OF_OK} ]; then
        echo "Success: check file contents of ${DIR_FOR_CHECK}"
    else
        RESULT_NG=1
        echo "Failed: check file contents of ${DIR_FOR_CHECK}"
        echo "        NG line(s) below"
        echo '******************************************************'
        egrep -v ': OK$' ${RESULT_FILE}
        echo '******************************************************'
    fi

    rm -f ${RESULT_FILE}
}

Chkownerpermission() {
    DIR_FOR_CHECK=${1}
    NAME_FILE=${2}
    EXPECTED_FILE=${3}
    RESULT_FILE=/tmp/chkfile.result.txt.${$}
    rm -f ${RESULT_FILE}
    DIFF_OUTPUT=/tmp/chkfile.diff.txt.${$}
    rm -f ${DIFF_OUTPUT}

    echo "Now checking file owner/permission of ${DIR_FOR_CHECK} ..."

    (
        cd ${DIR_FOR_CHECK}
        cat ${NAME_FILE} | while read line
        do
            sudo stat -c '%a %U %G %n' "${line}"
        done
    ) > ${RESULT_FILE}

    diff -dU0 ${EXPECTED_FILE} ${RESULT_FILE} > ${DIFF_OUTPUT}

    if [ ${?} -eq 0 ]; then
        echo "Success: check file owner/permission of ${DIR_FOR_CHECK}"
    else
        RESULT_NG=1
        echo "Failed: check file owner/permission of ${DIR_FOR_CHECK}"
        echo "        NG line(s) below"
        echo '******************************************************'
        cat ${DIFF_OUTPUT}
        echo '******************************************************'
    fi

    rm -f ${RESULT_FILE}
    rm -f ${DIFF_OUTPUT}
}

Chkpermission() {
    DIR_FOR_CHECK=${1}
    NAME_FILE=${2}
    EXPECTED_FILE=${3}
    RESULT_FILE=/tmp/chkfile.result.txt.${$}
    rm -f ${RESULT_FILE}
    DIFF_OUTPUT=/tmp/chkfile.diff.txt.${$}
    rm -f ${DIFF_OUTPUT}

    echo "Now checking file permission of ${DIR_FOR_CHECK} ..."

    (
        cd ${DIR_FOR_CHECK}
        cat ${NAME_FILE} | while read line
        do
            sudo stat -c '%a %n' "${line}"
        done
    ) > ${RESULT_FILE}

    diff -dU0 ${EXPECTED_FILE} ${RESULT_FILE} > ${DIFF_OUTPUT}

    if [ ${?} -eq 0 ]; then
        echo "Success: check file permission of ${DIR_FOR_CHECK}"
    else
        RESULT_NG=1
        echo "Failed: check file permission of ${DIR_FOR_CHECK}"
        echo "        NG line(s) below"
        echo '******************************************************'
        cat ${DIFF_OUTPUT}
        echo '******************************************************'
    fi

    rm -f ${RESULT_FILE}
    rm -f ${DIFF_OUTPUT}
}

if [ ! -f "${CHKFILE_DAT_org}" ]; then
    echo '++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
    echo 'Error: dat file for chkfile is not copied !!!'
    echo '++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
    exit 1
fi

## The temporary directory for Update or Chekfile
rm -fr ${TEMP_UPDATE_DIR}
mkdir ${TEMP_UPDATE_DIR}

## Extract archive to the temporary directory
tar -C ${TEMP_UPDATE_DIR} -Jxf "${CHKFILE_DAT_org}"

Chkfile /boot     ${TEMP_UPDATE_DIR}/md5sum.boot.txt

Chkfile /etc      ${TEMP_UPDATE_DIR}/md5sum.etc.txt

Chkfile /home_org ${TEMP_UPDATE_DIR}/md5sum.home.txt

Chkfile /var_org  ${TEMP_UPDATE_DIR}/md5sum.var.txt

Chkownerpermission /boot     ${TEMP_UPDATE_DIR}/ownerpermission.name.boot.txt ${TEMP_UPDATE_DIR}/ownerpermission.expected.boot.txt

Chkownerpermission /etc      ${TEMP_UPDATE_DIR}/ownerpermission.name.etc.txt  ${TEMP_UPDATE_DIR}/ownerpermission.expected.etc.txt

Chkpermission /home_org ${TEMP_UPDATE_DIR}/permission.name.home.txt ${TEMP_UPDATE_DIR}/permission.expected.home.txt

Chkpermission /var_org  ${TEMP_UPDATE_DIR}/permission.name.var.txt  ${TEMP_UPDATE_DIR}/permission.expected.var.txt

Chkownerpermission / ${TEMP_UPDATE_DIR}/ownerpermission.name.root.extra.txt ${TEMP_UPDATE_DIR}/ownerpermission.expected.root.extra.txt

Chkownerpermission /home_org ${TEMP_UPDATE_DIR}/ownerpermission.name.home.extra.txt ${TEMP_UPDATE_DIR}/ownerpermission.expected.home.extra.txt

rm -fr ${TEMP_UPDATE_DIR}

##rm -f ${CHKFILE_DAT}
##rm -f ${CHKFILE_DAT_org}

if [ ${RESULT_NG} -eq 0 ]; then
    echo "TOTAL RESULT: Success."
else
    echo "TOTAL RESULT: Failed."
    exit 1
fi

exit 0
