#include <iostream>
#include <stdint.h>
#include <picojson_misc.h>
#include "command_aec.h"
#include <dsp_lib/substitute/RaspberryPi_app.h>
#include <dsp_lib/include/dsp/aec.h>
#include <dsp_lib/include/misc/smoothing.h>
#include <spi/spi_api.h>
#include <control/misc.h>

using namespace std;
using namespace dsp_lib;

namespace control {

bool
CommandAec(
	picojson::object				&command_obj_,
	std::string						access_,
	dsp_lib::DspManager::NetList	netlist_
)
{
	//uint32_t config_base	= reinterpret_cast<uint32_t>(netlist_.config);
	uint32_t param_base		= reinterpret_cast<uint32_t>(netlist_.param);
	uint32_t status_base	= reinterpret_cast<uint32_t>(netlist_.status);

	//Aec::Config*	config_offset	= 0x00000000;
	Aec::Param*	param_offset = 0x00000000;
	Aec::Status*	status_offset	= 0x00000000;

	if (access_ == "set") {
		// paramがないとエラー
		picojson::object param_obj;
		if (get_object_member(command_obj_, "param", param_obj) == false) {
			cout << "\"param\" not found!" << endl;
			return false;
		}

		if (param_obj.count("on") == 1) {
			if (param_obj["on"].is<bool>() == false) {
				cout << "param.on must be bool!" << endl;
				return false;
			}
			bool on = param_obj["on"].get<bool>();
			uint32_t on_uint32;
			if (on == true) {
				on_uint32 = 1;
			} else {
				on_uint32 = 0;
			}

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->speex_echo_aec), on_uint32) == false) {
				return false;
			}
		}

		if (param_obj.count("echo_suppress") == 1) {
			if (param_obj["echo_suppress"].is<double>() == false) {
				cout << "param.attack_time must be number!" << endl;
				return false;
			}
			int32_t echo_suppress = static_cast<int32_t>(param_obj["echo_suppress"].get<double>());

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->speex_preprocess_echo_suppress), echo_suppress) == false) {
				return false;
			}
		}


	} else if (access_ == "get") {
		// paramがなくてもOK
		if (command_obj_.count("param") == 1) {
			if (command_obj_["param"].is<picojson::object>() == false) {
				cout << "\"param\" must be object!" << endl;
			}
			picojson::object param_obj = command_obj_["param"].get<picojson::object>();

			if (param_obj.count("on") == 1) {
				bool	 on;
				uint32_t on_uint32;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->speex_echo_aec), &on_uint32) == false) {
					return false;
				}
				if (on_uint32 == 0) {
					on = false;
				} else {
					on = true;
				}
				set_object_member(param_obj, "on", on);
			}

			if (param_obj.count("echo_suppress") == 1) {
				int32_t echo_suppress;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->speex_preprocess_echo_suppress), &echo_suppress) == false) {
					return false;
				}

				set_object_member(param_obj, "echo_suppress", echo_suppress);
			}

			if (param_obj.count("microphonearray") == 1) {
				uint32_t microphonearray;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->microphonearray), &microphonearray) == false) {
					return false;
				}
				set_object_member(param_obj, "microphonearray", microphonearray);
			}

			set_object_member(command_obj_, "param", param_obj);
		}

		// statusがなくてもOK
		if (command_obj_.count("status") == 1) {
			if (command_obj_["status"].is<picojson::object>() == false) {
				cout << "\"status\" must be object!" << endl;
			}
			picojson::object status_obj = command_obj_["status"].get<picojson::object>();

			if (status_obj.count("focus_x") == 1) {
				float focus_x;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->focus_x), &focus_x) == false) {
					return false;
				}
				set_object_member(status_obj, "focus_x", focus_x);
			}

			if (status_obj.count("focus_y") == 1) {
				float focus_y;
				if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->focus_y), &focus_y) == false) {
					return false;
				}
				set_object_member(status_obj, "focus_y", focus_y);
			}

			set_object_member(command_obj_, "status", status_obj);
		}
	}
	else {
		cout << "invalid accessor!\n" << endl;
		return false;
	}
	command_obj_["result"] = static_cast<picojson::value>(true);
	return true;
}

} // namespace command
