#include <iostream>
#include <stdint.h>
#include <picojson_misc.h>
#include "command_microphonearray.h"
#include <dsp_lib/substitute/RaspberryPi_app.h>
#include <dsp_lib/include/dsp/microphonearray.h>
#include <dsp_lib/include/misc/smoothing.h>
#include <spi/spi_api.h>
#include <control/misc.h>

using namespace std;
using namespace dsp_lib;

namespace control {

bool
CommandMicrophonearray(
	picojson::object				&command_obj_,
	std::string						access_,
	dsp_lib::DspManager::NetList	netlist_
)
{
	//uint32_t config_base	= reinterpret_cast<uint32_t>(netlist_.config);
	uint32_t param_base		= reinterpret_cast<uint32_t>(netlist_.param);
	uint32_t status_base	= reinterpret_cast<uint32_t>(netlist_.status);

	//Microphonearray::Config*	config_offset	= 0x00000000;
	Microphonearray::Param*	param_offset = 0x00000000;
	Microphonearray::Status*	status_offset	= 0x00000000;

	if (access_ == "set") {
		// paramがないとエラー
		picojson::object param_obj;
		if (get_object_member(command_obj_, "param", param_obj) == false) {
			cout << "\"param\" not found!" << endl;
			return false;
		}

		if (param_obj.count("update_mode") == 1) {
			if (param_obj["update_mode"].is<string>() == false) {
				cout << "param.update_mode must be string!" << endl;
				return false;
			}
			string update_mode_str = param_obj["update_mode"].get<string>();
			uint32_t update_mode;
			if (update_mode_str == "manual") {
				update_mode = Microphonearray::MICROPHONEARRAY_BF_UPDATE_MODE_MANUAL;
			} else if (update_mode_str == "semiauto") {
				update_mode = Microphonearray::MICROPHONEARRAY_BF_UPDATE_MODE_SEMIAUTO;
			} else if (update_mode_str == "auto") {
				update_mode = Microphonearray::MICROPHONEARRAY_BF_UPDATE_MODE_AUTO;
			} else {
				cout << "\"update_mode\" must be manual, semiauto or auto!" << endl;
				return false;
			}

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->update_mode), update_mode) == false) {
				return false;
			}
		}

		if (param_obj.count("dir") == 1) {
			if (param_obj["dir"].is<double>() == false) {
				cout << "param.dir must be number!" << endl;
				return false;
			}
			float dir = static_cast<float>(param_obj["dir"].get<double>());

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->dir), dir) == false) {
				return false;
			}
		}

		if (param_obj.count("dis") == 1) {
			if (param_obj["dis"].is<double>() == false) {
				cout << "param.dis must be number!" << endl;
				return false;
			}
			float dis = static_cast<float>(param_obj["dis"].get<double>());

			if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->dis), dis) == false) {
				return false;
			}
		}

		if (param_obj.count("doa") == 1) {
			if (param_obj["doa"].is<picojson::object>() == false) {
				cout << "param.doa must be object!" << endl;
				return false;
			}
			picojson::object doa_obj = param_obj["doa"].get<picojson::object>();

			if (doa_obj.count("peak_threshold") == 1) {
				if (doa_obj["peak_threshold"].is<double>() == false) {
					cout << "param.doa.peak_threshold must be number!" << endl;
					return false;
				}
				float peak_threshold = static_cast<float>(doa_obj["peak_threshold"].get<double>());
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->doa.peak_threshold), peak_threshold) == false) {
					return false;
				}
			}
			if (doa_obj.count("smooth_tc") == 1) {
				if (doa_obj["smooth_tc"].is<double>() == false) {
					cout << "param.doa.smooth_tc must be number!" << endl;
					return false;
				}
				float smooth_tc = static_cast<float>(doa_obj["smooth_tc"].get<double>());
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->doa.smooth_tc), smooth_tc) == false) {
					return false;
				}
			}
			if (doa_obj.count("source_pos_x_min") == 1) {
				if (doa_obj["source_pos_x_min"].is<double>() == false) {
					cout << "param.doa.source_pos_x_min must be number!" << endl;
					return false;
				}
				float source_pos_x_min = static_cast<float>(doa_obj["source_pos_x_min"].get<double>());
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->doa.source_pos_x_min), source_pos_x_min) == false) {
					return false;
				}
			}
			if (doa_obj.count("source_pos_x_max") == 1) {
				if (doa_obj["source_pos_x_max"].is<double>() == false) {
					cout << "param.doa.source_pos_x_max must be number!" << endl;
					return false;
				}
				float source_pos_x_max = static_cast<float>(doa_obj["source_pos_x_max"].get<double>());
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->doa.source_pos_x_max), source_pos_x_max) == false) {
					return false;
				}
			}
			if (doa_obj.count("source_pos_y_min") == 1) {
				if (doa_obj["source_pos_y_min"].is<double>() == false) {
					cout << "param.doa.source_pos_y_min must be number!" << endl;
					return false;
				}
				float source_pos_y_min = static_cast<float>(doa_obj["source_pos_y_min"].get<double>());
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->doa.source_pos_y_min), source_pos_y_min) == false) {
					return false;
				}
			}
			if (doa_obj.count("source_pos_y_max") == 1) {
				if (doa_obj["source_pos_y_max"].is<double>() == false) {
					cout << "param.doa.source_pos_y_max must be number!" << endl;
					return false;
				}
				float source_pos_y_max = static_cast<float>(doa_obj["source_pos_y_max"].get<double>());
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->doa.source_pos_y_max), source_pos_y_max) == false) {
					return false;
				}
			}
		}

		if (param_obj.count("bf") == 1) {
			if (param_obj["bf"].is<picojson::object>() == false) {
				cout << "param.bf must be object!" << endl;
				return false;
			}
			picojson::object bf_obj = param_obj["bf"].get<picojson::object>();

			if (bf_obj.count("focus_x") == 1) {
				if (bf_obj["focus_x"].is<double>() == false) {
					cout << "param.bf.focus_x must be number!" << endl;
					return false;
				}
				float focus_x = static_cast<float>(bf_obj["focus_x"].get<double>());
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->bf.focus_x), focus_x) == false) {
					return false;
				}
			}
			if (bf_obj.count("focus_y") == 1) {
				if (bf_obj["focus_y"].is<double>() == false) {
					cout << "param.bf.focus_y must be number!" << endl;
					return false;
				}
				float focus_y = static_cast<float>(bf_obj["focus_y"].get<double>());
				if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->bf.focus_y), focus_y) == false) {
					return false;
				}
			}
		}
	} else if (access_ == "get") {
		// paramがなくてもOK
		if (command_obj_.count("param") == 1) {
			if (command_obj_["param"].is<picojson::object>() == false) {
				cout << "\"param\" must be object!" << endl;
			}
			picojson::object param_obj = command_obj_["param"].get<picojson::object>();

			if (param_obj.count("update_mode") == 1) {
				uint32_t update_mode;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->update_mode), &update_mode) == false) {
					return false;
				}
				string update_mode_str;
				switch (update_mode) {
					case Microphonearray::MICROPHONEARRAY_BF_UPDATE_MODE_MANUAL:
					update_mode_str = "manual";
					break;
					case Microphonearray::MICROPHONEARRAY_BF_UPDATE_MODE_SEMIAUTO:
					update_mode_str = "semiauto";
					break;
					case Microphonearray::MICROPHONEARRAY_BF_UPDATE_MODE_AUTO:
					update_mode_str = "auto";
					break;
					default:
					cout << "current update_mode is broken!(" << update_mode << ")" << endl;
					return false;
				}
				set_object_member(param_obj, "update_mode", update_mode_str);
			}

			if (param_obj.count("dir") == 1) {
				float dir;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->dir), &dir) == false) {
					return false;
				}
				set_object_member(param_obj, "dir", dir);
			}

			if (param_obj.count("dis") == 1) {
				float dis;
				if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->dis), &dis) == false) {
					return false;
				}
				set_object_member(param_obj, "dis", dis);
			}

			if (param_obj.count("doa") == 1) {
				if (param_obj["doa"].is<picojson::object>() == false) {
					cout << "param.doa must be object!" << endl;
					return false;
				}
				picojson::object doa_obj = param_obj["doa"].get<picojson::object>();

				if (doa_obj.count("peak_threshold") == 1) {
					float peak_threshold;
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->doa.peak_threshold), &peak_threshold) == false) {
						return false;
					}
					set_object_member(doa_obj, "peak_threshold", peak_threshold);
				}
				if (doa_obj.count("smooth_tc") == 1) {
					float smooth_tc;
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->doa.smooth_tc), &smooth_tc) == false) {
						return false;
					}
					set_object_member(doa_obj, "smooth_tc", smooth_tc);
				}
				if (doa_obj.count("source_pos_x_min") == 1) {
					float source_pos_x_min;
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->doa.source_pos_x_min), &source_pos_x_min) == false) {
						return false;
					}
					set_object_member(doa_obj, "source_pos_x_min", source_pos_x_min);
				}
				if (doa_obj.count("source_pos_x_max") == 1) {
					float source_pos_x_max;
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->doa.source_pos_x_max), &source_pos_x_max) == false) {
						return false;
					}
					set_object_member(doa_obj, "source_pos_x_max", source_pos_x_max);
				}
				if (doa_obj.count("source_pos_y_min") == 1) {
					float source_pos_y_min;
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->doa.source_pos_y_min), &source_pos_y_min) == false) {
						return false;
					}
					set_object_member(doa_obj, "source_pos_y_min", source_pos_y_min);
				}
				if (doa_obj.count("source_pos_y_max") == 1) {
					float source_pos_y_max;
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->doa.source_pos_y_max), &source_pos_y_max) == false) {
						return false;
					}
					set_object_member(doa_obj, "source_pos_y_max", source_pos_y_max);
				}
				set_object_member(param_obj, "doa", doa_obj);
			}

			if (param_obj.count("bf") == 1) {
				if (param_obj["bf"].is<picojson::object>() == false) {
					cout << "param.bf must be object!" << endl;
					return false;
				}
				picojson::object bf_obj = param_obj["bf"].get<picojson::object>();

				if (bf_obj.count("focus_x") == 1) {
					float focus_x;
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->bf.focus_x), &focus_x) == false) {
						return false;
					}
					set_object_member(bf_obj, "focus_x", focus_x);
				}
				if (bf_obj.count("focus_y") == 1) {
					float focus_y;
					if (ReadSharedRam(param_base + ToDspAddressing(&param_offset->bf.focus_y), &focus_y) == false) {
						return false;
					}
					set_object_member(bf_obj, "focus_y", focus_y);
				}
				set_object_member(param_obj, "bf", bf_obj);
			}
			set_object_member(command_obj_, "param", param_obj);
		}

		if (command_obj_.count("status") == 1) {
			if (command_obj_["status"].is<picojson::object>() == false) {
				cout << "\"status\" must be object!" << endl;
			}
			picojson::object status_obj = command_obj_["status"].get<picojson::object>();

			if (status_obj.count("doa") == 1) {
				if (status_obj["doa"].is<picojson::object>() == false) {
					cout << "status.doa must be object!" << endl;
					return false;
				}
				picojson::object doa_obj = status_obj["doa"].get<picojson::object>();

				if (doa_obj.count("peak") == 1) {
					float peak[Doa::DOA_NUMBER_OF_SENSOR_PAIR];
					picojson::array peak_array(Doa::DOA_NUMBER_OF_SENSOR_PAIR);
					for (int i = 0; i < Doa::DOA_NUMBER_OF_SENSOR_PAIR; i++) {
						if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->doa.peak[i]), &peak[i]) == false) {
							return false;
						}
						peak_array[i] = get_number_value(peak[i]);
					}
					set_object_member(doa_obj, "peak", peak_array);
				}
				if (doa_obj.count("lag") == 1) {
					int32_t lag[Doa::DOA_NUMBER_OF_SENSOR_PAIR];
					picojson::array lag_array(Doa::DOA_NUMBER_OF_SENSOR_PAIR);
					for (int i = 0; i < Doa::DOA_NUMBER_OF_SENSOR_PAIR; i++) {
						if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->doa.lag[i]), &lag[i]) == false) {
							return false;
						}
						lag_array[i] = get_number_value(lag[i]);
					}
					set_object_member(doa_obj, "lag", lag_array);
				}
				if (doa_obj.count("source_pos_x") == 1) {
					float source_pos_x;
					if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->doa.source_pos_x), &source_pos_x) == false) {
						return false;
					}
					set_object_member(doa_obj, "source_pos_x", source_pos_x);
				}
				if (doa_obj.count("source_pos_y") == 1) {
					float source_pos_y;
					if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->doa.source_pos_y), &source_pos_y) == false) {
						return false;
					}
					set_object_member(doa_obj, "source_pos_y", source_pos_y);
				}
				set_object_member(status_obj, "doa", doa_obj);
			}

			if (status_obj.count("bf") == 1) {
				if (status_obj["bf"].is<picojson::object>() == false) {
					cout << "status.bf must be object!" << endl;
					return false;
				}
				picojson::object bf_obj = status_obj["bf"].get<picojson::object>();

				if (bf_obj.count("focus_x") == 1) {
					float focus_x;
					if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->bf.focus_x), &focus_x) == false) {
						return false;
					}
					set_object_member(bf_obj, "focus_x", focus_x);
				}
				if (bf_obj.count("focus_y") == 1) {
					float focus_y;
					if (ReadSharedRam(status_base + ToDspAddressing(&status_offset->bf.focus_y), &focus_y) == false) {
						return false;
					}
					set_object_member(bf_obj, "focus_y", focus_y);
				}
				set_object_member(status_obj, "bf", bf_obj);
			}
			set_object_member(command_obj_, "status", status_obj);
		}
	}
	else {
		cout << "invalid accessor!\n" << endl;
		return false;
	}
	command_obj_["result"] = static_cast<picojson::value>(true);
	return true;
}

} // namespace command
