//==========================================================================
// aec.h

//--------------------------------------------------------------------------
#pragma once

//==========================================================================
#include <stdint.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/dsp/iir.h>
#include <dsp_lib/include/dsp/microphonearray.h>
#include <dsp_lib/include/misc/dsp_const.h>

//==========================================================================
namespace dsp_lib {

//--------------------------------------------------------------------------
class Speex;
class Src;

//==========================================================================
class Aec : public DspModule {
	public:
		//----------------------------------------
		typedef struct {
		} Config;
		typedef struct {
			uint32_t					speex_echo_aec;							// <! aec on/off
			int32_t						speex_preprocess_noise_suppress;		// <! maximum attenuation of the noise in dB (negative number)
			int32_t						speex_preprocess_echo_suppress;			// <! maximum attenuation of the residual echo in dB (negative number)
			int32_t						speex_preprocess_echo_suppress_active;	// <! maximum attenuation of the residual echo in dB when near end is active (negative number)
			Microphonearray::Status*	microphonearray;
		} Param;
		typedef struct {
			float						focus_x;
			float						focus_y;
		} Status;

		//----------------------------------------
		Aec(
			Config*	/*config_*/,
			Param*	param_,
			Status* status_
		);
		static bool Configure()
		{
			return true;
		}
		static bool Attach()
		{
			return true;
		}
		bool Ready();
		bool Execute(
			const float*	input_,								// <! rcv i, snd i
			float*			output_								// <! rcv o, snd o
		);
		bool Task();

	protected:
		//----------------------------------------
		static const int	AEC_BLOCK_SIZE				= DSP_BLOCK_SIZE / DSP_SRC_RATE;
		static const int	AEC_NUMBER_OF_LPF			= 4;
		static const int	AEC_NUMBER_OF_HPF			= 2;
		static const int	AEC_XOVER_NUMBER_OF_STAGE	= 16;

		//----------------------------------------
		void Initialize();
		void ExecuteCtrl();
		void ExecuteThrough(
			float	(*i_)[AEC_BLOCK_SIZE],
			float	(*o_)[AEC_BLOCK_SIZE]
		);

		//----------------------------------------
		static float	s_aec_i[2][AEC_BLOCK_SIZE];
		static float	s_aec_o[2][AEC_BLOCK_SIZE];
		static float	s_snd_o[2][DSP_BLOCK_SIZE];

		//----------------------------------------
		bool			m_ready;

		Param*			m_param;
		Status*			m_status;

		float			m_snd_i_buf[DSP_BLOCK_SIZE];

		Iir*			m_iir_lpf[AEC_NUMBER_OF_LPF];
		Iir::Config		m_iir_lpf_config[AEC_NUMBER_OF_LPF];
		Iir::Param		m_iir_lpf_param[AEC_NUMBER_OF_LPF];

		Iir*			m_iir_hpf[AEC_NUMBER_OF_HPF];
		Iir::Config		m_iir_hpf_config[AEC_NUMBER_OF_HPF];
		Iir::Param		m_iir_hpf_param[AEC_NUMBER_OF_HPF];

		Src*			m_src_rcv;
		Src*			m_src_snd;

		Speex*			m_speex;
};

//==========================================================================
}	// namespace dsp_lib
