/*
 * gain.h
 *
 *  Created on: 2014/07/02
 *      Author: toa29548
 */

#pragma once

#include <stdint.h>
#include <dsp_lib/include/dsp/instant_gain.h>
#include <dsp_lib/include/misc/null_check.h>
#include <dsp_lib/include/dsp/dsp_module.h>

namespace dsp_lib {

class Gain : public DspModule {
public:
	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------

	struct Config{};

	struct Param{
		float attack;			//!< ゲイン上昇時の忘却係数, 1で完全忘却
		float release;			//!< ゲイン下降時の忘却係数, 1で完全忘却
		float gain_target;
	};

	struct Status{};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------
	static bool Configure(void) {
		return true;
	};
	static bool Attach(void) {
		return true;
	};
	// -------------------------------------------------------------
	// パラメータ変更
	// -------------------------------------------------------------
	/*!
	 *
	 * @param param_ パラメータ構造体
	 * @param gain_ ゲイン[真数]
	 * @return
	 */
	static bool
	SetGain(
		Param*	param_,
		float	gain_
	)
	{
		if (NullCheck(param_) == false) {
			return false;
		}

		param_->gain_target = gain_;

		return true;
	}

public:
	Gain(
			Config*	/*config_*/,
			Param*	param_,
			Status*	/*status_*/
	);
	bool Ready(void);
	bool Execute(
			const float*	input_,
			float*			output_
	);
	bool Task(void);

protected:
	Param*				m_param;

	InstantGain*		m_instant_gain;
	InstantGain::Param	m_instant_gain_param;

	float 				m_attack;			//!< ゲイン上昇時の忘却係数, 1で完全忘却
	float 				m_release;			//!< ゲイン下降時の忘却係数, 1で完全忘却
};

} //namespace dsp_lib
