//======================================================================
//
//	fdesign.h
//
//
//======================================================================
#pragma once

//======================================================================
#include <stdint.h>

//======================================================================
namespace dsp_lib {

enum FILTER_TYPE {
	FILTER_TYPE_THROUGH			= 0,
	FILTER_TYPE_PEQ				= 1,
	FILTER_TYPE_HPF6			= 2,
	FILTER_TYPE_LPF6			= 3,
	FILTER_TYPE_HPF12			= 4,
	FILTER_TYPE_LPF12			= 5,
	FILTER_TYPE_LOW_SHELVING	= 6,
	FILTER_TYPE_HIGH_SHELVING	= 7,
	FILTER_TYPE_ALL_PASS		= 8,
	FILTER_TYPE_NOTCH			= 9,
	FILTER_TYPE_HORN_EQ			= 10
};
enum COEFF {
	COEFF_SIZE	= 5
};

/*!
 * Audio-EQ-Cookbookに基づき下記の式を採用
 *
 *        b0 + b1*z^-1 + b2*z^-2
 * H(z) = -----------------------
 *        1 + a1*z^-1 + a2*z^-2
 *
 * coeff_[0]: b0
 * coeff_[1]: b1
 * coeff_[2]: b2
 * coeff_[3]: a1
 * coeff_[4]: a2
 *
 * @param filtertype_ フィルタの種類を FILTER_TYPE から選択
 * @param f
 * @param gain
 * @param q
 * @param alphaindex_
 * @param eqon_ 1:on, 0:off
 * @param coeff_ フィルタ係数, 順序は上記の通り
 */
extern void
Fdesign(
	int32_t	filtertype_,
	float	f,
	float	gain,
	float	q,
	int32_t	alphaindex_,
	int32_t	eqon_,
	float	*coeff_
);
	
	
/*
	static void	Get(
		int32_t	filtertype_,
		int32_t	freq_,
		int32_t	gain_,
		int32_t	qindex_,
		int32_t	alphaindex_,
		int32_t	eqon_,
		uint32_t	*a0_,
		uint32_t	*a1_,
		uint32_t	*a2_,
		uint32_t	*b1_,
		uint32_t	*b2_
	);
	static void	GetThrough(
		uint32_t	*a0_,
		uint32_t	*a1_,
		uint32_t	*a2_,
		uint32_t	*b1_,
		uint32_t	*b2_
	);

	static bool	CheckUpdateMute(
		int32_t	filtertype_,

		int32_t	freq_from_,
		int32_t	gain_from_,
		int32_t	qindex_from__,
		int32_t	alphaindex_from_,

		int32_t	freq_to_,
		int32_t	gain_to_,
		int32_t	qindex_to__,
		int32_t	alphaindex_to_
	);

	static void	GetEQQ(const int32_t  qindex_, float  &qvalue_);
	static void	GetFilterQ(const int32_t  qindex_, float  &qvalue_);

	static void	GetLPFAlpha(const int32_t  alphaindex_, float  &alpha_);
	static void	GetHPFAlpha(const int32_t  alphaindex_, float  &alpha_);

	static void	GetFrequency(float  &fvalue_);
	static void	GetFrequency(const int32_t  freq_, float  &fvalue_);
	static void GetFrequency(const float  fvalue_, int32_t  &f_);

	static void	GetGain(const int32_t  gain_, float  &gainvalue_);
	static void GetGain(const float  gainvalue_, int32_t  &gain_);


	static void	Get(
		int32_t	filtertype_,
		int32_t	freq_,
		int32_t	gain_,
		int32_t	qindex_,
		int32_t	alphaindex_,
		int32_t	eqon_,
		float	*coeff_
	);
	static void	GetThrough(float  *coeff_);

	static void	GetHonEQGainNo( const int32_t  gain_, int  &gain_no_);

	static bool	CheckUpdateMute_PEQ(
		int32_t	freq_from_,
		int32_t	gain_from_,
		int32_t	qindex_from__,

		int32_t	freq_to_,
		int32_t	gain_to_,
		int32_t	qindex_to__
	);
	static bool	CheckUpdateMute_Filter(
		int32_t	freq_from_,
		int32_t	qindex_from_,
		int32_t	alphaindex_from_,

		int32_t	freq_to_,
		int32_t	qindex_to_,
		int32_t	alphaindex_to_
	);

	
	static const float	m_lpf_alpha_table[];
	static const int	m_lpf_alpha_table_size;

	static const float	m_hpf_alpha_table[];
	static const int	m_hpf_alpha_table_size;

	static const float	m_filter_q_table[];
	static const int	m_filter_q_table_size;

	static const float	m_eq_q_table[];
	static const int	m_eq_q_table_size;
	*/

} // namespace dsp_lib
//======================================================================
