/************************************************************************
 *
 *	coeff_lowshelving.c
 *
 *	20020906	Tsuyoshi Ogawa
 *
 ************************************************************************/
#include "fdesign_prototypes.h"
#include "fdesign_const.h"


#include <cmath>

#ifndef	NULL
#include <stdio.h>
#endif

namespace dsp_lib {

/*============================================================*/
void
Fdesign_LowShelving_CheckParameter(
	float*	f_,
	float*	gain_
)
{
	Fdesign_PEQ_CheckParameter(f_, NULL, gain_);
}

/*============================================================*/
void
Fdesign_LowShelving(
	float	f_,
	float	gain_,
	int		bypass_,
	float*	coeff_
)
{
	/*------------------------------------------------------------*/
	float	f		= f_;
	float	gain	= gain_;

	float	pi		= M_PI;		/* 円周率 */
	float	fs		= M_FS;		/* サンプリング周波数 */

	float	b0, b1, b2, a1, a2;	/* 係数 */

	float	bb;					/* 分母 */

	float	tx, txtx, k, sqrk;
	float	var_dm;

	/*------------------------------------------------------------*/
	/* 1. 初期設定,入力値の整合性チェック, 制限値セット */
	Fdesign_LowShelving_CheckParameter(&f, &gain);

	/*------------------------------------------------------------*/
	if ((gain == 0.0f) || bypass_) {
		Fdesign_Through(coeff_);
	}
	else {
		/*------------------------------------------------------------*/
		/* 2. 共通項の計算 */
		var_dm		= pi * f / fs ;

		k		= powf(10.0f, fabsf(gain) / 20.0f);
		tx		= tanf(var_dm);
		txtx	= tx*tx;
		sqrk	= sqrtf(k);

		/*------------------------------------------------------------*/
		/* 3. 係数算出 */
		if (gain < 0.0f) {
			/*------------------------------------------------------------*/
			/* attenate <-12dB/oct> */
			var_dm	= sqrk * tx;
			bb	= (1.0f + var_dm) * (1.0f + var_dm);

			b0	= (1.0f + tx) * (1.0f + tx)		/ bb;
			b1	= 2.0f * (txtx - 1.0f)			/ bb;
			b2	= (1.0f - tx) * (1.0f - tx)		/ bb;
			a1	= 2.0f * (1.0f - k * txtx)		/ bb;
			a2	= -(1.0f - var_dm) * (1.0f - var_dm)	/ bb;

		} else {
			/*------------------------------------------------------------*/
			/* boost <-12dB/oct> */
			var_dm	= sqrk * tx ;
			bb	= (1.0f + tx) * (1.0f + tx) ;

			b0	= (1.0f + var_dm) * (1.0f + var_dm)		/ bb;
			b1	= 2.0f * (k * txtx - 1.0f)		/ bb;
			b2	= (1.0f - var_dm) * (1.0f - var_dm)		/ bb;
			a1	= 2.0f * (1.0f - txtx)			/ bb;
			a2	= -(1.0f - tx) * (1.0f - tx)	/ bb;
		}
		/*------------------------------------------------------------*/
		if (coeff_) {
			coeff_[0] = b0;
			coeff_[1] = b1;
			coeff_[2] = b2;
			coeff_[3] = -a1;
			coeff_[4] = -a2;
		}
		/*------------------------------------------------------------*/
	}
}
/*============================================================*/
} // namespace dsp_lib
