/************************************************************************
 *
 *	coeff_notch.c
 *
 *	20020920	Tsuyoshi Ogawa
 *
 ************************************************************************/
#include "fdesign_prototypes.h"
#include "fdesign_const.h"


#include <cmath>

#ifndef	NULL
#include <stdio.h>
#endif

namespace dsp_lib {

/*============================================================*/
void
Fdesign_NotchFilter_CheckParameter(
	float*	f_,
	float*	q_
)
{
	Fdesign_PEQ_CheckParameter(f_, q_, NULL);
}

/*============================================================*/
void
Fdesign_NotchFilter(
	float	f_,
	float	q_,
	int		bypass_,
	float*	coeff_
)
{
	/*------------------------------------------------------------*/
	float	f		= f_;
	float	q		= q_;

	float	pi		= M_PI;		/* 円周率 */
	float	fs		= M_FS;		/* サンプリング周波数 */

	float	b0, b1, b2, a1, a2;	/* 係数 */

	float	bb;					/* 分母 */
	float	r, k, rk;
	float	var_dm;

	/*------------------------------------------------------------*/
	/* 1. 初期設定,入力値の整合性チェック, 制限値セット */
	Fdesign_APF_CheckParameter(&f, &q);

	/*------------------------------------------------------------*/
	if (bypass_) {
		Fdesign_Through(coeff_);
	}
	else {
		/*------------------------------------------------------------*/
		/* 2. 共通項の計算 */
		var_dm	= pi * f / fs ;

		r	= 0.5f / q;
		k	= 2.0f * sinf(var_dm) ;
		rk	= r * k ;

		/*------------------------------------------------------------*/
		/* 3. 係数算出 */
		bb = 1.0f + rk;

		b0	= 1.0f				/ bb;
		b1	= (k * k - 2.0f)	/ bb;
		b2	= b0;
		a1	= -b1;
		a2	= -(1.0f - rk)		/ bb;

		/*------------------------------------------------------------*/
		if (coeff_) {
			coeff_[0] = b0;
			coeff_[1] = b1;
			coeff_[2] = b2;
			coeff_[3] = -a1;
			coeff_[4] = -a2;
		}
		/*------------------------------------------------------------*/
	}
}
/*============================================================*/
} // namespace dsp_lib
