/************************************************************************
 *
 *	fdesign_peq.cpp
 *
 *	20020906	Tsuyoshi Ogawa
 *
 ************************************************************************/
#include <cmath>
#include <dsp_lib/misc/biquad/fdesign_const.h>
#include <dsp_lib/misc/biquad/fdesign_prototypes.h>

//======================================================================
namespace dsp_lib {

/*============================================================*/
void
Fdesign_PEQ_CheckParameter(
	float*	f_,
	float*	q_,
	float*	gain_
)
{
	if (f_) {
		if (*f_ < 20.0f) {
			*f_ = 20.0f;
		} else if (20000.0f < *f_) {
			*f_ = 20000.0f;
		}
	}
	if (q_) {
		if (*q_ < 0.01f) {
			*q_ = 0.01f;
		} else if (70.0f < *q_) {
			*q_ = 70.00f;
		}
	}
	if (gain_) {
		if (*gain_ < -96.0f) {
			*gain_ = -96.0f;
		} else if (15.0f < *gain_) {
			*gain_ = 15.0f;
		}
	}
}

/*============================================================*/
void
Fdesign_PEQ(
	float	f_,
	float	q_,
	float	gain_,
	int		bypass_,
	float*	coeff_
)
{
	/*------------------------------------------------------------*/
	float	f		= f_;
	float	q		= q_;
	float	gain	= gain_;

	float	pi		= M_PI;		/* 円周率 */
	float	fs		= M_FS;		/* サンプリング周波数 */

	float	b0, b1, b2, a1, a2;	/* 係数 */

	float	bb;					/* 分母 */
	float	tx, txtx, tq, k;
	float	_dm;

	/*------------------------------------------------------------*/
	/* 1. 初期設定,入力値の整合性チェック, 制限値セット */
	Fdesign_PEQ_CheckParameter(&f, &q, &gain);

	/*------------------------------------------------------------*/
	if ((gain == 0.0f) || bypass_) {
		Fdesign_Through(coeff_);
	}
	else {
		/*------------------------------------------------------------*/
		/* 2. 共通項の計算 */
		_dm		= pi * f / fs ;

		k		= powf(10.0f, fabsf(gain) / 20.0f) ;
		tx		= tanf(_dm) ;
		tq		= tx / q;
		txtx	= tx*tx;

		/*------------------------------------------------------------*/
		/* 3. 係数算出 */
		if (gain < 0.0f) {
			/* Dip  */
			_dm	= k / q * tx ;
			bb	= 1.0f + _dm + txtx;

			b0	= (1.0f + tq + txtx)	/ bb;
			b1	= 2.0f * (txtx - 1.0f)	/ bb;
			b2	= (1.0f - tq + txtx)	/ bb;
			a1	= -b1;
			a2	= -(1.0f - _dm + txtx)	/ bb;
		}
		else {
			/* Peak */
			_dm	= k / q * tx;
			bb	= 1.0f + tq + txtx;

			b0	= (1.0f + _dm + txtx)	/ bb;
			b1	= 2.0f * (txtx - 1.0f)	/ bb;
			b2	= (1.0f - _dm + txtx)	/ bb;
			a1	= -b1;
			a2	= -(1.0f - tq + txtx)	/ bb;
		}
		/*------------------------------------------------------------*/
		if (coeff_) {
			coeff_[0] = b0;
			coeff_[1] = b1;
			coeff_[2] = b2;
			coeff_[3] = -a1;
			coeff_[4] = -a2;
		}
		/*------------------------------------------------------------*/
	}
	/*------------------------------------------------------------*/
}

} // namespace dsp_lib
//======================================================================
