/************************************************************************
 *
 *	coeff_xover_bessel.c
 *
 *	20020925	Tsuyoshi Ogawa
 *
 ************************************************************************/

#include "fdesign_prototypes.h"

namespace dsp_lib {

/*============================================================*/
#define	COEFF_XOVER_BS12_Q				0.57735f
#define	COEFF_XOVER_LPF_BS12_ALPHA		1.2742f

#define	COEFF_XOVER_HPF_BS12_ALPHA		(1.0f / COEFF_XOVER_LPF_BS12_ALPHA)

/*--------------------------------------------------------------------------------*/
#define	COEFF_XOVER_BS18_Q				0.69104f
#define	COEFF_XOVER_LPF_BS18_ALPHA1		1.32475f
#define	COEFF_XOVER_LPF_BS18_ALPHA2		1.44993f

#define	COEFF_XOVER_HPF_BS18_ALPHA1		(1.0f / COEFF_XOVER_LPF_BS18_ALPHA1)
#define	COEFF_XOVER_HPF_BS18_ALPHA2		(1.0f / COEFF_XOVER_LPF_BS18_ALPHA2)

/*--------------------------------------------------------------------------------*/
#define	COEFF_XOVER_BS24_Q1				0.52133f
#define	COEFF_XOVER_BS24_Q2				0.80554f
#define	COEFF_XOVER_LPF_BS24_ALPHA1		1.43241f
#define	COEFF_XOVER_LPF_BS24_ALPHA2		1.60594f

#define	COEFF_XOVER_HPF_BS24_ALPHA1		(1.0f / COEFF_XOVER_LPF_BS24_ALPHA1)
#define	COEFF_XOVER_HPF_BS24_ALPHA2		(1.0f / COEFF_XOVER_LPF_BS24_ALPHA2)

/*============================================================*/
void
Fdesign_XOverLPF_BS12(
	float	f_,
	int		bypass_,
	float	*coeff1_,
	float	*coeff2_
)
{
	Fdesign_XOverLPF12(f_, COEFF_XOVER_BS12_Q, COEFF_XOVER_LPF_BS12_ALPHA, bypass_, coeff1_);
	Fdesign_Through   (                                                             coeff2_);
}

void
Fdesign_XOverLPF_BS18(
	float	f_,
	int		bypass_,
	float	*coeff1_,
	float	*coeff2_
)
{
	Fdesign_XOverLPF12(f_, COEFF_XOVER_BS18_Q, COEFF_XOVER_LPF_BS18_ALPHA1, bypass_, coeff1_);
	Fdesign_XOverLPF6 (f_,                     COEFF_XOVER_LPF_BS18_ALPHA2, bypass_, coeff2_);
}

void
Fdesign_XOverLPF_BS24(
	float	f_,
	int		bypass_,
	float	*coeff1_,
	float	*coeff2_
)
{
	Fdesign_XOverLPF12(f_, COEFF_XOVER_BS24_Q1, COEFF_XOVER_LPF_BS24_ALPHA1, bypass_, coeff1_);
	Fdesign_XOverLPF12(f_, COEFF_XOVER_BS24_Q2, COEFF_XOVER_LPF_BS24_ALPHA2, bypass_, coeff2_);
}

/*--------------------------------------------------------------------------------*/
void
Fdesign_XOverHPF_BS12(
	float	f_,
	int		bypass_,
	float	*coeff1_,
	float	*coeff2_
)
{
	Fdesign_XOverHPF12(f_, COEFF_XOVER_BS12_Q, COEFF_XOVER_HPF_BS12_ALPHA, bypass_, coeff1_);
	Fdesign_Through   (                                                             coeff2_);
}

void
Fdesign_XOverHPF_BS18(
	float	f_,
	int		bypass_,
	float	*coeff1_,
	float	*coeff2_
)
{
	Fdesign_XOverHPF12(f_, COEFF_XOVER_BS18_Q, COEFF_XOVER_HPF_BS18_ALPHA1, bypass_, coeff1_);
	Fdesign_XOverHPF6 (f_,                     COEFF_XOVER_HPF_BS18_ALPHA2, bypass_, coeff2_);
}

void
Fdesign_XOverHPF_BS24(
	float	f_,
	int		bypass_,
	float	*coeff1_,
	float	*coeff2_
)
{
	Fdesign_XOverHPF12(f_, COEFF_XOVER_BS24_Q1, COEFF_XOVER_HPF_BS24_ALPHA1, bypass_, coeff1_);
	Fdesign_XOverHPF12(f_, COEFF_XOVER_BS24_Q2, COEFF_XOVER_HPF_BS24_ALPHA2, bypass_, coeff2_);
}
/*============================================================*/
} // namespace dsp_lib
