/************************************************************************
 *
 *	coeff_xover_hpf12.c
 *
 *	20020925	Tsuyoshi Ogawa
 *
 ************************************************************************/
#include "fdesign_prototypes.h"
#include "fdesign_const.h"


#include <cmath>

namespace dsp_lib {

/*============================================================*/
void
Fdesign_XOverHPF12_CheckParameter(
	float	*f_,
	float	*q_,
	float	*alpha_
)
{
	Fdesign_XOverLPF12_CheckParameter(f_, q_, alpha_);
}

/*============================================================*/
void
Fdesign_XOverHPF12(
	float	f_,
	float	q_,
	float	alpha_,
	int		bypass_,
	float*	coeff_
)
{
	/*------------------------------------------------------------*/
	float	f		= f_;
	float	q		= q_;
	float	alpha	= alpha_;

	float	pi		= M_PI;		/* 円周率 */
	float	fs		= M_FS;		/* サンプリング周波数 */

	float	b0, b1, b2, a1, a2;	/* 係数 */

	float	bb;					/* 分母 */
	float	tx, txtx, tq;
	float	var_dm;

	/*------------------------------------------------------------*/
	/* 1. 初期設定,入力値の整合性チェック, 制限値セット */
	Fdesign_XOverHPF12_CheckParameter(&f, &q, &alpha);

	/*------------------------------------------------------------*/
	if (bypass_) {
		Fdesign_Through(coeff_);
	}
	else {
		/*------------------------------------------------------------*/
		/* 2. 共通項の計算 */
		var_dm		= pi * f / fs ;
		tx		= tanf(var_dm) * alpha;

		tq		= tx / q;
		txtx	= tx*tx;

		/*------------------------------------------------------------*/
		/* 3. 係数算出 */
		bb	= 1.0f + tq + txtx;

		b0	= 1.0f					/ bb;
		b1	= -2.0f * b0;
		b2	= b0;
		a1	= 2.0f * (1.0f - txtx)	/ bb;
		a2	= -(1.0f - tq + txtx)	/ bb;

		/*------------------------------------------------------------*/
		if (coeff_) {
			coeff_[0] = b0;
			coeff_[1] = b1;
			coeff_[2] = b2;
			coeff_[3] = -a1;
			coeff_[4] = -a2;
		}
		/*------------------------------------------------------------*/
	}
}
/*============================================================*/
} // namespace dsp_lib
