#pragma warning(disable:4996)
#include <stdint.h>
#include <iostream>
#include <fstream>
#include <shared/boot_info.h>
#include <spi/spi_api.h>
#include "simulated_malloc.h"
#include "map.h"
#include "access_sharedram.h"

using namespace std;

namespace dsp_ctrl {

uint32_t
GetSharedRamAddress(uint32_t local_address_)
{
 	uint32_t address = local_address_;
	if (address == 0x00000000) {
		return 0x00000000;
	}

	address -= g_local_heap_address_base;	// heap先頭からの相対アドレスに変換
	address /= ADDRESS_UNIT_RATIO;			// 相対アドレスのアドレッシング単位をDSPに合わせる
	address += SHAREDRAM_ADDRESS;			// SharedRAM上の絶対アドレスに変換
	return address;
}

void
ConvertMapAddress(void)
{
	// ローカルRAMのアドレスを、SharedRAMのアドレスに変換する
	for (int i = 0; i < MAP_LENGTH; i++) {
		if(g_map[i] != NULL) {
			g_map[i] = GetSharedRamAddress(g_map[i]);
			//g_map[i] -= reinterpret_cast<uint32_t>(g_map);	// MAP先頭からの相対アドレスに変換
			//g_map[i] /= ADDRESS_UNIT_RATIO;					// 相対アドレスのアドレッシング単位をDSPに合わせる
			//g_map[i] += MAP_ADDRESS;							// SharedRAM上の絶対アドレスに変換
		}
	}
}

void
ConvertNetListAddress(void)
{
	for (uint32_t i = 0; i < g_dsp_manager_config->net_list_size; i++) {
		g_dsp_manager_config->net_list[i].config = GetSharedRamPointer(g_dsp_manager_config->net_list[i].config);
		g_dsp_manager_config->net_list[i].param	 = GetSharedRamPointer(g_dsp_manager_config->net_list[i].param);
		g_dsp_manager_config->net_list[i].status = GetSharedRamPointer(g_dsp_manager_config->net_list[i].status);
	}
	g_dsp_manager_config->net_list = GetSharedRamPointer(g_dsp_manager_config->net_list);
	// 以降、g_dsp_manager_config->net_listによるネットリストへのアクセスは不可
}

bool
WriteDspConfiguration(void)
{
	// MAP
	ConvertMapAddress();
	for (int i = 0; i < MAP_LENGTH; i++) {
		uint32_t address	= MAP_ADDRESS + i;
		uint32_t data		= *(reinterpret_cast<uint32_t*>(g_local_heap_address_map)+i);
		if (SpiWrite(address, data, SPI_DEV_NUM) == false) {
			cout << "WriteDspConfiguration MAP: SPI xfr error : RETURN = ERROR" << endl;
			return false;
		}
	}
	//----------------------------------------
	// DM
	ConvertNetListAddress();
	for (int i = 0; i < SHAREDRAM_DM_LENGTH; i++) {
		uint32_t address	= SHAREDRAM_DM_ADDRESS + i;
		uint32_t data		= *(reinterpret_cast<uint32_t*>(g_local_heap_address_dm)+i);
		if (SpiWrite(address, data, SPI_DEV_NUM) == false) {
			cout << "WriteDspConfiguration DM: SPI xfr error : RETURN = ERROR" << endl;
			return false;
		}
	}
	//----------------------------------------
	// PM
	for (int i = 0; i < SHAREDRAM_PM_LENGTH; i++) {
		uint32_t address	= SHAREDRAM_PM_ADDRESS + i;
		uint32_t data		= *(reinterpret_cast<uint32_t*>(g_local_heap_address_pm)+i);
		if (SpiWrite(address, data, SPI_DEV_NUM) == false) {
			cout << "WriteDspConfiguration PM: SPI xfr error : RETURN = ERROR" << endl;
			return false;
		}
	}
	return true;
}

bool
RequestToImportDspConfiguration(void)
{
	if (WriteSharedRam(BOOT_INFO_ADDRESS + BOOT_INFO_NO_DSP_INITIALIZE_REQUEST, 1) == false) {
		cout << "RequestToImportDspConfiguration is failed!" << endl;
		return false;
	}
	return true;
}

bool
GetDspConfigurationImportedFlag(uint32_t*	flag_)
{
	if (ReadSharedRam(BOOT_INFO_ADDRESS + BOOT_INFO_NO_DSP_INITIALIZED_FLAG, flag_) == false) {
		cout << "GetDspConfigurationImportedFlag is failed!" << endl;
		return false;
	}
	return true;
}

bool
RequestToImportDspSetup(void)
{
	if (WriteSharedRam(GetSharedRamAddress(&g_dsp_manager_param->ready), 1) == false) {
		cout << "RequestToImportDspSetup is failed!" << endl;
		return false;
	}
	return true;
}

bool
GetDspSetupImportedFlag(uint32_t* flag_)
{
	if (ReadSharedRam(GetSharedRamAddress(&g_dsp_manager_status->ready_flag), flag_) == false) {
		cout << "GetDspSetupImportedFlag is failed!" << endl;
		return false;
	}
	return true;
}

} // namespace dsp_ctrl
