/*
 * dsp_setup.cpp
 *
 *  Created on: 2016/11/28
 *      Author: toa29548
 */

#include <cmath>
#include <dsp_lib/misc/biquad/fdesign.h>
#include <shared/boot_info.h>
#include "dsp_config.h"
#include "map.h"
#include "access_sharedram.h"
#include "read_input_file.h"

using namespace shared;
using namespace dsp_lib;

namespace dsp_ctrl {

uint32_t
ToDspAddressing(void* cpu_addressing_)
{
	uint32_t dsp_addressing = reinterpret_cast<uint32_t>(cpu_addressing_)/SHAREDRAM_ADDRESS_UNIT_BYTE;
	return dsp_addressing;
}

bool
DspSetupAec()
{
	//----------------------------------------
	// setup for cear-aec
	// microphonearray status -> aec param
	// the number of microphonearray module must be 1
	// the number of aec module must be 1

	//----------------------------------------
	dsp_lib::DspManager::NetList	netlist_dst;
	dsp_lib::DspManager::NetList	netlist_src;

	for (uint32_t list_no = 0; list_no < g_number_of_netlist; list_no++) {
		bool ret = true;

		ret &= ReadSharedRam(
			GetSharedRamAddress(&g_dsp_manager_net_list[list_no].type), &netlist_dst.type
		);

		ret &= ReadSharedRam(
			GetSharedRamAddress(&g_dsp_manager_net_list[list_no].param), &netlist_dst.param
		);
		if (ret == false) {
			return false;
		}
		if (netlist_dst.type == dsp_lib::DspManager::DSP_MODULE_ID_AEC) {
			break;
		}
	}
	for (uint32_t list_no = 0; list_no < g_number_of_netlist; list_no++) {
		bool ret = true;

		ret &= ReadSharedRam(
			GetSharedRamAddress(&g_dsp_manager_net_list[list_no].type), &netlist_src.type
		);

		ret &= ReadSharedRam(
			GetSharedRamAddress(&g_dsp_manager_net_list[list_no].status), &netlist_src.status
		);
		if (ret == false) {
			return false;
		}
		if (netlist_src.type == dsp_lib::DspManager::DSP_MODULE_ID_MICROPHONEARRAY) {
			break;
		}
	}
	{
		uint32_t	param_base = reinterpret_cast<uint32_t>(netlist_dst.param);
		Aec::Param*	param_offset = 0x00000000;

		if (WriteSharedRam(param_base + ToDspAddressing(&param_offset->microphonearray), netlist_src.status) == false) {
			return false;
		}
	}

	return true;
}

float
ToAntiLog(float db_)
{
	return powf(10, db_ * 0.05f);
}

bool
DspConfigurationIsImported(uint32_t timeout_)
{
	uint32_t time = 0;
	//volatile uint32_t* flag = BOOT_INFO(BOOT_INFO_NO_DSP_INITIALIZED_FLAG);
	uint32_t flag = 0;
#if SPI_ENABLE
	while (flag == 0) {
		if (GetDspConfigurationImportedFlag(&flag) == false) {
			return false;
		}
		//time++;
		//if(time > timeout_)
		//{
		//	return false;
		//}
	}
#endif
	return true;
}

bool
DspSetupIsCompleted(uint32_t timeout_)
{
	uint32_t time = 0;
	uint32_t flag = 0;
#if SPI_ENABLE
	while (flag == 0) {
		if (GetDspSetupImportedFlag(&flag) == false) {
			return false;
		}
		//time++;
		//if(time > timeout_)
		//{
		//	return false;
		//}
	}
#endif
	return true;
}

bool
DspSetup(void)
{
	/* DSPパラメータ設定 */

#if 0
	// ----- Matrix -----

	if (Matrix::SetGain(
			&g_matrix_config[0],
			&g_matrix_param[0],
			/*input no*/0,
			/*output no*/0,
			ToAntiLog(-6.0f)
	) == false) {
		return false;
	}

	if (Matrix::SetGain(
			&g_matrix_config[0],
			&g_matrix_param[0],
			/*input no*/0,
			/*output no*/1,
			ToAntiLog(-12.0f)
	) == false) {
		return false;
	}

	if (Matrix::SetTimeFactor(
			&g_matrix_param[0],
			/*attack*/2.000f,
			/*release*/2.000f
	) == false) {
		return false;
	}

	// ----- Gain -----

	if (Gain::SetGain(
			&g_gain_param[0],
			ToAntiLog(-3.0f)
	) == false) {
		return false;
	};

	for (int gain_no = 1; gain_no < g_number_of_gain - 1; gain_no++)
	{
		if (Gain::SetGain(
				&g_gain_param[gain_no],
				ToAntiLog(0.0f)
		) == false) {
			return false;
		};
	}

	if (Gain::SetGain(
			&g_gain_param[g_number_of_gain - 1],
			ToAntiLog(-3.0f)
	) == false) {
		return false;
	};

	// ----- Iir -----

	if (Iir::SetIirCoeff(
			FILTER_TYPE_PEQ,
			500.0f,
			6.0f,
			4.318f,
			0,
			1,
			g_iir_param_coef_group[0] + 0
			) == false) {
		return false;
	}

	if (Iir::SetIirCoeff(
			FILTER_TYPE_PEQ,
			1000.0f,
			6.0f,
			4.318f,
			0,
			1,
			g_iir_param_coef_group[0] + g_number_of_iir_stage[0] - 1
			) == false) {
		return false;
	}

	uint32_t no = g_number_of_iir - 1;
	if (g_number_of_iir > 1) {
		if (Iir::SetIirCoeff(
				FILTER_TYPE_PEQ,
				2000.0f,
				6.0f,
				4.318f,
				0,
				1,
				g_iir_param_coef_group[no] + g_number_of_iir_stage[no] - 1
		) == false) {
			return false;
		}
	}

	// ----- Gate -----

	uint32_t gate_level_type = Gate::GATE_LEVEL_TYPE_RMS;

	g_gate_param[0].threshold	= -20.0f;
	g_gate_param[0].depth	 	= -8.0f;
	g_gate_param[0].level_type	= gate_level_type;

	if (g_number_of_gate > 1) {
		for (int gate_no = 1; gate_no < g_number_of_gate - 1; gate_no++)
		{
			g_gate_param[gate_no].threshold	= 0.0f;
			g_gate_param[gate_no].depth		= 0.0f;
			g_gate_param[gate_no].level_type	= gate_level_type;
		}

		g_gate_param[g_number_of_gate - 1].threshold	= -20.0f;
		g_gate_param[g_number_of_gate - 1].depth	 	= -3.0f;
		g_gate_param[g_number_of_gate - 1].level_type = gate_level_type;
	}

	// ----- Comp -----

	uint32_t comp_level_type = Comp::COMP_LEVEL_TYPE_PEAK;

	g_comp_param[0].threshold	= ToAntiLog(-12.0f);
	g_comp_param[0].ratio	 	= Comp::COMP_RATIO_INF;
	g_comp_param[0].level_type	= comp_level_type;

	if (g_number_of_comp > 1) {
		for (int comp_no = 1; comp_no < g_number_of_comp - 1; comp_no++)
		{
			g_comp_param[comp_no].threshold		= ToAntiLog(0.0f);
			g_comp_param[comp_no].ratio	 		= Comp::COMP_RATIO_1;
			g_comp_param[comp_no].level_type 	= comp_level_type;
		}

		g_comp_param[g_number_of_comp - 1].threshold	= ToAntiLog(0.0f);
		g_comp_param[g_number_of_comp - 1].makeup_gain	= ToAntiLog(3.0f);
		g_comp_param[g_number_of_comp - 1].level_type 	= comp_level_type;
	}

	// ----- Delay -----

	uint32_t delay_samples = 48;

	g_delay_param[0].delay_samples = delay_samples;

	if (g_number_of_delay > 1) {
		for (int delay_no = 1; delay_no < g_number_of_delay - 1; delay_no++)
		{
			g_delay_param[delay_no].delay_samples = delay_samples;
		}

		g_delay_param[g_number_of_delay - 1].delay_samples = delay_samples;
	}

	// ----- Aec -----

	uint32_t aec_on = 1;

	g_aec_param[0].speex_echo_aec = aec_on;

	if (g_number_of_aec > 1) {
		for (int aec_no = 1; aec_no < g_number_of_aec - 1; aec_no++)
		{
			g_aec_param[aec_no].speex_echo_aec = aec_on;
		}

		g_aec_param[g_number_of_aec - 1].speex_echo_aec = aec_on;
	}

	// ----- Nc -----

	int		nc_bypass = 1;

	g_nc_param[0].bypass = nc_bypass;

	if (g_number_of_nc > 1) {
		for (int nc_no = 1; nc_no < g_number_of_nc - 1; nc_no++)
		{
			g_nc_param[nc_no].bypass = nc_bypass;
		}

		g_nc_param[g_number_of_nc - 1].bypass = nc_bypass;
	}
	// ----- Microphonearray -----

	// do nothing

	// ----- Gain -----
	//WriteSharedRam(
	//	GetSharedRamAddress(&g_gain_param[0].instant_gain_param.gain_target),
	//	ToAntiLog(-3.0f)
	//);

	if (Gain::SetGain(
			&g_gain_param[0],
			ToAntiLog(-3.0f)
	) == false) {
		return false;
	};

	WriteSharedRam(
		GetSharedRamAddress(&g_gain_param[0].gain_target),
		g_gain_param[0].gain_target
	);

	// ----- Iir -----
	if (Iir::SetIirCoeff(
			FILTER_TYPE_PEQ,
			500.0f,
			6.0f,
			4.318f,
			0,
			1,
			g_iir_param_coef_group[0] + 0
			) == false) {
		return false;
	}

	if (Iir::SetIirCoeff(
			FILTER_TYPE_PEQ,
			1000.0f,
			6.0f,
			4.318f,
			0,
			1,
			g_iir_param_coef_group[0] + g_number_of_iir_stage[0] - 1
			) == false) {
		return false;
	}

	for (int coef_no = 0; coef_no < g_number_of_iir_stage[0]; coef_no++)
	{
		WriteSharedRam(
			GetSharedRamAddress(g_iir_param_coef_group[0] + 0) + coef_no,
			*(reinterpret_cast<float*>(g_iir_param_coef_group[0] + 0) + coef_no)
			);
		WriteSharedRam(
			GetSharedRamAddress(g_iir_param_coef_group[0] + g_number_of_iir_stage[0] - 1) + coef_no,
			*(reinterpret_cast<float*>(g_iir_param_coef_group[0] + g_number_of_iir_stage[0] - 1) + coef_no)
			);
	}
	//if (g_number_of_iir > 1) {
	//	uint32_t no = g_number_of_iir - 1;
	//	if (Iir::SetIirCoeff(
	//			FILTER_TYPE_PEQ,
	//			2000.0f,
	//			6.0f,
	//			4.318f,
	//			0,
	//			1,
	//			g_iir_param_coef_group[no] + g_number_of_iir_stage[no] - 1
	//	) == false) {
	//		return false;
	//	}
	//}

#endif
#if 1
	if (DspSetupAec() == false) {
		return false;
	}
#endif

    /* DSPに初期パラメータ取り込みを依頼 */
	if (RequestToImportDspSetup() == false) {
		return false;
	}
    return true;
}

} // namespace dsp_ctrl
