/*
 *	dsp_gate.h
 *
 *	20161021	Takashi Yamaji
 */
#pragma once

#include <stdint.h>

#include <dsp_lib/include/misc/dsp_const.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/dsp/peak.h>
#include <dsp_lib/include/dsp/rms.h>

namespace dsp_lib {

class Gate : public DspModule {
public:
	// -------------------------------------------------------------
	// 定数
	// -------------------------------------------------------------
	enum GATE_LEVEL_TYPE {
		GATE_LEVEL_TYPE_PEAK	= 0,
		GATE_LEVEL_TYPE_RMS		= 1
	};

	enum GATE_STATUS {
		GATE_STATUS_CLOSE	= 0,
		GATE_STATUS_HOLD	= 1,
		GATE_STATUS_OPEN	= 2
	};

	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------
	struct Config{};
	struct Param {
		uint32_t	attack;				//!< ゲイン上昇完了に要するサンプル数[samples]
		uint32_t	release;			//!< ゲイン下降完了に要するサンプル数[samples]
		uint32_t	hold;				//!< ホールドするブロック数[blocks]
		float		threshold;			//!< [dBFS]
		float		hysteresis;			//!< [dB]
		float		depth;				//!< CLOSE時のゲインの目標値[dB]
		uint32_t	level_type;			//!< レベルセンシングの種類, GATE_LEVEL_TYPE 参照
	};

	struct Status {
		float		gain_target;		//!< ゲインの目標値[dB]
		float		gain_current;		//!< ゲインの現在値[dB]
		float		peak;				//!< 直近5msecのpeak, 単位:振幅の2乗
		float		peak_release;		//!< peakと同様に上昇するが、ゆっくりと下降する, 単位:振幅の2乗
		float		rms;				//!< サンプルの2乗平均, 平方根は取っていない
		uint32_t	gate_status;		//!< ゲート開閉情報, GATE_STATUS 参照
	};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------
	static bool Configure(void) {
		return true;
	};
	static bool Attach(void) {
		return true;
	}

	Gate(
			Config*	/*config_*/,
			Param*	param_,
			Status*	status_
	);

	bool Ready(void);

	bool Execute(
			const float*	/*input_*/,
				  float*	/*output_*/
	);
	bool Task(void);

protected:
	void execute_gate_calc_level(
			const float*	input_
	);
	void execute_gate_calc_target_gain(void);
	void execute_gate_calc_current_gain(void);
	void execute_gate_gain(
			const float*	input_,
			float*			output_
	);

//	Config*			m_config;
	Param*			m_param;
	Status*			m_status;

	Rms*			m_rms;
	Rms::Status		m_rms_status;

	Peak*			m_peak;
	Peak::Status	m_peak_status;

	float			m_gain[DSP_BLOCK_SIZE];	//!< TODO:TIはDSPライブラリの仕様に合わせるため8byte境界に置くこと, 単位:真数
	float			m_gain_current;			//!< 単位:dB
	float			m_level;				//!< 単位:振幅の2乗
	uint32_t		m_hold_count;
};

} //namespace dsp_lib
