/*
 * main.cpp
 */

//********************************************************************************
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include "spi/spi_driver.h"
#include "misc/error.h"
#include "dsp_ctrl/dsp_init.h"
#include "dsp_ctrl/dsp_setup.h"
#include "dsp_ctrl/read_input_file.h"
#include "dsp_ctrl/map.h"
#include "dsp_ctrl/access_sharedram.h"
#include "dsp_ctrl/memory_dump.h"
#include "dsp_ctrl/write_netlist.h"
#include <picojson.h>

using namespace std;

//********************************************************************************
int main(int argc, char *argv[])
{
	/* DSPがSharedRAMを0クリア */
	/* SPI通信を確立 */

	/* 設定ファイル読み込み */
	cout << "Reading input file..." << endl;
	if(argc != 3){
		cout << "usage: startup.out BLOCK.json NETLIST.bin" << endl;
		return -1;
	}

	picojson::array block_cfg;	
	if (dsp_ctrl::ReadInputFile(argv[1], block_cfg) == false) {
		cout << "failed!" << endl;
		return -1;
	}

	/* CPUがコンフィギュレーション書き込み */
	cout << "Dsp configuration..." << endl;
	if (dsp_ctrl::DspConfiguration(block_cfg) == false) {
		cout << "failed!" << endl;
		return -1;
	}

	/* DSPがCPUからのコンフィギュレーションを取り込み要求を待つ */
	/* DSPがコンフィギュレーションを取り込み */

	/* CPUがDSPのコンフィギュレーション取り込み完了を待つ */
	cout << "Waiting dsp configuration import..." << endl;
	if (dsp_ctrl::DspConfigurationIsImported(10000) == false) {
		cout << "failed!" << endl;
		return -1;
	}

	/* CPUが初期パラメーター設定書き込み */
	cout << "Dsp setup..." << endl;
	if (dsp_ctrl::DspSetup() == false) {
		cout << "failed!" << endl;
		return -1;
	}

	/* DSPが初期パラメーターを取り込み */

	/* 初期パラメーター設定完了を確認 */
	cout << "Waiting Dsp setup completed..." << endl;
	if (dsp_ctrl::DspSetupIsCompleted(10000) == false) {
		cout << "failed!" << endl;
		return -1;
	}

	/* LocalRAMとSharedRAMのメモリダンプ */
	cout << "Memory dump..." << endl;
	if (dsp_ctrl::MemoryDump() == false) {
		cout << "failed!" << endl;
		return -1;
	}

	cout << "Save netlist..." << endl;
	if (dsp_ctrl::SaveNetList(argv[2]) == false) {
		cout << "failed!" << endl;
		return -1;
	}

	cout << "Save heap info..." << endl;
	if (dsp_ctrl::SaveHeapInfo() == false) {
		cout << "failed!" << endl;
		return -1;
	}

	cout << "finished!" << endl;
	return 0;
}
