/*--------------------------------------------------------------------------*/
/*																			*/
/*	CtimerLib.c (Rev 1.00)													*/
/*																			*/
/*	1.概要																	*/
/*		カウントアップタイマ												*/
/*																			*/
/*	2.改版履歴																*/
/*																			*/
/*																			*/
/*--------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <time.h>
#include "CtimerLib.h"

#define   CTIME_LIST			16
#define   CTME_ACCURACY			100											/* タイマーの解像度 msec単位		*/

int32_t					CoutupTimerLoop    = 0; 
int32_t					IsCountupTimerOpen = 0; 
pthread_t 				ThreadCountupTimer; 
COUTUP_TIMER_LIST		TimerList[CTIME_LIST];


static int32_t CoutupTimerLibDeinit( void );


static void Bm64DPrintf( const char *Fmt, ... ) 
{ 
#if 0
    int8_t str[512]; 
    va_list ap; 
        
    va_start(ap, Fmt); 
    vsprintf(str, Fmt, ap); 
    va_end(ap); 
    printf(str);

/*	fflush(console); */
#endif
}


void* CoutupTimerLibWorkerThread( void *Arg ) 
{ 
	uint32_t ii;
    while (CoutupTimerLoop) {

		usleep((CTME_ACCURACY*1000));										/* カウントアップタイマの周期生成	*/

		for(ii=0; ii<CTIME_LIST; ii++){
			if ((TimerList[ii].TimerID[0] != 0) 
			&&  (TimerList[ii].ThresholdFirst > TimerList[ii].Counter)){	

				TimerList[ii].Counter 			+= CTME_ACCURACY;			/* カウントアップ(msec)				*/
			}
			else{
																			/* 何もしない						*/
			};
		};
	};

    return((void *)CTIMER_STATUS_SUCCESS);
};

int32_t LibCTIMER_setTimer( char* name, long first){

	uint32_t ii;
	int			ret = CTIMER_STATUS_FALSE;

	if( ((name[0] != 0) && (strlen(name) < 31)) 
    &&  ((first >= 100)  && (first <= (60 * 60 * 1000))) ){

		for(ii=0; ii<CTIME_LIST; ii++){
			if( strcmp((const char*)name, (const char*)TimerList[ii].TimerID) == 0 ){					/* 登録済みtimer再設定		*/

				TimerList[ii].ThresholdFirst 	= first;
				TimerList[ii].Counter 			= 0;
				TimerList[ii].Correct 			= 0;													/* 補正値（未使用）			*/
				ret = CTIMER_STATUS_SUCCESS;
				break;
			}
			else if (TimerList[ii].TimerID[0] == 0){													/* timer新規作成			*/

				strcpy(TimerList[ii].TimerID, name);
				TimerList[ii].ThresholdFirst 	= first;
				TimerList[ii].Counter 			= 0;
				ret = CTIMER_STATUS_SUCCESS;
				break;
			};
		};
	};
	return ret;
}


int32_t LibCTIMER_getTimeout( char* name ){

	uint32_t ii;
	int			ret = 0;

	if(name != 0){

		for(ii=0; ii<CTIME_LIST; ii++){
			if( strcmp((const char*)name, (const char*)TimerList[ii].TimerID) == 0 ){					/* 登録済みtimer再設定						*/

				if(TimerList[ii].ThresholdFirst <= TimerList[ii].Counter){
					TimerList[ii].TimerID[0] = 0;														/* タイムアウトを渡すとタイマー削除する		*/
					ret = 1;																			/* タイムアウト								*/
				}
				break;
			};
		};
	};
	return ret;
}


void LibCTIMER_cancelTimer( char* name )
{

	uint32_t ii;

	if(name != 0)
	{

		for(ii=0; ii<CTIME_LIST; ii++)
		{
			if( strcmp((const char*)name, (const char*)TimerList[ii].TimerID) == 0 )					/* 登録済みtimer検索		*/
			{

				TimerList[ii].TimerID[0] = 0;															/* タイマー削除する	*/
				break;
			}
		}
	}
}

int32_t LibCTIMER_Init( void ) 
{ 
    int32_t ret; 
    
    Bm64DPrintf("LibCTIMER build:%s Time:%s\n", __DATE__, __TIME__); 
    
    if (IsCountupTimerOpen) 
    {
        return(CTIMER_STATUS_FALSE);
    };
    
    CoutupTimerLoop = 1; 
	memset(TimerList,0,sizeof(TimerList));

    ret = pthread_create(&ThreadCountupTimer, NULL, CoutupTimerLibWorkerThread, NULL); 
    if (ret != 0) 
    {
        CoutupTimerLibDeinit(); 
        return(CTIMER_STATUS_FALSE);
    };
    IsCountupTimerOpen = 1;
    return(CTIMER_STATUS_SUCCESS);
}


int32_t LibCTIMER_Deinit( void ) 
{ 
    
    if (IsCountupTimerOpen == 0) 
    {
        return(CTIMER_STATUS_FALSE);
    }
    
	CoutupTimerLibDeinit();
    IsCountupTimerOpen = 0;
    return(CTIMER_STATUS_SUCCESS);
}

static int32_t CoutupTimerLibDeinit( void ) 
{ 
    void *tRet;

    if (IsCountupTimerOpen)
    {
        CoutupTimerLoop = 0;
        pthread_join(ThreadCountupTimer, &tRet);
        
        ThreadCountupTimer = 0;
        IsCountupTimerOpen = 0;
    }
    
    return(CTIMER_STATUS_SUCCESS);
}
