#include <stdio.h>
#include <stdint.h>
#include "wiringPi.h"
#include "GpioControl.h"

// #define g_Test_Enable	// S1-Test

st_inputGPIO gst_inputGpio[MAX_INPUT_NUMBER];
uint32_t g_MuteStatus = 0;
uint32_t g_StandbyMode = 0;
uint32_t g_ShutdownMode = 0;
//uint32_t g_CountOffset = 0; //non used
//uint32_t g_CountShutdown = 0; //non used
uint32_t g_CountStopShutdown = 0;
uint32_t g_CfgCtrlInput[MAX_CTRL_IO_NUMBER];
uint32_t g_CfgCtrlOutput[MAX_CTRL_IO_NUMBER];

void Initialize_GPIO(void)
{
    uint32_t i;

    for(i = 0; i < MAX_INPUT_NUMBER; i++){
        gst_inputGpio[i].count = 0;
        gst_inputGpio[i].flag = 0;
    }

    for(i = 0; i < MAX_CTRL_IO_NUMBER; i++){
        g_CfgCtrlInput[i] = CIN_FUNC_NONE;
        g_CfgCtrlOutput[i] = COUT_FUNC_NONE;
    }

	if (wiringPiSetupGpio() == -1) {
		return;
	}

    pullUpDnControl(GPIO_MOSI, PUD_UP);
	pullUpDnControl(GPIO_MISO, PUD_UP);

    pinMode(GPIO_HEART_BEAT, INPUT);                // Shutdown Mode 以外はInputに設定
    pullUpDnControl(GPIO_HEART_BEAT, PUD_UP);

	pinMode(GPIO_CONTROL_IN1, INPUT);
	pullUpDnControl(GPIO_CONTROL_IN1, PUD_DOWN);

	pinMode(GPIO_CONTROL_IN2, INPUT);
	pullUpDnControl(GPIO_CONTROL_IN2, PUD_DOWN);

	pinMode(GPIO_PWR_DOWN_DET, INPUT);
	pullUpDnControl(GPIO_PWR_DOWN_DET, PUD_UP);

	pinMode(GPIO_USB_AUDIO_ST, INPUT);
	pullUpDnControl(GPIO_USB_AUDIO_ST, PUD_DOWN);

	pinMode(GPIO_CONTROL_OUT1, OUTPUT);
    digitalWrite(GPIO_CONTROL_OUT1, 0);

	pinMode(GPIO_CONTROL_OUT2, OUTPUT);
    digitalWrite(GPIO_CONTROL_OUT2, 0);

	pinMode(GPIO_PC_VOL_UP, OUTPUT);
    digitalWrite(GPIO_PC_VOL_UP, 0);

	pinMode(GPIO_PC_VOL_DOWN, OUTPUT);
    digitalWrite(GPIO_PC_VOL_DOWN, 0);

	pinMode(GPIO_PC_VOL_MUTE, OUTPUT);
    digitalWrite(GPIO_PC_VOL_MUTE, 0);

    gst_inputGpio[INPUT2_CONTROL_INPUT1].prev = 0;
    gst_inputGpio[INPUT2_CONTROL_INPUT1].current = gst_inputGpio[INPUT2_CONTROL_INPUT1].prev;
    gst_inputGpio[INPUT2_CONTROL_INPUT1].temp[0] = gst_inputGpio[INPUT2_CONTROL_INPUT1].prev;
    gst_inputGpio[INPUT2_CONTROL_INPUT1].temp[1] = gst_inputGpio[INPUT2_CONTROL_INPUT1].prev;
    gst_inputGpio[INPUT2_CONTROL_INPUT1].index = 0;

    gst_inputGpio[INPUT3_CONTROL_INPUT2].prev = 0;
    gst_inputGpio[INPUT3_CONTROL_INPUT2].current = gst_inputGpio[INPUT3_CONTROL_INPUT2].prev;
    gst_inputGpio[INPUT3_CONTROL_INPUT2].temp[0] = gst_inputGpio[INPUT3_CONTROL_INPUT2].prev;
    gst_inputGpio[INPUT3_CONTROL_INPUT2].temp[1] = gst_inputGpio[INPUT3_CONTROL_INPUT2].prev;
    gst_inputGpio[INPUT3_CONTROL_INPUT2].index = 0;

    g_MuteStatus = 0;
    //g_CountOffset = 0;//non used
    g_ShutdownMode = 0;
    //g_CountShutdown = 0; //non used
    g_StandbyMode = 0;
    g_CountStopShutdown = 0;
}

void Initialize_GPIOStatus(void)
{
    uint32_t i;

    for(i = 0; i < MAX_INPUT_NUMBER; i++){
        gst_inputGpio[i].count = 0;
        gst_inputGpio[i].flag = 0;
    }

	// For No16,No120
	// コンフィグの設定は前回値を保持しておく。
	// (I2CのLED設定も同様にスタンバイモードに移行しても設定はリセットしていないので
	//  GPIOも同じくリセットしない。)
	/*
    for(i = 0; i < MAX_CTRL_IO_NUMBER; i++){
        g_CfgCtrlInput[i] = CIN_FUNC_NONE;
        g_CfgCtrlOutput[i] = COUT_FUNC_NONE;
    }
	*/

    //gst_inputGpio[INPUT0_POWER_SWITCH].prev = Get_PowerSwitch();								//Second PCBからI2Cで制御
    //gst_inputGpio[INPUT0_POWER_SWITCH].current = gst_inputGpio[INPUT0_POWER_SWITCH].prev;			//Second PCBからI2Cで制御

	// For No191,No197
    //gst_inputGpio[INPUT2_CONTROL_INPUT1].prev = Get_ControlInput1();
    //gst_inputGpio[INPUT2_CONTROL_INPUT1].current = gst_inputGpio[INPUT2_CONTROL_INPUT1].prev;
    //gst_inputGpio[INPUT2_CONTROL_INPUT1].temp[0] = gst_inputGpio[INPUT2_CONTROL_INPUT1].prev;
    //gst_inputGpio[INPUT2_CONTROL_INPUT1].temp[1] = gst_inputGpio[INPUT2_CONTROL_INPUT1].prev;
    //gst_inputGpio[INPUT2_CONTROL_INPUT1].index = 0;

    //gst_inputGpio[INPUT3_CONTROL_INPUT2].prev = Get_ControlInput2();
    //gst_inputGpio[INPUT3_CONTROL_INPUT2].current = gst_inputGpio[INPUT3_CONTROL_INPUT2].prev;
    //gst_inputGpio[INPUT3_CONTROL_INPUT2].temp[0] = gst_inputGpio[INPUT3_CONTROL_INPUT2].prev;
    //gst_inputGpio[INPUT3_CONTROL_INPUT2].temp[1] = gst_inputGpio[INPUT3_CONTROL_INPUT2].prev;
    //gst_inputGpio[INPUT3_CONTROL_INPUT2].index = 0;

    g_MuteStatus = 0;
    //g_CountOffset = 0; //non used
    //g_ShutdownMode = 0; //non used
    //g_CountShutdown = 0; //non used

#ifdef g_Test_Enable	// S1-Test
	printf( "IN1[%d] IN2[%d] OUT1[%d] OUT2[%d]\n", g_CfgCtrlInput[0], g_CfgCtrlInput[1], g_CfgCtrlOutput[0], g_CfgCtrlOutput[1] );
#endif
}

void Initialize_CtrlInputConfig(uint32_t i_cfg1, uint32_t i_cfg2)
{
    if((CIN_FUNC_MAX_NUMBER <= i_cfg1) || (CIN_FUNC_MAX_NUMBER <= i_cfg2)){
        return;
    }
    g_CfgCtrlInput[0] = i_cfg1;
    g_CfgCtrlInput[1] = i_cfg2;
}

void Initialize_CtrlOutputConfig(uint32_t i_cfg1, uint32_t i_cfg2)
{
    if((COUT_FUNC_MAX_NUMBER <= i_cfg1) || (COUT_FUNC_MAX_NUMBER <= i_cfg2)){
        return;
    }
    g_CfgCtrlOutput[0] = i_cfg1;
    g_CfgCtrlOutput[1] = i_cfg2;
}

uint32_t Is_ValidCtrlIoNumber(uint32_t i_num)
{
    if(MAX_CTRL_IO_NUMBER <= i_num){
        return 0;   //invalid
    }
    return 1;   //valid
}

uint32_t Is_CtrlInputCfgPulse(uint32_t i_num)
{
    uint32_t cfg;

    if(!Is_ValidCtrlIoNumber(i_num)){
        return(CFG_LEVEL);
    }
    cfg = g_CfgCtrlInput[i_num];
    if((cfg == CIN_FUNC_NONE) || (cfg == CIN_FUNC_MUTE_LEVEL)) {
        return(CFG_LEVEL);
    } else {
        return(CFG_PULSE);
    }
}

uint32_t Get_CtrlInputCfg(uint32_t i_num)
{
    if(!Is_ValidCtrlIoNumber(i_num)){
        return(CIN_FUNC_NONE);
    }
    return(g_CfgCtrlInput[i_num]);
}

uint32_t Get_ControlInput1(void)
{
	if (digitalRead(GPIO_CONTROL_IN1)) {
        //printf( "GPIO_CONTROL_IN1　Break \n");
    	return (0);	// Break
	}
    //printf( "GPIO_CONTROL_IN1　Make \n");
    return (1);	    // Make
}

uint32_t Get_ControlInput2(void)
{
	if (digitalRead(GPIO_CONTROL_IN2)) {
        //printf( "GPIO_CONTROL_IN2　Break \n");
    	return (0);	// Break
	}
    //printf( "GPIO_CONTROL_IN2　Make \n");
    return (1);	    // Make
}

uint32_t Get_PowerDownDetect(void)
{

	if (digitalRead(GPIO_PWR_DOWN_DET)) {
		return (0);	// Power on
	}
	return (1);	// Power Off
}

uint32_t Get_UsbAudioStatus(void)
{
	if (digitalRead(GPIO_USB_AUDIO_ST)) {
		return (1);	// working
	}
	return (0);	// suspended
}

void Set_ControlOutput1(uint32_t i_ctrl)
{
#ifdef g_Test_Enable	// S1-Test
	printf( "OUT1 : %d\n", i_ctrl );
#endif
    digitalWrite(GPIO_CONTROL_OUT1, i_ctrl);    //0:open, 1:close
}

void Set_ControlOutput2(uint32_t i_ctrl)
{
    digitalWrite(GPIO_CONTROL_OUT2, i_ctrl);    //0:open, 1:close
}

void Set_ControlOutputIndividual(uint32_t i_num, uint32_t i_ctrl)
{
    if(!Is_ValidCtrlIoNumber(i_num)){
        return;
    }
    if(!i_num){
        Set_ControlOutput1(i_ctrl);
    } else {
        Set_ControlOutput2(i_ctrl);
    }
}

void Set_PC_VolumeUp(uint32_t i_ctrl)
{
    digitalWrite(GPIO_PC_VOL_UP, i_ctrl);    //0:no effect, 1:vol up
}

void Set_PC_VolumeDown(uint32_t i_ctrl)
{
    digitalWrite(GPIO_PC_VOL_DOWN, i_ctrl);    //0:no effect, 1:vol down
}

void Set_PC_VolumeMute(uint32_t i_ctrl)
{
    digitalWrite(GPIO_PC_VOL_MUTE, i_ctrl);    //0:no effect, 1:mute
}

void Set_ADDA_Reset(uint32_t i_ctrl)
{
    digitalWrite(GPIO_ADDA_RESET, i_ctrl);    //0:Reset, 1:Normal operation
}

void Set_DSP_Reset(uint32_t i_ctrl)
{
    digitalWrite(GPIO_DSP_RESET, i_ctrl);    //0:Reset, 1:Normal operation
}

void Set_gpioMuteStatus(uint32_t i_status)
{
    g_MuteStatus = i_status;
}

uint32_t Get_gpioMuteStatus(void)
{
    return(g_MuteStatus);   //0:unmute, 1:mute
}

uint32_t Is_ResetSwitchStatusChange(void)
{
    if(gst_inputGpio[INPUT1_RESET_SWITCH].flag){
        gst_inputGpio[INPUT1_RESET_SWITCH].flag = 0;
        return 1;
    }
    return 0;
}

uint32_t Is_ControlInput1StatusChange(void)
{
    if(gst_inputGpio[INPUT2_CONTROL_INPUT1].flag){
        gst_inputGpio[INPUT2_CONTROL_INPUT1].flag = 0;
        return 1;
    }
    return 0;
}

uint32_t Is_ControlInput2StatusChange(void)
{
    if(gst_inputGpio[INPUT3_CONTROL_INPUT2].flag){
        gst_inputGpio[INPUT3_CONTROL_INPUT2].flag = 0;
        return 1;
    }
    return 0;
}

uint32_t Read_ResetSwitchStatus(void)
{
    return gst_inputGpio[INPUT1_RESET_SWITCH].current; // 0:off, 1:on
}

uint32_t Read_ControlInput1Status(void)
{
    return gst_inputGpio[INPUT2_CONTROL_INPUT1].current; // 0:break, 1:make
}

uint32_t Read_ControlInput2Status(void)
{
    return gst_inputGpio[INPUT3_CONTROL_INPUT2].current; // 0:break, 1:make
}

int32_t Get_gpioChangedSwitch(void)
{
    uint32_t i;

    for(i = 0; i < MAX_INPUT_NUMBER; i++){
        if(gst_inputGpio[i].flag){
            gst_inputGpio[i].flag = 0;
            return i;
        }
    }
    return -1;
}

int32_t Set_HeartBeat_OUTPUT(void)
{
    pinMode(GPIO_HEART_BEAT, OUTPUT);              // For Second PCB
	digitalWrite(GPIO_HEART_BEAT, 0);              // For Second PCB

    return 0;
}

int32_t CinScan(void)
{
    uint32_t i;
	uint32_t current[MAX_INPUT_NUMBER];
    current[INPUT2_CONTROL_INPUT1] = Get_ControlInput1();
    current[INPUT3_CONTROL_INPUT2] = Get_ControlInput2();

    for(i = INPUT2_CONTROL_INPUT1; i < MAX_INPUT_NUMBER; i++){
        if(current[i] == gst_inputGpio[i].prev) {
            if(current[i] != gst_inputGpio[i].current) {
                if (MAX_COUNT_CTRL_IN <= gst_inputGpio[i].count++) {
					// For No189
                    gst_inputGpio[i].count = 0;
                    gst_inputGpio[i].current = current[i];
                    gst_inputGpio[i].temp[gst_inputGpio[i].index] = current[i];
                    gst_inputGpio[i].index ^= 1;
                    if (Is_CtrlInputCfgPulse(i-INPUT2_CONTROL_INPUT1) == CFG_LEVEL) {
                        gst_inputGpio[i].flag = 1;
                    } else {
                        // set cin status to buffer 2 times

                        // get cin status 2 times
                        if(!gst_inputGpio[i].index) {
                            //[1,0]:Make, [0,0][0,1][1,1]:N/A
                            if((gst_inputGpio[i].temp[0]) && (!gst_inputGpio[i].temp[1])){
                                gst_inputGpio[i].flag = 1;
                            } else {
                                gst_inputGpio[i].temp[0] = 0;
                                gst_inputGpio[i].temp[1] = 0;
                            }
                        }
                    }
                }
            }
        } else {
            gst_inputGpio[i].count = 0;
        }
        gst_inputGpio[i].prev = current[i];
    }
    return(Get_gpioChangedSwitch());
}

void Update_CtrlOutput(void)
{
    uint32_t i;
    uint32_t mute = Get_gpioMuteStatus();
    uint32_t cin1 = Read_ControlInput1Status();
    uint32_t cin2 = Read_ControlInput2Status();

    for(i = 0; i < MAX_CTRL_IO_NUMBER; i++){
        switch(g_CfgCtrlOutput[i]){
        case COUT_FUNC_MUTE_STATUS: Set_ControlOutputIndividual(i, mute);   break;
        case COUT_FUNC_SYNC_INPUT1: Set_ControlOutputIndividual(i, cin1);   break;
        case COUT_FUNC_SYNC_INPUT2: Set_ControlOutputIndividual(i, cin2);   break;
        case COUT_FUNC_NONE:        Set_ControlOutputIndividual(i, 0);      break; //For No201
        default:                break;
        }
    }
}
