#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <fcntl.h>
#include <math.h>
#include "i2c_control.h"
#include "CtimerLib.h"
//#define  DO_USLEEP_AFTER_I2CGETSET
#ifdef   DO_USLEEP_AFTER_I2CGETSET
#include <time.h>
#define  PERIOD_USLEEP_AFTER_I2CGSETGET (1000)
#endif /*DO_USLEEP_AFTER_I2CGETSET*/

#define MAX_I2C_COMMAND_LEN 100
static char i2c_command[MAX_I2C_COMMAND_LEN];
static void Initialize_SidePanel_LED_AND_SW(void);

#define I2C_BUS 1

//#define g_Test_Enable	// S1-Test

#define POPEN_RESPONSE_BUF_SZ  (100)

/*************************************************/
/* I2C API                                       */
/*************************************************/

/********************************************************************************
*
*  API Function : int i2c_read ( int i2c_addr, int * i2c_data )
*
*
*  "i2c_read" function above reads the register of the I2C device
*  with address "i2c_addr" of the I2C Bus #1
*
*  The data read is returned in the i2c_data pointer provided as a parameter
*  The function returns the error status as a retuned value.
*
*  The i2c_read is perfromed using the call to the external "i2cget" program
*  which is a part of "i2ctools" package which needs to be installed
*  before using the i2c API
*
********************************************************************************/


int32_t i2c_read(uint32_t i2c_addr, uint32_t *i2c_data) {

	// Prepare the I2C TOOLS "get" Command line that
	// targets I2C bus 1 with the adderess provided by caller
	// of the API function
	//
	sprintf(i2c_command, "i2cget -y %d %d", I2C_BUS, i2c_addr);

	FILE* file = popen(i2c_command, "r");
	if(file == NULL){
		// Failed to fork or pipe.
		return -1;
	}
#ifdef   DO_USLEEP_AFTER_I2CGETSET
	usleep(PERIOD_USLEEP_AFTER_I2CGSETGET);
#endif /*DO_USLEEP_AFTER_I2CGETSET*/

	//printf ("%s\n", i2c_command);

	char i2c_response[POPEN_RESPONSE_BUF_SZ];
	char *save_ptr;
	char *end_ptr;
	char *ret_fgets;
	int ret_pclose;

	// Make sure that i2c_response is initially of the null type
	// so that if nothing is returned by the i2c_command, we do not
	// display any garbage

	memset(i2c_response, 0x00, sizeof(i2c_response));

	ret_fgets = fgets(i2c_response, sizeof(i2c_response), file);
	if(ret_fgets == NULL) {
		// Failed to get string.
		return -1;
	}

	i2c_response[sizeof(i2c_response) - 1] = 0x00;

	ret_pclose = pclose(file);
	if(ret_pclose < 0){
		// Failed to get the status of child process(?)
		return -1;
	}

	//printf("i2cget response is :\"%s\"", i2c_response);

	char *i2c_data_string;
	i2c_data_string = strtok_r(i2c_response, "\n", &save_ptr);

	//printf ("returned string value : %s\n", i2c_data_string);

    if(i2c_data_string == NULL){
        return -1;
    }

	*i2c_data = (int)strtol(i2c_data_string, &end_ptr, 16);
	if(end_ptr == i2c_data_string){
		// Failed to convert from string to integer because no 0-9a-f char found.
		return -1;
	}

	//printf ("returned value : %d\n", *i2c_data);

	// Extract the returned i2c read value from the i2c response string
	//
	return 0; // For now, always return OK status
}

int32_t i2c_read_w_reg(uint32_t i2c_addr, uint32_t i2c_reg, uint32_t *i2c_data) {

	// Prepare the I2C TOOLS "get" Command line that
	// targets I2C bus 1 with the adderess provided by caller
	// of the API function
	//
	sprintf(i2c_command, "i2cget -y %d %d %d", I2C_BUS, i2c_addr, i2c_reg);

	FILE* file = popen(i2c_command, "r");
	if(file == NULL){
		// Failed to fork or pipe.
		return -1;
	}
#ifdef   DO_USLEEP_AFTER_I2CGETSET
	usleep(PERIOD_USLEEP_AFTER_I2CGSETGET);
#endif /*DO_USLEEP_AFTER_I2CGETSET*/

	//printf ("%s\n", i2c_command);

	char i2c_response[POPEN_RESPONSE_BUF_SZ];
	char *save_ptr;
	char *end_ptr;
	char *ret_fgets;
	int ret_pclose;

	// Make sure that i2c_response is initially of the null type
	// so that if nothing is returned by the i2c_command, we do not
	// display any garbage

	memset(i2c_response, 0x00, sizeof(i2c_response));

	ret_fgets = fgets(i2c_response, sizeof(i2c_response), file);
	if(ret_fgets == NULL) {
		// Failed to get string.
		return -1;
	}

	i2c_response[sizeof(i2c_response) - 1] = 0x00;

	ret_pclose = pclose(file);
	if(ret_pclose < 0){
		// Failed to get the status of child process(?)
		return -1;
	}

	//printf("i2cget response is :\"%s\"", i2c_response);

	char *i2c_data_string;
	i2c_data_string = strtok_r(i2c_response, "\n", &save_ptr);

	//printf ("returned string value : %s\n", i2c_data_string);

    if(i2c_data_string == NULL){
        return -1;
    }

	*i2c_data = (int)strtol(i2c_data_string, &end_ptr, 16);
	if(end_ptr == i2c_data_string){
		// Failed to convert from string to integer because no 0-9a-f char found.
		return -1;
	}

	//printf ("returned value : %d\n", *i2c_data);

	// Extract the returned i2c read value from the i2c response string
	//
	return 0; // For now, always return OK status
}


/********************************************************************************
*
*  API Function : int i2c_write ( int  i2c_addr, int i2c_data )
*
*
*  "i2c_write" function above wirtes the register of the I2C device
*  with address "i2c_addr" of I2C Bus #1 and returns the status code.
*
*  The i2c_write is perfromed using the call to the external "i2cset" program
*  which is a part of "i2ctools" package which needs to be installed
*  before using the i2c API
*
********************************************************************************/

int32_t i2c_write(uint32_t i2c_addr, uint32_t i2c_data) {

	// Prepare the I2C TOOLS "set" Command line that
	// targets I2C bus 1 with the adderess provided by caller
	// of the API function as well as with the data value to
	// be written
	//
	sprintf(i2c_command, "i2cset -y %d  %d %d", I2C_BUS, i2c_addr, i2c_data);

	FILE* file = popen(i2c_command, "r");
	if(file == NULL){
		// Failed to fork or pipe.
		return -1;
	}
#ifdef   DO_USLEEP_AFTER_I2CGETSET
	usleep(PERIOD_USLEEP_AFTER_I2CGSETGET);
#endif /*DO_USLEEP_AFTER_I2CGETSET*/

	//printf ("%s\n", i2c_command);

	char i2c_response[POPEN_RESPONSE_BUF_SZ];
	int ret_pclose;

	// Make sure that i2c_response is initially of the null type
	// so that if nothing is returned by the i2c_command, we do not
	// display any garbage

	memset(i2c_response, 0x00, sizeof(i2c_response));

	fgets(i2c_response, sizeof(i2c_response), file);
	// Return value can be NULL in this case, so ignore it.

	i2c_response[sizeof(i2c_response) - 1] = 0x00;

	ret_pclose = pclose(file);
	if(ret_pclose < 0){
		// Failed to get the status of child process(?)
		return -1;
	}

	// The expected response should be  null. If it is not,
	// then some error must have been reported
	//
	if (strnlen(i2c_response, sizeof(i2c_response)) != 0) {
		printf("i2cset response is :%s", i2c_response);

		// Return the error value to the caller
		return (-1);
	}

	// In case of success, return OK value
	//
	return 0;
}

int32_t i2c_write_w_reg(uint32_t i2c_addr, uint32_t i2c_reg, uint32_t i2c_data) {

	// Prepare the I2C TOOLS "set" Command line that
	// targets I2C bus 1 with the adderess provided by caller
	// of the API function as well as with the data value to
	// be written
	//
	sprintf(i2c_command, "i2cset -y %d  %d %d %d", I2C_BUS, i2c_addr, i2c_reg, i2c_data);

	FILE* file = popen(i2c_command, "r");
	if(file == NULL){
		// Failed to fork or pipe.
		return -1;
	}
#ifdef   DO_USLEEP_AFTER_I2CGETSET
	usleep(PERIOD_USLEEP_AFTER_I2CGSETGET);
#endif /*DO_USLEEP_AFTER_I2CGETSET*/

	//printf ("%s\n", i2c_command);

	char i2c_response[POPEN_RESPONSE_BUF_SZ];
	int ret_pclose;

	// Make sure that i2c_response is initially of the null type
	// so that if nothing is returned by the i2c_command, we do not
	// display any garbage

	memset(i2c_response, 0x00, sizeof(i2c_response));

	fgets(i2c_response, sizeof(i2c_response), file);
	// Return value can be NULL in this case, so ignore it.

	i2c_response[sizeof(i2c_response) - 1] = 0x00;

	ret_pclose = pclose(file);
	if(ret_pclose < 0){
		// Failed to get the status of child process(?)
		return -1;
	}

	// The expected response should be  null. If it is not,
	// then some error must have been reported
	//
	if (strnlen(i2c_response, sizeof(i2c_response)) != 0) {
		printf("i2cset response is :%s", i2c_response);

		// Return the error value to the caller
		return (-1);
	}

	// In case of success, return OK value
	//
	return 0;
}

void Initialize_PCF8574A(void){
	//PCF8574A  must write hi on the input pin

	i2c_write(I2C_CHIP_ADDRESS_SIDE_PANEL,	0xff);	//bitO-3 Input  BT_SW(0), MUTE_SW(1), VOL_UP_SW(2), VOL _DOWN_SW(3),
															//bit4-7 Output POWER_GREEN_LED(4), POWER_RED_LED(5), USB_GREEN_LED(6),BT_BLUE_LED(7)

	i2c_write(I2C_CHIP_ADDRESS_SIDE_PANEL_2,	0xbf);	//bit0   Input  POWER_SW(0)
															//bit1-7 Output POWER_BLUE_LED(1),USB_BLUE_LED(2),USB_GREEN_LED(3),BT_RED_LED(4),BT_GREEN_LED(5)
															//		 		POWER_BLUE_LED_ENABLE(6) はまだ点灯状態で  NC(7)

  i2c_write(I2C_CHIP_ADDRESS_MB,			0xff);	//bit0-5 Input  CIN1(0),CIN2(1)
															//	 		    AMP_STATUS(2),AC_ADAPTER_STATUS(3),POE_A_STATUS(4),POE_B_STATUS(5),
}															//bit6-7 Output USB_POWER(6),OUTPUT_MUTE_AND_AMP_OFF(7)

st_input g_st_inputI2C[1];
uint32_t g_MuteStatus;

uint32_t g_StandbyMode;			//POWERSW
uint32_t g_ShutdownMode;		//POWERSW
uint32_t g_CountOffset;			//POWERSW
uint32_t g_CountShutdown;		//POWERSW
uint32_t g_CountStopShutdown;	//POWERSW

#define INPUT0_POWER_SWITCH 0
#define MIN_COUNT_PWSW      40
#define MAX_COUNT_PWSW      80
#define MAX_COUNT_SHUTDOWN  200

//MAX_COUNT_SWITCH	10 // 0.63s
//					20 // 1.31s
//					30 // 2.31s
//					40 // 3.84s



uint32_t Get_PowerSwitch(void);


void Initialize_PowerSW(void){
	g_st_inputI2C[INPUT0_POWER_SWITCH].prev = Get_PowerSwitch();								//Second PCBからI2C制御への移行を検討すること
    g_st_inputI2C[INPUT0_POWER_SWITCH].current = g_st_inputI2C[INPUT0_POWER_SWITCH].prev;			//Second PCBからI2C制御への移行を検討すること

    g_st_inputI2C[INPUT0_POWER_SWITCH].flag = 0; //Power SW が変化していない

    g_CountOffset = 0;			//POWERSW
    g_ShutdownMode = 0;			//POWERSW
    g_CountShutdown = 0;		//POWERSW
    g_StandbyMode = 0;			//POWERSW
    g_CountStopShutdown = 0;	//POWERSW
}




/*************************************************/
/* Initialize I2C Devices                        */
/*************************************************/
void Initialize_I2C(void)
{
    Initialize_DAC();
    Initialize_USBHub();

    Initialize_PCF8574A();

    Initialize_SidePanel_LED_AND_SW();
    Initialize_FrontPanel();
    Set_TrickleChargerEnable();

    Initialize_MB();

    Initialize_PowerSW();
}

/*************************************************/
/* DAC I/F                                       */
/*************************************************/
void Initialize_DAC(void)
{
	i2c_write_w_reg(I2C_CHIP_ADDRESS_DAC, REG_CONTROL01, 0x0f);	// I2S 32bit compatible
	i2c_write_w_reg(I2C_CHIP_ADDRESS_DAC, REG_CONTROL01, 0x0e);	// OSC Reset
	i2c_write_w_reg(I2C_CHIP_ADDRESS_DAC, REG_CONTROL01, 0x0f);	// OSC Normal
	i2c_write_w_reg(I2C_CHIP_ADDRESS_DAC, REG_CONTROL02, 0x22);	// Normal Speed
	i2c_write_w_reg(I2C_CHIP_ADDRESS_DAC, REG_CONTROL06, 0x4d);	// TDM128
}

void Reset_OSC(void)
{
	i2c_write_w_reg(I2C_CHIP_ADDRESS_DAC, REG_CONTROL01, 0x0e);	// OSC Reset
}

void Ctrl_Smute(uint32_t i_ctrl)
{
    if(i_ctrl){
        i2c_write_w_reg(I2C_CHIP_ADDRESS_DAC, REG_CONTROL02, 0x22|MASK_SMUTE_ENABLE);
    } else {
        i2c_write_w_reg(I2C_CHIP_ADDRESS_DAC, REG_CONTROL02, 0x22);
    }
}

/*************************************************/
/* USB HUB                                       */
/*************************************************/
void Initialize_USBHub(void)
{
	uint32_t i;
	char product_str[] = "A.M.-.C.F.1.";                        //UTF-16LE
	char manufacture_str[] = "T.O.A. .C.o.r.p.o.r.a.t.i.o.n.";  //UTF-16LE

	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_DEVICE_CONFIG1, 0x90);		// Enable custom Vendor/ProductID
	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_VENDOR_ID_LSB, 0x04);			// TOA VendorID 0x1104
	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_VENDOR_ID_MSB, 0x11);			// TOA VendorID 0x1104
	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_PRODUCT_ID_LSB, 0x00);		// TOA ProductID 0x000~FFF
	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_PRODUCT_ID_MSB, 0xf0);		// F:Prototype, E:SpecialOrder
	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_DEVICE_REMOVE_CONFIG, 0x8f);	// All Port removable
    i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_LANG_ID_LSB, 0x09);           // Lang ID LSB
    i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_LANG_ID_MSB, 0x04);           // Lang ID MSB

	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_MANUFACTURE_STR_LEN, MAX_MANUFACTURE_STR_LENGTH);	// Length 30 byte
	for(i = 0; i < MAX_MANUFACTURE_STR_LENGTH; i++){
        if(i%2){
    		i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_MANUFACTURE_STR_BYTE+i, 0);
        } else {
    		i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_MANUFACTURE_STR_BYTE+i, (uint32_t)manufacture_str[i]);
        }
	}

	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_PRODUCT_STR_LEN, MAX_PRODUCT_STRINGS_LENGTH);		// Length 12 byte
	for(i = 0; i < MAX_PRODUCT_STRINGS_LENGTH; i++){
        if(i%2){
		    i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_PRODUCT_STR_BYTE+i, 0);
        } else {
		    i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_PRODUCT_STR_BYTE+i, (uint32_t)product_str[i]);
        }
	}

	i2c_write_w_reg(I2C_CHIP_ADDRESS_USB_HUB, REG_DEVICE_STATUS_COMMAND, 0x01);	// cfg active
}

/*************************************************/
/* RTC                                           */
/*************************************************/
uint32_t Int_to_BCD(uint32_t i_num)
{
    return(i_num + (i_num>>4)*6);
}

uint32_t BCD_to_Int(uint32_t i_num)
{
    if(i_num < 10) {
        return i_num;
    }
    return(i_num - (i_num>>4)*6);
}

void Initialize_RTC(
    int32_t i_year,
    int32_t i_month,
    int32_t i_date,
    int32_t i_day,
    int32_t i_hour,
    int32_t i_min,
    int32_t i_sec)
{
    uint32_t year, month, date, day, hour, min, sec;
    int32_t century = 0;

    if((i_year < 0) || (99 < i_year)){
	    year = 0;
    } else {
        year = (i_year/10<<4) + (i_year%10);
    }
    // judgement leap year
    if((i_year % 4) == 0){
        century = 0x80;
    }

    if((i_month < 1) || (12 < i_month)){
        month = 1;
    } else if (9 < i_month){
        month = (i_month/10<<4) + (i_month%10);
    } else {
        month = i_month;
    }

    if(i_date < 1) {
        i_date = 1;
    }
    if(i_month == 2){
        if(28 < i_date){
            if(century) {
                i_date = 29;
            } else {
                i_date = 28;
            }
        }
    } else if((i_month == 4) || (i_month == 6) || (i_month == 9) || (i_month == 11)){
        if(30 < i_date){
            i_date = 30;
        }
    } else {
        if(31 < i_date){
            i_date = 31;
        }
    }
    date = (i_date/10<<4) + (i_date%10);

    // Spec RTC : 1:Sun, 2:Mon, ... , 7:Sat
    if((i_day < 1) || (7 < i_day)){
        day = 1;
    } else {
        day = i_day;
    }

    // 24hours mode
    if((i_hour <= 0) || (23 < i_hour)){
        hour = 0;
    } else if ((9 < i_hour) && (i_hour < 20)){
        hour = (i_hour - 10) | 0x10;
    } else if ((19 < i_hour) && (i_hour < 24)) {
        hour = (i_hour - 20) | 0x20;
    } else {
        hour = i_hour;
    }

    if((i_min <= 0) || (59 < i_min)){
        min = 0;
    } else {
        min = (i_min/10<<4) + (i_min%10);
    }

    if((i_sec <= 0) || (59 < i_sec)){
        sec = 0;
    } else {
        sec = (i_sec/10<<4) + (i_sec%10);
    }

#ifdef g_Test_Enable	// S1-Test
	printf("YY:0x%02X MM:0x%02X DD:0x%02X DAY:0x%02X HH:0x%02X MM:0x%02X SS:0x%02X\n",
		   year, month, date, day, hour, min, sec);
#endif
	i2c_write_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_YEAR, year);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_MONTH_CENTURTY, month);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_DATE, date);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_DAY, day);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_HOURS, hour);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_MINUTES, min);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_SECONDS, sec);
}

void Set_TrickleChargerEnable(void)
{
    i2c_write_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_TRICKLE_CHARGER, 0xaa); // 1diode,2kohm
}

uint32_t Get_Year(void)
{
    uint32_t year = 0;
    int32_t ret = i2c_read_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_YEAR, &year);

    if(ret != 0){
        return 0;
    }
    return (BCD_to_Int(year));
}

uint32_t Get_Month(void)
{
    uint32_t month = 1;
    int32_t ret = i2c_read_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_MONTH_CENTURTY, &month);

    if(ret != 0){
        return 1;
    }
    return (BCD_to_Int(month));
}

uint32_t Get_Date(void)
{
    uint32_t date = 1;
    int32_t ret = i2c_read_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_DATE, &date);

    if(ret != 0){
        return 1;
    }
    return (BCD_to_Int(date));
}

uint32_t Get_Day(void)
{
    uint32_t day = 1;
    int32_t ret = i2c_read_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_DAY, &day);

    if(ret != 0){
        return 1;
    }
    return (day);
}

uint32_t Get_Hour(void)
{
    uint32_t hour = 0;
    int32_t ret = i2c_read_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_HOURS, &hour);

    if(ret != 0){
        return 0;
    }
    return (BCD_to_Int(hour));
}

uint32_t Get_Minute(void)
{
    uint32_t minute = 0;
    int32_t ret = i2c_read_w_reg(I2C_CHIP_ADDRESS_RTC, REG_RTC_MINUTES, &minute);

    if(ret != 0){
        return 0;
    }
    return (BCD_to_Int(minute));
}

/*************************************************/
/* Mother Board I/F                                */
/*************************************************/
uint32_t 		g_MotherBoard_Value;	//For Second PCB

void Initialize_MB(void){
	g_MotherBoard_Value = 0xff;

	Ctrl_USB_POWER(POWER_FROM_PC);
	Ctrl_AUDIO_OUTPUT_ENABLE(AUDIO_OUTPUT_DISENABLE);		// For Second PCB

}

void Ctrl_USB_POWER(uint32_t i_state){
	uint32_t value;

	value = g_MotherBoard_Value & 0xbf;	// x*xx xxxx  set bit6

	switch(i_state){

	case(POWER_FROM_PC):
		value += 0x40;
		break;
	case(POWER_FROM_CF1):
		value += 0x00;
		break;
	default:
		value += 0x40;
		break;
	}

	g_MotherBoard_Value = value | 0x3f; //入力ピンには1を書く
	i2c_write(I2C_CHIP_ADDRESS_MB, g_MotherBoard_Value);

	//printf ("Ctrl_USB_POWER %x\n", g_MotherBoard_Value);
}

void Ctrl_AUDIO_OUTPUT_ENABLE(uint32_t i_state){	//For Second PCB
	uint32_t value;

	value = g_MotherBoard_Value & 0x7f;	// *xxx xxx		set bit7

	switch(i_state){

	case(AUDIO_OUTPUT_ENABLE):
		value += 0x00;
		break;
	case(AUDIO_OUTPUT_DISENABLE):
		value += 0x80;
		break;
	default:
		value += 0x80;
		break;
	}

	g_MotherBoard_Value = value | 0x3f; //入力ピンには1を書く
	i2c_write(I2C_CHIP_ADDRESS_MB, g_MotherBoard_Value);

	//printf ("Ctrl_AUDIO_OUTPUT_ENABLE %x\n", g_MotherBoard_Value);

}

/*************************************************/
/* Side Panel I/F                                */
/*************************************************/
uint32_t 		g_MuteCurrentStatus;
uint32_t 		g_CountBackToNormalIndMode;
uint32_t 		g_SidePanelLED_Value;

uint32_t 		g_SidePanelLED_2_Value;	//For Second PCB


st_SwitchState	g_st_SwitchState[MAX_SW_NUMBER];


void Initialize_SidePanel_LED_AND_SW(void)
{
    g_SidePanelLED_Value 	= 0xff;	// 1 = LED OFF
	g_SidePanelLED_2_Value	= 0xff;	// For Second PCB

	Ctrl_PowerLED(LED_ON_GREEN);

	Ctrl_BluetoothLED(LED_OFF);

	Ctrl_USBConnectionLED(LED_OFF);

    Initialize_SwitchStatus();
}

void Initialize_SwitchStatus(void)
{
	uint32_t i;

    g_MuteCurrentStatus = 0;

    g_CountBackToNormalIndMode = 0;

    for(i=0; i<MAX_SW_NUMBER; i++){

        g_st_SwitchState[i].current	= 0x0f;
        g_st_SwitchState[i].count 	= 0;
        g_st_SwitchState[i].flag 	= 0;

    }
}

void Ctrl_PowerLED(uint32_t i_state) {

	uint32_t SidePanelCtrl = g_SidePanelLED_Value & 0xcf;	// xx** xxxx bit4 POWER GREEN LED, bit5 POWER RED LED

	switch (i_state) {
	case LED_OFF:				SidePanelCtrl += 0x30;	break;	// 0 = ON  1 =OFF
	case LED_ON_GREEN:	SidePanelCtrl += 0x20;	break;	// 1
	case LED_ON_RED:		SidePanelCtrl += 0x10;	break;	// 2
  case LED_ON_ORANGE:	SidePanelCtrl += 0x00;	break;	// 3
	default: break;
	}

	g_SidePanelLED_Value = SidePanelCtrl | 0x0f;

	i2c_write(I2C_CHIP_ADDRESS_SIDE_PANEL, g_SidePanelLED_Value);
}

void Ctrl_BluetoothLED(uint32_t i_state)
{
	uint32_t value;

	value = g_SidePanelLED_Value & 0x7f;	// *xxx xxxx  set bit7

	switch(i_state){

	case(LED_OFF):
		value += 0x80;
		break;
	case(LED_ON_BLUE):
		value += 0x00;
		break;
	default:
		value += 0x00;
		break;
	}

	g_SidePanelLED_Value = value | 0x0f; //入力ピンには1を書く
	i2c_write(I2C_CHIP_ADDRESS_SIDE_PANEL, g_SidePanelLED_Value);
}

void Ctrl_USBConnectionLED(uint32_t i_state)
{
	// For Second PCB
	uint32_t value;

	value = g_SidePanelLED_2_Value & 0xf3;	// xxxx **xx  set bit2 bit3

	//printf ("Ctrl_USBConnectionLED value0 %x\n", value);
	switch(i_state){
	case(LED_OFF):
		//value += 0x08;
		value += 0x0c;
		break;
	case(LED_ON_GREEN):
		//value += 0x00;
		value += 0x04;
		break;
	case(2):
		value += 0x08;
		break;

	default:
		value += 0x0c;
		break;
	}

	//printf ("Ctrl_USBConnectionLED i_state %x\n", i_state);
	//printf ("Ctrl_USBConnectionLED value %x\n", value);

	g_SidePanelLED_2_Value = value | 0x01; //入力ピンには1を書く
	i2c_write(I2C_CHIP_ADDRESS_SIDE_PANEL_2, g_SidePanelLED_2_Value);

	//printf ("Ctrl_USBConnectionLED g_SidePanelLED_2_Value %x\n", g_SidePanelLED_2_Value);
}

void Ctrl_POWER_Blue_LED_SOFT(uint32_t i_state)	//For Second PCB
{
	uint32_t value;

	value = g_SidePanelLED_2_Value & 0xfd;	// xxxx xx*x  set bit1

	switch(i_state){

	case(LED_OFF):
		value += 0x02;
		break;
	case(LED_ON_BLUE):
		value += 0x00;
		break;
	default:
		value += 0x02;
		break;
	}

	//printf ("Ctrl_POWER_Blue_LED_SOFT i_state %x\n", i_state);
	//printf ("Ctrl_POWER_Blue_LED_SOFT value %x\n", value);

	g_SidePanelLED_2_Value = value | 0x01; //入力ピンには1を書く
	i2c_write(I2C_CHIP_ADDRESS_SIDE_PANEL_2, g_SidePanelLED_2_Value);

	//printf ("Ctrl_POWER_Blue_LED_SOFT g_SidePanelLED_2_Value %x\n", g_SidePanelLED_2_Value);
}

void Ctrl_POWER_Blue_LED_HARD(uint32_t i_state)	//For Second PCB
{
	uint32_t value;

	value = g_SidePanelLED_2_Value & 0xbf;	// x*xx xxxx  set bit6

	switch(i_state){

	case(LED_OFF):
		value += 0x00;
		break;
	case(LED_ON_BLUE):
		value += 0x40;
		break;
	default:
		value += 0x40;
		break;
	}

	//printf ("Ctrl_POWER_Blue_LED_HARD i_state %x\n", i_state);
	//printf ("Ctrl_POWER_Blue_LED_HARD value %x\n", value);

	g_SidePanelLED_2_Value = value | 0x01; //入力ピンには1を書く
	i2c_write(I2C_CHIP_ADDRESS_SIDE_PANEL_2, g_SidePanelLED_2_Value);

	//printf ("Ctrl_POWER_Blue_LED_HARD g_SidePanelLED_2_Value %x\n", g_SidePanelLED_2_Value);
}


uint32_t Get_PowerSwitch(void){	//For Second PCB
	uint32_t 	data;
    int32_t 	ret = i2c_read(I2C_CHIP_ADDRESS_SIDE_PANEL_2, &data);

    if (ret != 0) {
        return 0;
    }

    data = (data << 8);

	if (data & MASK_POWER_SWITCH) {
		//printf ("Get_PowerSwitch Power Switch:OFF\n");
		return (0);	// Power Switch:OFF
	}

	//printf ("Get_PowerSwitch Power Switch:ON\n");
	return (1);		// Power Switch:ON
}

uint32_t Get_BluetoothSwitch(void)
{
	uint32_t 	data;
    int32_t 	ret = i2c_read(I2C_CHIP_ADDRESS_SIDE_PANEL, &data);

    if (ret != 0) {
        return 0;
    }
	if (data & MASK_BLUETOOTH_SWITCH) {
		return (0);	// Bluetooth Switch:OFF
	}
	return (1);		// Bluetooth Switch:ON
}

#if 0
uint32_t Get_VolumeUpSwitch(void)
{
	uint32_t 	data;
    int32_t 	ret = i2c_read(I2C_CHIP_ADDRESS_SIDE_PANEL, &data);

    if (ret != 0) {
        return 0;
    }
	if (data & MASK_VOLUMEUP_SWITCH) {
		return (0);	// Vol up Switch:OFF
	}
	return (1);		// Vol up Switch:ON
}
#endif

#if 0
uint32_t Get_VolumeDownSwitch(void)
{
	uint32_t 	data;
    int32_t 	ret = i2c_read(I2C_CHIP_ADDRESS_SIDE_PANEL, &data);

    if (ret != 0) {
        return 0;
    }
	if (data & MASK_VOLUMEDOWN_SWITCH) {
		return (0);	// Vol down Switch:OFF
	}
	return (1);		// Vol down Switch:ON
}
#endif

#if 0
uint32_t Get_MuteSwitch(void)
{
	uint32_t 	data;
    int32_t 	ret = i2c_read(I2C_CHIP_ADDRESS_SIDE_PANEL, &data);

    if (ret != 0) {
        return 0;
    }
	if (data & MASK_MUTE_SWITCH) {
		return (0);	// Mute Switch:OFF
	}
	return (1);		// Mute Switch:ON
}
#endif

void Set_i2cMuteStatus(uint32_t i_status)
{
    g_MuteCurrentStatus = i_status;

#if 0
    if(i_status) {
        g_CountOffset = 8 * MIN_COUNT_PWSW;   // At mute, 1loop takes 10ms
    } else {
        g_CountOffset = 0;
    }

    g_CountShutdown = 0;							//POWERSW
    g_st_inputI2C[INPUT0_POWER_SWITCH].count = 0;		//POWERSW
#endif
}

uint32_t Get_i2cMuteStatus(void)
{
    return g_MuteCurrentStatus;
}

int32_t Exec_KeyScan(void)
{
    int32_t ret;

    ret = KeyScan();

    switch(ret){
        case SW_BT:
	    // ret = 0 を返す
            break;

	    case SW_MUTE:
	    // ret = 1 を返す
            /*
            if(Read_MuteSwitch()){
                g_MuteCurrentStatus ^= 1;
                if(g_MuteCurrentStatus){
                    Chg_IndicateMode(MODE_IND_MUTE);
                } else {
                    Chg_IndicateMode(MODE_IND_NORMAL);
                }
            }
            */
            break;

	    case SW_V_UP:
            if(Read_VolumeUpSwitch()){
                g_CountBackToNormalIndMode = 0;
                Chg_IndicateMode(MODE_IND_VOLUME);
            }
            break;

	    case SW_V_DN:
            if(Read_VolumeDownSwitch()){
                g_CountBackToNormalIndMode = 0;
                Chg_IndicateMode(MODE_IND_VOLUME);
            }
            break;

        default:

        	//複数押しの場合はここにくる

        	//この下はだめでしょ

            ret = -1;
            if(Get_IndicateMode() == MODE_IND_VOLUME) {
                if(50 < g_CountBackToNormalIndMode++) {
                    g_CountBackToNormalIndMode = 0;
                    if(Get_i2cMuteStatus()){
                        Chg_IndicateMode(MODE_IND_MUTE);
                    } else {
                        Chg_IndicateMode(MODE_IND_NORMAL);
                    }
                }
            }
            break;
    }
    return ret;
}

int32_t KeyScan(void)	//変化したスイッチ番号を返す
{
	uint32_t i, current_1, current_2,current_total;
    st_SwitchState  st_temp[MAX_SW_NUMBER];
    int32_t ret;

    ret = i2c_read(I2C_CHIP_ADDRESS_SIDE_PANEL, &current_1);
    if (ret != 0) {
        return -1;
    }

    //For Second PCB
    ret = i2c_read(I2C_CHIP_ADDRESS_SIDE_PANEL_2, &current_2);
    if (ret != 0) {
        return -1;
    }

	current_1 &= 0x0f;	//入力ピンのみ選択
	current_2 &= 0x01;	//入力ピンのみ選択
	current_total = (current_2 << 8) + (current_1);

	//printf ("KeyScan %x %x %x\n", current_1, current_2, current_total);

	st_temp[SW_BT].current     = current_total & MASK_BLUETOOTH_SWITCH;	//For Second PCB
    st_temp[SW_MUTE].current   = current_total & MASK_MUTE_SWITCH;
    st_temp[SW_V_UP].current   = current_total & MASK_VOLUMEUP_SWITCH;
    st_temp[SW_V_DN].current   = current_total & MASK_VOLUMEDOWN_SWITCH;
    st_temp[SW_POWER].current   = current_total & MASK_POWER_SWITCH;

    for(i=0; i<MAX_SW_NUMBER; i++){
        if(!st_temp[i].current){
            if(g_st_SwitchState[i].count++ <= MAX_COUNT_SW_STATE) {
                g_st_SwitchState[i].count = 0xff;
                g_st_SwitchState[i].flag = 1;
            }
        } else {
            g_st_SwitchState[i].count = 0;
        }
        g_st_SwitchState[i].current = st_temp[i].current;

        #if 0
        if(i == 3){
			printf ("g_st_SwitchState[%d].current %x \n", i, g_st_SwitchState[i].current);
		}
        if(i == 4){
			printf ("g_st_SwitchState[%d].current %x \n", i, g_st_SwitchState[i].current);
		}
        #endif

    }
    return (Get_i2cChangedSwitch());
}

int32_t Get_i2cChangedSwitch(void)
{
	uint32_t i;

    for(i=0; i<MAX_SW_NUMBER; i++){
        if(g_st_SwitchState[i].flag){
            g_st_SwitchState[i].flag = 0;
            return i;
        }
    }
    return -1;
}

uint32_t Read_BluetoothSwitch(void)
{
	if (g_st_SwitchState[SW_BT].current & MASK_BLUETOOTH_SWITCH) {
		return (0);	// Bluetooth Switch:OFF
	}
	return (1);		// Bluetooth Switch:ON
}

uint32_t Read_VolumeUpSwitch(void)
{
	if (g_st_SwitchState[SW_V_UP].current & MASK_VOLUMEUP_SWITCH) {
		return (0);	// Vol up Switch:OFF
	}
	return (1);		// Vol up Switch:ON
}

uint32_t Read_VolumeDownSwitch(void)
{
	if (g_st_SwitchState[SW_V_DN].current & MASK_VOLUMEDOWN_SWITCH) {
		return (0);	// Vol down Switch:OFF
	}
	return (1);		// Vol down Switch:ON
}

uint32_t Read_MuteSwitch(void)
{
	if (g_st_SwitchState[SW_MUTE].current & MASK_MUTE_SWITCH) {
		return (0);	// Mute Switch:OFF
	}
	return (1);		// Mute Switch:ON
}

uint32_t Read_PowerSwitch(void)
{
	if (g_st_SwitchState[SW_POWER].current & MASK_POWER_SWITCH) {
		//printf ("Read_PowerSwitch Power Switch:OFF\n");
		return (0);	// Power Switch:OFF
	}
	//printf ("Read_PowerSwitch Power Switch:ON\n");
	return (1);		// Power Switch:ON
}


#if 0
uint32_t Debug_GetCount(void)
{
    return(g_st_SwitchState[SW_MUTE].count);
}
#endif

/*************************************************/
/* Front Panel I/F                               */
/*************************************************/
uint32_t g_IndicateMode = MODE_IND_NORMAL;
uint32_t g_flag_mute_mode = 0;
uint32_t g_VolumePosition = POS_VOL_4;
uint32_t g_LedControl[MAX_LED_BANK_NUMBER];
uint32_t g_modeChange_except_Normal = 0;
uint32_t g_modeChange_to_Normal = 0;
uint32_t g_cfg_tracking = CFG_TRACKING_ON;
uint32_t g_cfg_level_indicate = CFG_LEVEL_INDICATE_ON;
uint32_t g_cfg_mute_led_pattern = CFG_MUTE_INDICATE_ALL;
st_LedState	g_st_LedState[MAX_LED_ID_NUMBER];

void Initialize_FrontPanelVariable(void)
{
    g_IndicateMode = MODE_IND_NORMAL;
    g_flag_mute_mode = 0;
    g_VolumePosition = POS_VOL_4;
    g_modeChange_except_Normal = 0;
    g_modeChange_to_Normal = 0;
    g_cfg_tracking = CFG_TRACKING_ON;
    g_cfg_level_indicate = CFG_LEVEL_INDICATE_ON;
    g_cfg_mute_led_pattern = CFG_MUTE_INDICATE_ALL;
}

void Initialize_FrontPanel(void)
{
	uint32_t i;

	for (i = 0; i<MAX_LED_ID_NUMBER; i++) {
		g_st_LedState[i].state = LED_STATE_OFF;
		g_st_LedState[i].flag  = 0;
		g_st_LedState[i].level_current = 0;
        g_st_LedState[i].level_target  = 0;
        g_st_LedState[i].volume_disp = 0;
        g_st_LedState[i].f_level_current = -70.0;
        g_st_LedState[i].f_level_target = -70.0;
        g_st_LedState[i].i_level = 0.0;
	}

	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_MODE1, 0x01);	// OSC Normal Mode
	for (i = 0; i<MAX_LED_ID_NUMBER; i++) {
        i2c_write_w_reg(
            I2C_CHIP_ADDRESS_FRONT_PANEL,
            REG_BRIGHTNESS_CTRL_LED0 + 2*i,
            g_st_LedState[LED_NUM_0+i].level_current
        );
    }

	Chg_AllLEDState(LED_STATE_BRIGHTNESS);
    Initialize_SmoothingParameter();
    Initialize_FrontPanelVariable();
    i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_ALL_CALL_ADDRESS, 0xdc);  // avoid RTC adr duplicate
}

void Initialize_LedConfig(uint32_t i_cfg1, uint32_t i_cfg2, uint32_t i_cfg3)
{
    if((1 < i_cfg1) || (1 < i_cfg2) || (1 < i_cfg3)) {
        return;
    }
    g_cfg_tracking = i_cfg1;
    g_cfg_level_indicate = i_cfg2;
    g_cfg_mute_led_pattern = i_cfg3;
}

uint32_t Get_CfgTracking(void)
{
    return(g_cfg_tracking);
}

uint32_t Get_CfgLevelIndicate(void)
{
    return(g_cfg_level_indicate);
}

uint32_t Get_CfgMuteLedPattern(void)
{
    return(g_cfg_mute_led_pattern);
}

uint32_t Is_MuteModeChange(void)
{
    if(g_flag_mute_mode) {
        g_flag_mute_mode = 0;
        return 1;
    }
    return 0;
}

uint32_t Is_VolMode(void)
{
    if(g_modeChange_except_Normal) {
        g_modeChange_except_Normal = 0;
        return 1;
    }
    return 0;
}

uint32_t Is_NormalMode(void)
{
    if(g_modeChange_to_Normal) {
        g_modeChange_to_Normal = 0;
        return 1;
    }
    return 0;
}

void Chg_IndicateMode(uint32_t i_mode)
{
    uint32_t i;

    if(MODE_IND_NONE <= i_mode) {
	    return;
    }
    if(i_mode != g_IndicateMode){
        if((i_mode == MODE_IND_MUTE) || (g_IndicateMode == MODE_IND_MUTE)){
            g_flag_mute_mode = 1;
        }
        if(i_mode == MODE_IND_VOLUME){
            g_modeChange_except_Normal = 1;
        } else {
            g_modeChange_to_Normal = 1;
        }
    }
	if(i_mode == MODE_IND_NORMAL){
		for (i = 0; i < MAX_LED_ID_NUMBER; i++) {
			g_st_LedState[LED_NUM_0+i].level_current = 0;
			i2c_write_w_reg(
                I2C_CHIP_ADDRESS_FRONT_PANEL,
                REG_BRIGHTNESS_CTRL_LED0 + 2*i,
                g_st_LedState[LED_NUM_0+i].level_current
            );
		}
	}
    if(i_mode == MODE_IND_STANDBY){
        Chg_AllLEDState(LED_STATE_BRIGHTNESS);
    }
    g_IndicateMode = i_mode;
}

uint32_t Get_IndicateMode(void)
{
    return g_IndicateMode;
}

void Chg_VolumePosition(uint32_t i_pos)
{
    if(POS_VOL_MAX <= i_pos) {
        return;
    }
    g_VolumePosition = i_pos;
}

uint32_t Get_VolumePosition(void)
{
    return g_VolumePosition;
}
const int32_t VolPosition_GainTable[8] = {-24,-18,-12,-9,-6,-3,0,1};

void Calc_VolumePosition(int32_t i_gain)
{
    uint32_t position_temp,i;

    for(i=0;i<sizeof(VolPosition_GainTable);i++){
	    if(i_gain < VolPosition_GainTable[i]){
	    	position_temp = i;
	    	break;
	    }
	}
	g_VolumePosition = position_temp;
}

uint32_t Is_LEDNumberValid(uint32_t i_ledno)
{
	if (MAX_LED_ID_NUMBER <= i_ledno) {
		return 0;
	}
	return 1;
}

uint32_t Is_LEDStatusChange(uint32_t i_ledno)
{
	if (!Is_LEDNumberValid(i_ledno)) {
		return 0;
	}
	if (g_st_LedState[i_ledno].flag) {
		g_st_LedState[i_ledno].flag = 0;
		return 1;
	}
	return 0;
}

void Chg_IndividualLEDState(
    uint32_t i_led0st,
    uint32_t i_led1st,
    uint32_t i_led2st,
    uint32_t i_led3st,
    uint32_t i_led4st,
    uint32_t i_led5st,
    uint32_t i_led6st)
{
    uint32_t i;
    uint32_t LedBank[MAX_LED_BANK_NUMBER];

	g_st_LedState[LED_NUM_0].state = i_led0st;
    g_st_LedState[LED_NUM_1].state = i_led1st;
    g_st_LedState[LED_NUM_2].state = i_led2st;
	g_st_LedState[LED_NUM_3].state = i_led3st;
    g_st_LedState[LED_NUM_4].state = i_led4st;
    g_st_LedState[LED_NUM_5].state = i_led5st;
	g_st_LedState[LED_NUM_6].state = i_led6st;

    LedBank[LED_BANK_0] = (g_st_LedState[LED_NUM_1].state << 4) | (g_st_LedState[LED_NUM_0].state);
    LedBank[LED_BANK_1] = (g_st_LedState[LED_NUM_3].state << 4) | (g_st_LedState[LED_NUM_2].state);
    LedBank[LED_BANK_2] = (g_st_LedState[LED_NUM_5].state << 4) | (g_st_LedState[LED_NUM_4].state);
    LedBank[LED_BANK_3] = (g_st_LedState[LED_NUM_6].state);

    for(i=0; i<MAX_LED_BANK_NUMBER; i++){
        g_LedControl[i] = LedBank[i];
    }
	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_LED0_1_OUTPUT_CTRL, g_LedControl[LED_BANK_0]);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_LED2_3_OUTPUT_CTRL, g_LedControl[LED_BANK_1]);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_LED4_5_OUTPUT_CTRL, g_LedControl[LED_BANK_2]);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_LED6_OUTPUT_CTRL,   g_LedControl[LED_BANK_3]);
}

void Chg_AllLEDState(uint32_t i_ledst)
{
    uint32_t i;
    uint32_t LedBank[MAX_LED_BANK_NUMBER];

    for(i=0; i<MAX_LED_ID_NUMBER; i++){
    	g_st_LedState[i].state = i_ledst;
    }
    LedBank[LED_BANK_0] = (g_st_LedState[LED_NUM_1].state << 4) | (g_st_LedState[LED_NUM_0].state);
    LedBank[LED_BANK_1] = (g_st_LedState[LED_NUM_3].state << 4) | (g_st_LedState[LED_NUM_2].state);
    LedBank[LED_BANK_2] = (g_st_LedState[LED_NUM_5].state << 4) | (g_st_LedState[LED_NUM_4].state);
    LedBank[LED_BANK_3] = (g_st_LedState[LED_NUM_6].state);

    for(i=0; i<MAX_LED_BANK_NUMBER; i++){
        g_LedControl[i] = LedBank[i];
    }
	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_LED0_1_OUTPUT_CTRL, g_LedControl[LED_BANK_0]);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_LED2_3_OUTPUT_CTRL, g_LedControl[LED_BANK_1]);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_LED4_5_OUTPUT_CTRL, g_LedControl[LED_BANK_2]);
	i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_LED6_OUTPUT_CTRL,   g_LedControl[LED_BANK_3]);
}

void Set_FrontLEDStatus(uint32_t i_ledno, uint32_t i_state, float i_level)
{
	st_LedState st_temp;

	if (!Is_LEDNumberValid(i_ledno)) {
		return;
	}

	st_temp.state = g_st_LedState[i_ledno].state;
	st_temp.level_current = g_st_LedState[i_ledno].level_current;
	g_st_LedState[i_ledno].state = i_state;

	Chg_LEDBrightness(i_ledno, i_level);

	if (st_temp.state != i_state) {
		g_st_LedState[i_ledno].flag = 1;
	}
	if (st_temp.level_current != g_st_LedState[i_ledno].level_current) {
		g_st_LedState[i_ledno].flag = 1;
	}
}

void Ctrl_FrontLEDforMute(uint32_t i_mode)
{
    if(i_mode == MODE_IND_MUTE) {
        if(Get_CfgMuteLedPattern() == CFG_MUTE_INDICATE_ALL) {
            Chg_AllLEDState(LED_STATE_BRIGHTNESS_RED);
        } else {
            Chg_IndividualLEDState(
                LED_STATE_BRIGHTNESS,
                LED_STATE_BRIGHTNESS,
                LED_STATE_BRIGHTNESS,
                LED_STATE_BRIGHTNESS_RED,
                LED_STATE_BRIGHTNESS,
                LED_STATE_BRIGHTNESS,
                LED_STATE_BRIGHTNESS);
        }
    } else {
        Chg_AllLEDState(LED_STATE_BRIGHTNESS);
    }
}

void Ctrl_LEDforPowerStandby(uint32_t i_mode)
{
    switch(i_mode){
    case MODE_PWR_NORMAL:
        Chg_IndicateMode(MODE_IND_NORMAL);
        Ctrl_PowerLED(LED_ON_GREEN);
        break;
    case MODE_PWR_STANDBY:
        Chg_IndicateMode(MODE_IND_STANDBY);
        Ctrl_PowerLED(LED_ON_RED);
        break;
    case MODE_PWR_SHUTDOWN:
        Chg_IndicateMode(MODE_IND_STANDBY);
        Ctrl_PowerLED(LED_OFF);
        break;
    default: break;
    }
}

// ------------------------------------------------
#define ATTACK_TIME     0.001f  // 0.200f ---> 0.001f (cut&try)
#define RELEASE_TIME    0.500f  // 0.200f ---> 0.500f (cut&try)
#define DSP_BLOCK_FREQ  12      // 48k(SamplingFreq)/384(DspBlockSize)125 ---> 12(Cycle update LED per 1min)
float g_attack_time  = 0.0f;
float g_release_time = 0.0f;

void Initialize_SmoothingParameter(void)
{
    g_attack_time   = TimeToForgetFactor(ATTACK_TIME, DSP_BLOCK_FREQ);
    g_release_time  = TimeToForgetFactor(RELEASE_TIME, DSP_BLOCK_FREQ);
}

float TimeToForgetFactor(const float i_time, const float i_freq)
{
    float forget_factor = 1.0f - i_time/(i_time + (1.0f/(i_freq)));
    return forget_factor;
}

void Set_TargetLevel(uint32_t i_ledno, const float i_target)
{
	if (!Is_LEDNumberValid(i_ledno)) {
		return;
	}
    g_st_LedState[i_ledno].f_level_target = i_target;
}

void ExecuteSmoothing(uint32_t i_ledno)
{
    int32_t diff, next;
    int32_t target, current;

	if (!Is_LEDNumberValid(i_ledno)) {
		return;
	}
    current = g_st_LedState[i_ledno].level_current;
    target  = g_st_LedState[i_ledno].level_target;
    diff = target - current;
	if (diff >= 0) {
        // through
        next = target;
	} else {
		current -= 5;   // cut&try
        if(current < 0) {
            current = 0;
        }
        next = current;
	}
	g_st_LedState[i_ledno].level_current = next;
}

void Chg_LEDBrightness(uint32_t i_ledno, float i_level)
{
    uint32_t level_tmp_int;

	g_st_LedState[i_ledno].i_level = i_level;

	if (i_level < -50.0f){
		g_st_LedState[i_ledno].level_target = 0x00;
	}
	else if (i_level >= 0.0f){
		g_st_LedState[i_ledno].level_target = 0xff;
	}
	else {
        //level_tmp_int = (uint32_t)(powf(10.0f, i_level*0.05f)*255);
        //level_tmp_int = (uint32_t)(powf(10.0f, ((i_level+30)*1.5f)*0.033f)*255);
        level_tmp_int = (uint32_t)(powf(10.0f, ((i_level+20)*1.5f)*0.033f)*255);

        if(level_tmp_int >= 255)
        	level_tmp_int = 255;
        g_st_LedState[i_ledno].level_target = level_tmp_int;
	}
}

void Update_FrontLEDStatus(void)
{
	uint32_t i;
    uint32_t pos;
    uint32_t mode = Get_IndicateMode();
    uint32_t cfg_tracking = Get_CfgTracking();
    uint32_t cfg_leveldsp = Get_CfgLevelIndicate();

    switch(mode) {
    case MODE_IND_NORMAL:
        if(Is_MuteModeChange()) {
            Ctrl_FrontLEDforMute(MODE_IND_NORMAL);
        }
        // Send brightness value
        if(cfg_tracking == CFG_TRACKING_OFF){
            // Level indicate off -> not change brightness
            if(cfg_leveldsp == CFG_LEVEL_INDICATE_OFF){
                for (i = 0; i < MAX_LED_ID_NUMBER; i++) {
                    g_st_LedState[LED_NUM_0+i].level_current = 0;
                }
            } else {
                // Level indicate on -> change brightness
                for (i = 0; i < MAX_LED_ID_NUMBER; i++) {
                    if(i != LED_NUM_3){
                        g_st_LedState[LED_NUM_0+i].level_current = 0;
                    } else {
                        ExecuteSmoothing(LED_NUM_3);
                    }
                }
            }
            for (i = 0; i < MAX_LED_ID_NUMBER; i++) {
		        i2c_write_w_reg(
                	I2C_CHIP_ADDRESS_FRONT_PANEL,
                	REG_BRIGHTNESS_CTRL_LED0 + 2*i,
                	g_st_LedState[LED_NUM_0+i].level_current
            	);
            }
        } else {
            // tracking on
            for (i = 0; i < MAX_LED_ID_NUMBER; i++) {
		        if (g_st_LedState[LED_NUM_0+i].i_level == BF_LED_TURNOFF){
				g_st_LedState[LED_NUM_0+i].i_level = 0;
				g_st_LedState[LED_NUM_0+i].level_current = 0;
			  	i2c_write_w_reg(
                    			I2C_CHIP_ADDRESS_FRONT_PANEL,
                    			REG_BRIGHTNESS_CTRL_LED0 + 2*i,
                    			g_st_LedState[LED_NUM_0+i].level_current
                		);
		        } else {
                    ExecuteSmoothing(i);
		        }
	            if (g_st_LedState[LED_NUM_0+i].level_current != 0){
                    if(cfg_leveldsp == CFG_LEVEL_INDICATE_OFF){
                        g_st_LedState[LED_NUM_0+i].level_current = 255;
                    }
	                i2c_write_w_reg(
                	   	I2C_CHIP_ADDRESS_FRONT_PANEL,
                	   	REG_BRIGHTNESS_CTRL_LED0 + 2*i,
                	   	g_st_LedState[LED_NUM_0+i].level_current
                   	);
                }
            }
        }
        break;

    case MODE_IND_VOLUME:
        if(Is_MuteModeChange()) {
            Ctrl_FrontLEDforMute(MODE_IND_VOLUME);
        }
        // Set volume position
        pos = Get_VolumePosition();
        if(pos == POS_VOL_0) {
            for(i = 0; i < MAX_LED_ID_NUMBER; i++) {
                g_st_LedState[i].volume_disp = 0;
            }
        } else {
            for(i = 0; i < pos; i++) {
                g_st_LedState[i].volume_disp = 0xff;
            }
            for(i = pos; i < MAX_LED_ID_NUMBER; i++) {
                g_st_LedState[i].volume_disp = 0;
            }
        }
        // Send brightness value
	    for (i = 0; i < MAX_LED_ID_NUMBER; i++) {
		    i2c_write_w_reg(
                I2C_CHIP_ADDRESS_FRONT_PANEL,
                REG_BRIGHTNESS_CTRL_LED0 + 2*i,
                g_st_LedState[LED_NUM_0+i].volume_disp
            );
	    }
        break;

    case MODE_IND_MUTE:
        if(Is_MuteModeChange()) {
            if(Get_CfgMuteLedPattern() == CFG_MUTE_INDICATE_ALL) {
	            for (i = 0; i < MAX_LED_ID_NUMBER; i++) {
		            i2c_write_w_reg(
                        I2C_CHIP_ADDRESS_FRONT_PANEL,
                        REG_BRIGHTNESS_CTRL_LED0 + 2*i,
                        BRIGHTNESS_AT_MUTE
                    );
	            }
            } else {
	            i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_BRIGHTNESS_CTRL_LED0, 0);
	            i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_BRIGHTNESS_CTRL_LED1, 0);
	            i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_BRIGHTNESS_CTRL_LED2, 0);
	            i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_BRIGHTNESS_CTRL_LED3, BRIGHTNESS_AT_MUTE);
	            i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_BRIGHTNESS_CTRL_LED4, 0);
	            i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_BRIGHTNESS_CTRL_LED5, 0);
	            i2c_write_w_reg(I2C_CHIP_ADDRESS_FRONT_PANEL, REG_BRIGHTNESS_CTRL_LED6, 0);
            }
            Ctrl_FrontLEDforMute(MODE_IND_MUTE);
        }
        break;

    case MODE_IND_STANDBY:
        for (i = 0; i < MAX_LED_ID_NUMBER; i++) {
	        i2c_write_w_reg(
                I2C_CHIP_ADDRESS_FRONT_PANEL,
                REG_BRIGHTNESS_CTRL_LED0 + 2*i,
                0
            );
        }
        break;

    default : return;
    }
}

uint32_t Is_ShutdownMode(void)	//Second PCBからI2C制御への移行を検討すること
{
    return(g_ShutdownMode); // 0:normal, 1:shutdown		//POWERSW
}

uint32_t Read_PowerSwitchStatus(void)
{
	//printf ("Read_PowerSwitchStatus  g_st_inputI2C[INPUT0_POWER_SWITCH].curren %x \n", g_st_inputI2C[INPUT0_POWER_SWITCH].current);
    return g_st_inputI2C[INPUT0_POWER_SWITCH].current; // 0:off, 1:on	//POWERSW
}

int32_t Get_ChangedPowerSwitch(void)
{	if(g_st_inputI2C[INPUT0_POWER_SWITCH].flag){
		g_st_inputI2C[INPUT0_POWER_SWITCH].flag = 0;
		return 0;
	}
    return -1;
}

int32_t PowerSW_KeyScan(void)	//PowerSwのみ　状態監視
{
	static uint32_t current = 0;
	static uint32_t status = 0;

	current <<= 1;
	current |= g_st_inputI2C[INPUT0_POWER_SWITCH].current = Get_PowerSwitch();
	// ON 1　OFF O
	if(status == 0){				/* OFF*/

		/* off -> on 		*/
		if((current & 0x0003) == 0x0003){				/* 2回チェック				*/
			status = 1;
            if (g_IndicateMode == MODE_IND_STANDBY){
			    LibCTIMER_setTimer( "STANDBY", 1000);
            }else{
                LibCTIMER_setTimer( "STANDBY", 2000);
            }
			LibCTIMER_setTimer( "SHUT", 10000);
		}

	}
	else if ((current & 0x0003) == 0x0000){				/* ON -> OFF(連続取得2回)	*/
			status = 0;
	}
	else{

		if(LibCTIMER_getTimeout("STANDBY")){				/* スタンバイタイマー　タイムアウト	*/
            g_st_inputI2C[INPUT0_POWER_SWITCH].flag = 1;	//スイッチの状態が変化した
		}
		else{
            g_st_inputI2C[INPUT0_POWER_SWITCH].flag = 0;	//変化なし
		};

		if(LibCTIMER_getTimeout("SHUT")){				/* シャッドダウンタイマー　タイムアウト	*/
			g_ShutdownMode = 1;
		}

	}
	// flag 1 遷移した　0 変化なし　　　　flag 1 なら　return 0  0ならreturn -1
    return(Get_ChangedPowerSwitch());
}
