#pragma once
/* Macros Chip Address */
#define I2C_CHIP_ADDRESS_DAC			0x12
#define I2C_CHIP_ADDRESS_SIDE_PANEL		0x3a	//PCM8574
#define I2C_CHIP_ADDRESS_USB_HUB		0x44
#define I2C_CHIP_ADDRESS_RTC            0x68
#define I2C_CHIP_ADDRESS_FRONT_PANEL	0x6f

#define I2C_CHIP_ADDRESS_SIDE_PANEL_2	0x3b	//PCM8574
#define I2C_CHIP_ADDRESS_MB				0x3f	//PCM8574

/* Macros for DAC */
#define REG_CONTROL01		0x00
#define REG_CONTROL02		0x01
#define REG_CONTROL03		0x02
#define REG_L1CH_ATT		0x03
#define REG_R1CH_ATT		0x04
#define REG_CONTROL04		0x05
#define REG_DSD1			0x06
#define REG_CONTROL05		0x07
#define REG_SOUND_CONTROL	0x08
#define REG_DSD2			0x09
#define REG_CONTROL06		0x0a
#define REG_CONTROL07		0x0b
#define REG_CONTROL08		0x0c
#define REG_CONTROL09		0x0d
#define REG_CONTROL10		0x0e
#define REG_L2CH_ATT		0x0f
#define REG_R2CH_ATT		0x10
#define REG_L3CH_ATT		0x11
#define REG_R3CH_ATT		0x12
#define REG_L4CH_ATT		0x13
#define REG_R4CH_ATT		0x14
#define MASK_SMUTE_ENABLE   0x01

/* Macros for USB HUB */
#define REG_ROM_SIGNATURE			0x00
#define REG_VENDOR_ID_LSB			0x01
#define REG_VENDOR_ID_MSB			0x02
#define REG_PRODUCT_ID_LSB			0x03
#define REG_PRODUCT_ID_MSB			0x04
#define REG_DEVICE_CONFIG1			0x05
#define REG_BATTERY_CHARGE_SUPPORT	0x06
#define REG_DEVICE_REMOVE_CONFIG	0x07
#define REG_PORT_USED_CONFIG		0x08
#define REG_DEVICE_CONFIG2			0x0a
#define REG_USB2_PORT_POLARITY_CTRL	0x0b
#define REG_UUID					0x10
#define REG_LANG_ID_LSB				0x20
#define REG_LANG_ID_MSB				0x21
#define REG_SERIAL_NUMBER_STR_LEN	0x22
#define REG_MANUFACTURE_STR_LEN		0x23
#define REG_PRODUCT_STR_LEN			0x24
#define REG_SERIAL_NUMBER_STR_BYTE	0x30
#define REG_MANUFACTURE_STR_BYTE	0x50
#define REG_PRODUCT_STR_BYTE		0x90
#define REG_ADD_FEATURE_CONFIG		0xf0
#define REG_DEVICE_STATUS_COMMAND	0xf8
#define MAX_SERIAL_NUMBER_LENGTH    0x18
#define MAX_MANUFACTURE_STR_LENGTH  30
#define MAX_PRODUCT_STRINGS_LENGTH  12

/* Macros for RTC */
#define REG_RTC_SECONDS             0x00
#define REG_RTC_MINUTES             0x01
#define REG_RTC_HOURS               0x02
#define REG_RTC_DAY                 0x03
#define REG_RTC_DATE                0x04
#define REG_RTC_MONTH_CENTURTY      0x05
#define REG_RTC_YEAR                0x06
#define REG_RTC_ALM1_SECOUNDS       0x07
#define REG_RTC_ALM1_MINUTES        0x08
#define REG_RTC_ALM1_HOURS          0x09
#define REG_RTC_ALM1_DAY_DATE       0x0a
#define REG_RTC_ALM2_MINUTES        0x0b
#define REG_RTC_ALM2_HOURS          0x0c
#define REG_RTC_ALM2_DAY_DATE       0x0d
#define REG_RTC_CONTROL             0x0e
#define REG_RTC_STATUS              0x0f
#define REG_RTC_TRICKLE_CHARGER     0x10

/* Macros for Side Panel */
#define ALL_LED_OFF				0xff
#define MASK_POWER_LED_BLUE		0x10
#define MASK_POWER_LED_RED		0x30
#define MASK_POWER_LED_ORG		0x30
#define MASK_POWER_LED_OFF		0x30
#define MASK_DOUBLE_TALK		0x40	//これ間違いだと思います。
#define MASK_BLUETOOTH_LED		0x80
#define MASK_BLUETOOTH_SWITCH	0x01
#define MASK_MUTE_SWITCH		0x02
#define MASK_VOLUMEUP_SWITCH	0x04
#define MASK_VOLUMEDOWN_SWITCH	0x08
#define MAX_COUNT_SW_STATE		1

#define	MASK_POWER_SWITCH		0x0100
#define MASK_POWER_BLUE_LED		0x42

enum {
	LED_OFF,
	LED_ON_GREEN,
	LED_ON_BLUE = 1,
	LED_ON_RED,
	LED_ON_ORANGE
};

enum {
	SW_BT,
	SW_MUTE,
	SW_V_UP,
	SW_V_DN,
	SW_POWER,			// For Second PCB
	MAX_SW_NUMBER
};

typedef struct {
	uint32_t current;
	uint32_t count;
	uint32_t flag;
} st_SwitchState;

//For PowerSW
typedef struct {
	uint32_t prev;
	uint32_t current;
    	//uint32_t temp[2];	//non used
    	//uint32_t index;	//non used
	uint32_t flag;
	uint32_t count;
} st_input;


/* Macros for Front Panel */
#define REG_MODE1						0x00
#define REG_MODE2						0x01
#define REG_BRIGHTNESS_CTRL_LED0		0x02
#define REG_BRIGHTNESS_CTRL_LED1		0x04
#define REG_BRIGHTNESS_CTRL_LED2		0x06
#define REG_BRIGHTNESS_CTRL_LED3		0x08
#define REG_BRIGHTNESS_CTRL_LED4		0x0a
#define REG_BRIGHTNESS_CTRL_LED5		0x0c
#define REG_BRIGHTNESS_CTRL_LED6		0x0e
#define REG_GROUP_DUTY_CYCLE_CTRL		0x12
#define REG_GROUP_FREQ_CTRL				0x13
#define REG_LED0_1_OUTPUT_CTRL			0x14
#define REG_LED2_3_OUTPUT_CTRL			0x15
#define REG_LED4_5_OUTPUT_CTRL			0x16
#define REG_LED6_OUTPUT_CTRL			0x17
#define REG_SUB_ADDRESS1                0x18
#define REG_SUB_ADDRESS2                0x19
#define REG_SUB_ADDRESS3                0x1a
#define REG_ALL_CALL_ADDRESS            0x1b
#define BRIGHTNESS_AT_MUTE              0x64    // (100/255)
#define BF_LED_TURNOFF					-5000.0f

enum {
    CFG_TRACKING_OFF,
    CFG_TRACKING_ON
};

enum {
    CFG_LEVEL_INDICATE_OFF,
    CFG_LEVEL_INDICATE_ON
};

enum {
    CFG_MUTE_INDICATE_CENTER,
    CFG_MUTE_INDICATE_ALL
};

enum {
	LED_NUM_0,
	LED_NUM_1,
	LED_NUM_2,
	LED_NUM_3,
	LED_NUM_4,
	LED_NUM_5,
	LED_NUM_6,
	MAX_LED_ID_NUMBER
};

enum {
	LED_BANK_0,
	LED_BANK_1,
	LED_BANK_2,
	LED_BANK_3,
	MAX_LED_BANK_NUMBER
};

enum {
	LED_STATE_OFF,
	LED_STATE_FULLY_ON,
	LED_STATE_BRIGHTNESS,
	LED_STATE_BLINK,
	LED_STATE_BRIGHTNESS_RED = 0x06
};

enum {
	LED_COLOR_GRN,
	LED_COLOR_RED,
	LED_COLOR_ORG,
};

enum {
    MODE_IND_NORMAL,
    MODE_IND_VOLUME,
    MODE_IND_MUTE,
    MODE_IND_STANDBY,
    MODE_IND_NONE
};

enum {
    MODE_PWR_NORMAL,
    MODE_PWR_STANDBY,
    MODE_PWR_SHUTDOWN,
    MODE_PWR_NONE
};

enum {
    POS_VOL_0,  // -Infinity
    POS_VOL_1,  // -30dB
    POS_VOL_2,  // -24dB
    POS_VOL_3,  // -18dB
    POS_VOL_4,  // -12dB
    POS_VOL_5,  // -6dB
    POS_VOL_6,  // -3dB
    POS_VOL_7,  // 0dB
    POS_VOL_MAX
};

typedef struct {
	uint32_t state;
	uint32_t flag;
	uint32_t level_current;
	uint32_t level_target;
	//uint32_t level_mute;	//non used
	uint32_t volume_disp;
	float f_level_current;
	float f_level_target;
	float i_level;
} st_LedState;

/* Init */
void Initialize_I2C(void);

/* Functions for DAC */
void Initialize_DAC(void);
void Reset_OSC(void);
void Ctrl_Smute(uint32_t i_ctrl);

/* Functions for USB HUB */
void Initialize_USBHub(void);

/* Functions for RTC */
void Initialize_RTC(
    int32_t i_year,
    int32_t i_month,
    int32_t i_date,
    int32_t i_day,
    int32_t i_hour,
    int32_t i_min,
    int32_t i_sec
);
void Set_TrickleChargerEnable(void);
uint32_t Get_Year(void);
uint32_t Get_Month(void);
uint32_t Get_Date(void);
uint32_t Get_Day(void);
uint32_t Get_Hour(void);
uint32_t Get_Minute(void);

/* Functions for Side Panel */
void Initialize_SidePanel(void);
void Initialize_SwitchStatus(void);
void Ctrl_PowerLED(uint32_t i_state);
void Ctrl_BluetoothLED(uint32_t i_state);
void Ctrl_USBConnectionLED(uint32_t i_state);
uint32_t Get_BluetoothSwitch(void);
uint32_t Get_VolumeUpSwitch(void);
uint32_t Get_VolumeDownSwitch(void);
uint32_t Get_MuteSwitch(void);
void Set_i2cMuteStatus(uint32_t i_status);
uint32_t Get_i2cMuteStatus(void);
int32_t Exec_KeyScan(void);
int32_t KeyScan(void);
int32_t Get_i2cChangedSwitch(void);
uint32_t Read_BluetoothSwitch(void);
uint32_t Read_VolumeUpSwitch(void);
uint32_t Read_VolumeDownSwitch(void);
uint32_t Read_MuteSwitch(void);
uint32_t Debug_GetCount(void);
uint32_t Read_PowerSwitch(void);


/* Functions for Front Panel */
void Initialize_FrontPanel(void);
void Initialize_LedConfig(uint32_t i_cfg1, uint32_t i_cfg2, uint32_t i_cfg3);
uint32_t Get_CfgTracking(void);
uint32_t Get_CfgLevelIndicate(void);
uint32_t Get_CfgMuteLedPattern(void);

void Chg_IndividualLEDState(
    uint32_t i_led0st,
    uint32_t i_led1st,
    uint32_t i_led2st,
    uint32_t i_led3st,
    uint32_t i_led4st,
    uint32_t i_led5st,
    uint32_t i_led6st
);
void Chg_AllLEDState(uint32_t i_ledst);
void Set_FrontLEDStatus(uint32_t i_ledno, uint32_t i_state, float i_level);
void Ctrl_FrontLEDforMute(uint32_t i_mode);
void Ctrl_LEDforPowerStandby(uint32_t i_mode);

void Chg_LEDBrightness(uint32_t i_ledno, float i_level);
uint32_t Is_MuteModeChange(void);
void Chg_IndicateMode(uint32_t i_mode);
uint32_t Get_IndicateMode(void);
void Chg_VolumePosition(uint32_t i_pos);
uint32_t Get_VolumePosition(void);
void Calc_VolumePosition(int32_t i_gain);

void Initialize_SmoothingParameter(void);
float TimeToForgetFactor(const float i_time, const float i_freq);
void Set_TargetLevel(uint32_t i_ledno, const float i_target);
void ExecuteSmoothing(uint32_t i_ledno);
void Update_FrontLEDStatus(void);

float Calc_SteeringAngle(float focus_y,float focus_x);
uint32_t Get_Bf_FrontLED_Num(float radian_value);

uint32_t Is_VolMode(void);
uint32_t Is_NormalMode(void);


/* Macros for MB */
#define POWER_FROM_PC 			0
#define	POWER_FROM_CF1 			1
#define	AUDIO_OUTPUT_ENABLE		1
#define	AUDIO_OUTPUT_DISENABLE	0

void Initialize_MB(void);
void Ctrl_USB_POWER(uint32_t);
void Ctrl_AUDIO_OUTPUT_ENABLE(uint32_t);
